#!/usr/bin/perl

use	strict		;
use	File::Find	;
use	File::Basename	;

my	$dirtag		= "%attr(-,root,root) %dir " ;
my	$filetag	= "%attr(-,root,root) "	     ;
my	$libpfx		= 'rekall'	;
my	$cfgsfx		= '_kde'	;
my	$tgz		= 0 		;

A : while ($ARGV[0] =~ /^--/)
{
	if ($ARGV[0] eq "--tgz")
	{
		$dirtag	= ""	;
		$filetag= ""	;
		$tgz	= 1	;
		shift		;
		next	A	;
	}

	if ($ARGV[0] =~ /^--libprefix=(.*)$/)
	{
		$libpfx	= $1	;
		shift		;
		next	A	;
	}

	if ($ARGV[0] =~ /^--cfgsuffix=(.*)$/)
	{
		$cfgsfx	= "_$1"	;
		shift		;
		next	A	;
	}

	shift	;
}


my	$appname	= $ARGV[0] ;
my	$builddir	= $ARGV[1] ;
my	$rpmdir		= $ARGV[2] ;

if (open CONFIG, "scons/rkConfig${cfgsfx}.py")
{
	my	$line	;
	while  ($line = <CONFIG>)
	{
		if ($line =~ /libPrefix\s*=\s*'(.*)'/)
		{
			$libpfx	= $1	;
		}

	}
	close	CONFIG	;
}

print	STDERR		"libpfx : $libpfx\n"  ;
print	STDERR		"appname: $appname\n" ;


my	@filelist	;
my	@dirlist	;

sub	wanted
{
	my	$file	= $File::Find::name ;
	my	$path	= $file ;

	$path	=~ s/$builddir// ;

	if ( -f $file)
	{
		push	@filelist, $path ;
		return	;
	}
	if ( -l $file)
	{
		push	@filelist, $path ;
		return	;
	}
	if ( -d $file )
	{
		push	@dirlist,  $file ;
		return	;
	}
	print "no [$file]\n" ;
}

sub	listToDirs
{
	my	$name	 ;
	my	@result	 ;
	my	%alldirs ;

	foreach my $dir (@dirlist)
	{
		$alldirs{$dir} = 0 ;
	}

	NAME : while ($name = shift)
	{
		my	$dest	= $builddir . $name ;

		DIR : foreach my $dir (@dirlist)
		{
			if ($dest =~ /^$dir/)
			{
#				print	"[$dest][$dir]\n" ;
				$alldirs{$dir} = 1 ;
			}
		}
	}

	foreach my $dir (keys %alldirs)
	{
		if ($alldirs{$dir})
		{
			$dir	=~ s/$builddir// ;
			push	@result, $dir ;
		}
	}

	return	sort { $a cmp $b } @result ;
}

sub	makeList
{
	my	$in	= shift	;
	my	@out	;

	if (! $tgz)
	{
		foreach my $dir  (&listToDirs (@$in))
		{
			push	@out, "$dirtag$dir"
				if "$dir/" ne $builddir	;
		}
	}

	foreach my $file (@$in)
	{
		push	@out, "$filetag$file"
			if -f $builddir . $file && ! -l $builddir . $file ;
	}

	foreach my $file (@$in)
	{
		push	@out, "$filetag$file"
			if -l $builddir . $file ;
	}

	return	@out	;
}

sub	makeListFile
{
	my	$file	= shift	;
	my	$in	= shift	;
	my	@list	= &makeList ($in) ;

#	if (scalar(@list) > 0)
#	{
		open	LIST, ">$file" ;
		print	LIST join "\n", @list ;
		print	LIST "\n" ;
		close	LIST
#	}
}

find (\&wanted, $builddir) ;

my	@orphans  ;

my	@mappings =
(
	# Documentation (ie., the manual)
	#
	[
		'docs',
		[],
		'/doc/',
		'/manual/',
		'/rekallManual'
	],
	# Driver library used for rekall and for standalone
	# drivers; also the rkdcop program
	#
	[
		'drivers',
		[],
		"/lib${libpfx}_common\.so",
		"/lib${libpfx}_common\.la",
		"/lib${libpfx}_rkdcop\.so",
		"/lib${libpfx}_rkdcop\.la",
		'/rekall_dummy\.desktop',
		'/rkdcop'
	],
	# Python scripting common libraries and stuff
	#
	[
		'python-common',
		[],
		'/script/py/'
	],
	# Python scripting full version
	#
	[
		'python',
		[],
		"lib${libpfx}_script_py.*\.so",
		"lib${libpfx}_script_py.*\.la",
		'rekall_script_py.desktop',
		'/stock/component/py/'
	],
	# Python scripting runtime version
	#
	[
		'python-runtime',
		[],
		"lib${libpfx}rt_script_py.*\.so",
		"lib${libpfx}rt_script_py.*\.la",
		'rekallrt_script_py.desktop'
	],
	# KJS scripting common libraries and stuff
	#
	[
		'kjs-common',
		[],
		'/script/kjs/'
	],
	# KJS scripting full version
	#
	[
		'kjs',
		[],
		"lib${libpfx}_script_kjs.*\.so",
		"lib${libpfx}_script_kjs.*\.la",
		'rekall_script_kjs.*\.desktop',
		'/stock/component/kjs/'
	],
	# KJS scripting runtime version
	#
	[
		'kjs-runtime',
		[],
		"lib${libpfx}rt_script_kjs.*\.so",
		"lib${libpfx}rt_script_kjs.*\.la",
		'rekallrt_script_kjs.*\.desktop'
	],
	# ODBC driver. Note that the drivers preceed the libraries in
	# order to pick up the driver desktop files.
	#
	[
		'odbc',
		[],
		"/lib${libpfx}_driver_odbc\.so",
		"/lib${libpfx}_driver_odbc\.la",
		'/rekall_driver_odbc.desktop'
	],
	# IODBC driver.
	#
	[
		'iodbc',
		[],
		"/lib${libpfx}_driver_iodbc\.so",
		"/lib${libpfx}_driver_iodbc\.la",
		'/rekall_driver_iodbc.desktop'
	],
	# MySQL driver
	#
	[
		'mysql',
		[],
		"/lib${libpfx}_driver_mysql\.so",
		"/lib${libpfx}_driver_mysql\.la",
		'/rekall_driver_mysql.desktop'
	],
	# PgSQL driver
	#
	[
		'pgsql',
		[],
		"/lib${libpfx}_driver_pgsql\.so",
		"/lib${libpfx}_driver_pgsql\.la",
		'/rekall_driver_pgsql.desktop'
	],
	# XBase/XBSQL driver
	#
	[
		'xbase',
		[],
		"/lib${libpfx}_driver_xbase\.so",
		"/lib${libpfx}_driver_xbase\.la",
		'/rekall_driver_xbase.desktop',
		"/lib${libpfx}_support_xbase\.la",
		"/lib${libpfx}_support_xbase\.so",
		"/lib${libpfx}_support_xbsql\.la",
		"/lib${libpfx}_support_xbsql\.so"
	],
	# SQLite3 driver
	#
	[
		'sqlite3',
		[],
		"/lib${libpfx}_driver_sqlite3\.so",
		"/lib${libpfx}_driver_sqlite3\.la",
		'/rekall_driver_sqlite3.desktop'
	],
	# IBM DBCLI driver
	#
	[
		'db2cli',
		[],
		"/lib${libpfx}_driver_db2cli\.so",
		"/lib${libpfx}_driver_db2cli\.la",
		'/rekall_driver_db2cli.desktop'
	],
	# DBTCP driver
	#
	[
		'dbtcp',
		[],
		"/lib${libpfx}_driver_dbtcp\.so",
		"/lib${libpfx}_driver_dbtcp\.la",
		"/lib${libpfx}_support_dbtcp\.la",
		"/lib${libpfx}_support_dbtcp\.so",
		'/rekall_driver_dbtcp.desktop'
	],
	# UniSQL driver
	#
	[
		'unisql',
		[],
		"/lib${libpfx}_driver_unisql\.so",
		"/lib${libpfx}_driver_unisql\.la",
		'/rekall_driver_unisql.desktop'
	],
	#  Extra stuff, mainly macros
	#
	[	'extra',
		[],
		"/lib${libpfx}_plugin_extra\.so",
		"/lib${libpfx}_plugin_extra\.la",
		'/rekall_plugin_extra\.desktop'
	],
	#  Runtime extra stuff, mainly macros
	#
	[	'runtime-extra',
		[],
		"/lib${libpfx}rt_plugin_extra\.so",
		"/lib${libpfx}rt_plugin_extra\.la",
		'/rekallrt_plugin_extra\.desktop'
	],
	# KDE Plugins
	#
	[
		'kde',
		[],
		"/lib${libpfx}_plugin_kde\.so",
		"/lib${libpfx}_plugin_kde\.la",
		'/rekall_plugin_kde\.desktop',
		'/kdeparts\.lst'
	],
	# Runtime KDE Plugins
	#
	[
		'runtime-kde',
		[],
		"/lib${libpfx}rt_plugin_kde\.so",
		"/lib${libpfx}rt_plugin_kde\.la",
		'/rekallrt_plugin_kde\.desktop',
		'/kdeparts\.lst'
	],
	# Libraries for the full version
	#
	[
		'libs',
		[],
		'/services/rekall_',
		"/lib${libpfx}_script.*\.so",
		"/lib${libpfx}_script.*\.la",
		"/lib${libpfx}\.so",
		"/lib${libpfx}\.la",
		"/lib${libpfx}_app\.so",
		"/lib${libpfx}_app\.la",
		"/lib${libpfx}_editor\.so",
		"/lib${libpfx}_editor\.la",
		"/lib${libpfx}_wizard\.so",
		"/lib${libpfx}_wizard\.la",
		"/lib${libpfx}_.*view\.so",
		"/lib${libpfx}_.*view\.la",
		'/rekall_.*\.desktop',
		"${appname}Help",
		'/wizards',
		'/.*\.hilight'
	],
	# Libraries for the runtime version
	#
	[
		'runtime-libs',
		[],
		'/services/rekallrt_',
		"/lib${libpfx}rt_script.*\.so",
		"/lib${libpfx}rt_script.*\.la",
		"/lib${libpfx}rt\.so",
		"/lib${libpfx}rt\.la",
		"/lib${libpfx}rt_app\.so",
		"/lib${libpfx}rt_app\.la",
		"/lib${libpfx}rt_.*view\.so",
		"/lib${libpfx}rt_.*view\.la",
		"/lib${libpfx}rt_wizard.*\.so",
		"/lib${libpfx}rt_wizard.*\.la",
		'/rekallrt_.*\.desktop',
		'/.*\.hilight'
	],
	# Stuff common to both full and runtime versions
	#
	[
		'common',
		[],
		"/lib${libpfx}_python.*\.so",
		"/lib${libpfx}_python.*\.la",
		"/lib${libpfx}_kde\.so",
		"/lib${libpfx}_kde\.la",
		"/lib${libpfx}_el32\.so",
		"/lib${libpfx}_el32\.la",
		'/dict',
		'/help/',
		'/icons/',
		'/databases/',
		'/pics/',
		'/rekallui.',
		'/highlight/',
		'/script/',
		'/keymap',
		'/rekall.png',
		'/rekall_lite.png',
		'/LICENSE',
		'/rekall.mo'
	],
	# Full version executables
	#
	[
		'full',
		[],
		"${appname}\$",
		'/rekall.desktop'
	],
	# Runtime version executables
	#
	[
		'runtime',
		[],
		"${appname}rt\$",
		'/rekallrt.desktop'
	],
	# Development headers and library .la files
	#
	[
		'devel',
		[],
		'/include/'
	],
	# Tests
	#
	[
		'test',
		[],
		'/test/',
		'/rkt_.*',
		'/alltests.sh',
		"/lib${libpfx}_rkt_.*\.so",
		"/lib${libpfx}_rkt_.*\.la",
		"/lib${libpfx}rt_rkt_.*\.so",
		"/lib${libpfx}rt_rkt_.*\.la",
		"/lib${libpfx}_driver_test\.so",
		"/lib${libpfx}_driver_test\.la"
	],
)	;

FILE : foreach my $file (@filelist)
{
	foreach my $map (@mappings)
	{
		for (my $idx = 2 ; $idx < scalar(@$map) ; $idx += 1)
		{
			if ($file =~ /$map->[$idx]/)
			{
				push	@{ $map->[1] }, $file ;
				next	FILE ;
			}
		}
	}


	push	@orphans, $file ;
}

if (scalar(@orphans) > 0)
{
	print	"Orphans:\n" ;

	foreach my $file (@orphans)
	{
		print	"\t", $file ;

		if ($file =~ /.sconsign$/)
		{
			unlink	$file		;
			print	" (removed)"	;
		}

		print	"\n" ;
	}
}

LIST : foreach my $file (@mappings)
{
	&makeListFile
	(	"$rpmdir/$appname-$file->[0]",
		$file->[1]
	)	;
}
