/***************************************************************************
    file	         : kb_libloader_qt3.cpp
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
			   (C) 2000,2001,2002,2003 by theKompany.com
			   (C) 2001,2002,2003 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


/*  This file contains the library loader code for Win32/QT builds of	*/
/*  Rekall. By preference it would be in libkbase_kde, but in order	*/
/*  that libkbase_common does not have any depend on any other Rekall	*/
/*  libraries (so that the drivers can be used outside of Rekall) we	*/
/*  have to put it here.						*/

#if		__KB_TKC

#include	<qglobal.h>

#include	"kb_classes.h"
#include	"kb_libloader_win.h"

#define		DLERROR()	"Unknown error"

#if		__KB_DLLISSO
#define		LIBEXTN		".so"
#else
#define		LIBEXTN		""
#endif

#define		__DEBUG_LIBLOADER	(0)
#if		__DEBUG_LIBLOADER
#include	"kb_dprintf.h"
#endif

void	*KBLibrary::factory ()
{
	typedef	void	*(*IFPROC)()	;

	DPRINTF
	((	"KBLibrary::factory: resolve [%s]\n",
		m_name.latin1()
	))	;


	IFPROC	ifproc	= (IFPROC)m_library->resolve("init_" + m_name) ;
	if (ifproc == 0)
	{
		DPRINTF
		((	"KBLibrary::factory: no interface %s [%s]\n",
			m_name.latin1(),
			DLERROR()
		))	;
		return	0 ;
	}

	return	(*ifproc)() ;

}

/*  KBLibLoader								*/
/*  self	: Instantiate the library loader			*/
/*  (returns)	: KBLibLoader *	: Loader object				*/

KBLibLoader
	*KBLibLoader::self ()
{
	static	KBLibLoader	*loader	;
	if (loader == 0) loader = new KBLibLoader ;
	return	loader	;
}

/*  KBLibLoader								*/
/*  getLibrary	: Load a library					*/
/*  libname	: const QString & : Library name			*/
/*  (returns)	: KBLibrary *	  : Library object or null on error	*/

KBLibrary
	*KBLibLoader::getLibrary
	(	const QString	&libname
	)
{
	DPRINTF
	((	"KBLibLoader::getLibrary: [%s]\n",
		libname.latin1()
	))	;

	QLibrary *lib	= new QLibrary (libname + LIBEXTN) ;
	if (!lib->load())
	{
		m_error	= DLERROR() ;
		return	0 ;
	}

	DPRINTF
	((	"KBLibLoader::getLibrary: loaded [%s]\n",
		libname.latin1()
	))	;

	return	new KBLibrary (lib, libname) ;
}

/*  KBLibLoader								*/
/*  getGlobalLibrary							*/
/*		: Load a library with global symbol access		*/
/*  libname	: const QString & : Library name			*/
/*  (returns)	: KBLibrary *	  : Library object or null on error	*/


KBLibrary
	*KBLibLoader::getGlobalLibrary
	(	const QString	&libname
	)
{
	DPRINTF
	((	"KBLibLoader::getGlobalLibrary: [%s]\n",
		libname.latin1()
	))	;

	QLibrary *lib	= new QLibrary (libname + LIBEXTN) ;
	if (!lib->load())
	{
		m_error	= DLERROR() ;
		return	0 ;
	}

	return	new KBLibrary (lib, libname) ;
}

QString	KBLibLoader::lastErrorMessage ()
{
	return	m_error	;
}


/*  ------------------------------------------------------------------  */

KBPartFactory::KBPartFactory ()
{
}

KBPartFactory::~KBPartFactory ()
{
}

QObject	*KBPartFactory::create
	(	QObject *		,
		const char *		,
		const char *		,
		const QStringList &
	)
{
	return	0 ;
}

QString KBPartFactory::ident ()
{
	return	"Unspecified plugin identifier" ;
}


KBFactory::KBFactory ()
{
}

KBFactory::~KBFactory ()
{
}

QObject	*KBFactory::create
	(	QObject *		,
		const char *		,
		const char *		,
		const QStringList &
	)
{
	return	0 ;
}

QString	KBFactory::ident ()
{
	return	0 ;
}

#endif	// __KB_TKC
