/***************************************************************************
    file	         : kb_locator_kde.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#if		__KB_KDE

#include	<kglobal.h>
#include	<kstddirs.h>
#include	<kinstance.h>

#include	<qfileinfo.h>

#include	"kb_classes.h"
#define		__DEBUG_LOCATOR		(0)
#if		__DEBUG_LOCATOR
#include	"kb_dprintf.h"
#endif

#include	"tk_messagebox.h"

#define	DATAPATH	"/share/apps/" 		__KB_APPNAME	"/"
#define	DOCPATH		"/share/doc/HTML/en/"	__KB_APPNAME	"/"
static	QString		rootdir	;

/*  locateFile	: Locate a file of specified type			*/
/*  type	: const char *	  : File type				*/
/*  name	: const QString & : File name				*/
/*  (returns)	: QString	  : Path				*/

QString	locateFile
	(	const char	*type,
		const QString	&name
	)
{
	DPRINTF
	((	"locateFile: %s: %s (with %s)\n",
		type,
		name   .latin1(),
		rootdir.latin1()
	))	;

	/* If an explicit root directory has been set then act like the	*/
	/* QT3 versions. The code below covers the cases we need.	*/
	if (!rootdir.isEmpty())
	{
		QString	path	;

		if	(qstrcmp (type, "appdata") == 0) path = rootdir + DATAPATH + name ;
		else if (qstrcmp (type, "doc"    ) == 0) path = rootdir + DOCPATH  + name ;

		if (path != 0)
			if (!QFileInfo(path).exists())
				path	= QString::null ;

		return	path	;
	}

	DPRINTF
	((	"locateFile: instance=%p\n",
		(void *)KGlobal::instance()
	))	;

	QString	res	= locate (type, name) ;

	DPRINTF
	((	"locate: [%s][%s]->[%s]\n",
		type,
		name.latin1(),
		res .latin1()
	))	;

	return	res	;
}

/*  locateDir	: Locate a directory of specified type			*/
/*  type	: const char *	  : Directory type			*/
/*  name	: const QString & : Directory name			*/
/*  (returns)	: QString	  : Path				*/

QString	locateDir
	(	const char	*type,
		const QString	&name
	)
{
	/* If an explicit root directory has been set then act like the	*/
	/* QT3 versions.						*/
	if (!rootdir.isEmpty())
		return	locateFile (type, "") ;

	QString	res	= KGlobal::instance()->dirs()->findResourceDir (type, name) ;

	DPRINTF
	((	"locateDir: [%s][%s]->[%s]\n",
		type,
		name.latin1(),
		res .latin1()
	))	;

	return	res	;
}

/*  setRootDir	: Set different root directory				*/
/*  _rootdir	: const QString & : New root directory name		*/
/*  (returns)	: void		  :					*/

void	setRootDir
	(	const QString	&_rootdir
	)
{
	DPRINTF
	((	"setRootDir(%s)\n",
		_rootdir.latin1()
	))	;

	rootdir	= _rootdir ;

	if (locateDir ("appdata", "services").isNull())
		TKMessageBox::sorry
		(	0,
			"Rekall has set a root directory that does not "
			"appear to have the right contents. Rekall will "
			"probably not run",
			"Root Directory Error"
		)	;
}
#endif		// __KB_KDE
