
#include	"s1sDialogTransfer.h"

#include	<qstring.h>
#include	<qxml.h>
#include	<qwidget.h>
#include	<qlineedit.h>
#include	<qcombobox.h>
#include	<qlistbox.h>
#include	<qcheckbox.h>
#include	<qgroupbox.h>
#include	<qobjectlist.h>

void	S1SDataTransfer::transfer
	(	const QString		&name,
		QString			&value,
		QXmlAttributes		&attributes,
		const QString		&defval,
		bool			load
	)
{
	if (load)
	{
		QString	text	=  attributes.value(name) ;
		value	= text.isNull() ? defval : text	  ;
		return	;
	}

	attributes.append
	(	name,
		QString::null,
		QString::null,
		value
	)	;
}

void	S1SDataTransfer::transfer
	(	const QString		&name,
		int			&value,
		QXmlAttributes		&attributes,
		int			defval,
		bool			load
	)
{
	if (load)
	{
		QString	text	=  attributes.value(name)	;
		value	= text.isNull() ? defval : text.toInt() ;
		return	;
	}

	attributes.append
	(	name,
		QString::null,
		QString::null,
		QString::number (value)
	)	;
}

void	S1SDataTransfer::transfer
	(	const QString		&name,
		bool			&value,
		QXmlAttributes		&attributes,
		bool			defval,
		bool			load
	)
{
	if (load)
	{
		QString	text	=  attributes.value(name)	;
		value	= text.isNull() ? defval : text.toInt() != 0 ;
		return	;
	}

	attributes.append
	(	name,
		QString::null,
		QString::null,
		value ? "1" : "0"
	)	;
}

QVariant
	S1SDataTransfer::widgetValue
	(	QWidget			*widget
	)
{
	if (widget->inherits ("QLineEdit"))
		return	((QLineEdit *)widget)->text	  () ;

	if (widget->inherits ("QComboBox"))
		return	((QComboBox *)widget)->currentItem() ;

	if (widget->inherits ("QListBox" ))
		return	((QListBox  *)widget)->currentItem() ;

	if (widget->inherits ("QCheckBox"))
		return	((QCheckBox *)widget)->isChecked  () ;

	if (widget->inherits ("QGroupBox"))
		return	((QGroupBox *)widget)->isChecked  () ;

	return	QVariant() ;
}

void	S1SDataTransfer::setWidgetValue
	(	QWidget			*widget,
		const QVariant		&value
	)
{
	if (widget->inherits ("QLineEdit"))
	{
		((QLineEdit *)widget)->setText	      (value.toString()) ;
		return	;
	}

	if (widget->inherits ("QComboBox"))
	{
		((QComboBox *)widget)->setCurrentItem (value.toInt   ()) ;
		return	;
	}

	if (widget->inherits ("QListBox" ))
	{
		((QListBox  *)widget)->setCurrentItem (value.toInt   ()) ;
		return	;
	}

	if (widget->inherits ("QCheckBox"))
	{
		((QCheckBox *)widget)->setChecked     (value.toBool  ()) ;
		return	;
	}

	if (widget->inherits ("QGroupBox"))
	{
		((QGroupBox *)widget)->setChecked     (value.toBool  ()) ;
		return	;
	}
}

void	S1SDataTransfer::transfer
	(	const QString		&,
		QString			&value,
		QWidget			*widget,
		const QString		&defval,
		bool			load
	)
{
	if (load)
	{
		QVariant text	= widgetValue (widget) ;
		value	 = text.isNull() ? defval : text.toString() ;
		return	 ;
	}

	setWidgetValue (widget, value) ;
}

void	S1SDataTransfer::transfer
	(	const QString		&,
		int			&value,
		QWidget			*widget,
		int			defval,
		bool			load
	)
{
	if (load)
	{
		QVariant text	= widgetValue (widget)	;
		value	 = text.isNull() ? defval : text.toInt() ;
		return	 ;
	}

	setWidgetValue (widget, value) ;
}

void	S1SDataTransfer::transfer
	(	const QString		&,
		bool			&value,
		QWidget			*widget,
		bool			defval,
		bool			load
	)
{
	if (load)
	{
		QVariant text	= widgetValue (widget)	;
		value	 = text.isNull() ? defval : text.toBool() ;
		return	 ;
	}

	setWidgetValue (widget, value) ;
}


void	S1SDataTransfer::transfer
	(	const QString		&name,
		QString			&value,
		QDict<QWidget>		&widgetDict,
		const QString		&defval,
		bool			load
	)
{
	QWidget	*widget	= widgetDict.find (name) ;

	if (load)
	{
		if (widget != 0)
			transfer (name, value, widget, defval, true) ;
		else	value	 = defval ;

		return	;
	}

	if (widget != 0)
		transfer (name, value, widget, defval, false) ;
}

void	S1SDataTransfer::transfer
	(	const QString		&name,
		int			&value,
		QDict<QWidget>		&widgetDict,
		int			defval,
		bool			load
	)
{
	QWidget	*widget	= widgetDict.find (name) ;

	if (load)
	{
		if (widget != 0)
			transfer (name, value, widget, defval, true) ;
		else	value	 = defval ;

		return	;
	}

	if (widget != 0)
		transfer (name, value, widget, defval, false) ;
}

void	S1SDataTransfer::transfer
	(	const QString		&name,
		bool			&value,
		QDict<QWidget>		&widgetDict,
		bool			defval,
		bool			load
	)
{
	QWidget	*widget	= widgetDict.find (name) ;

	if (load)
	{
		if (widget != 0)
			transfer (name, value, widget, defval, true) ;
		else	value	 = defval ;

		return	;
	}

	if (widget != 0)
		transfer (name, value, widget, defval, false) ;
}


void	S1SDataTransfer::makeWidgetDict
	(	QWidget			*topWidget,
		QDict<QWidget>		&widgetDict,
		const char		*prefix
	)
{
	QObjectList *children	;

	if (prefix == 0)
		children = topWidget->queryList
				(	"QWidget",
					QString("^%1").arg(prefix),
					true,
					true
				)	;
	else	children = topWidget->queryList
				(	"QWidget",
					0,
					false,
					true
				)	;

	if (children != 0)
	{
		uint	plen	= prefix == 0 ? 0 : qstrlen(prefix) ;

		for
		(	QObjectListIt iter (*children)	;
			iter.current() != 0 ;
			iter += 1
		)
			widgetDict.insert
			(	&iter.current()->name()[plen],
				(QWidget *)iter.current()
			)	;

		delete	children ;
	}
}
