import	rkLocator
import	sys
import	re



def getPrefix (settings, libDir) :

    prefix = settings.lookup ('prefix')
    if prefix is not None and prefix != '' : return prefix

    prefix = settings.env ('KDEDIR')
    if prefix is not None and prefix != '' : return prefix

    if libDir is not None :
        for patt in ['^(/opt/kde.*?)/', '^(/usr)/' ] :
            m = re.compile (patt).match(libDir)
            if m : return m.group(1)

    return None



def Scan (settings) :

    if settings.lookup ('mode') != 'kde' :

        text = ''
        text = text + "kde3IncDir = None\n"
        text = text + "kde3LibDir = None\n"
        return text


    incList	= \
		[
		settings.lookup ('with-kde-includes'),
		settings.lookup ('prefix', 'include'),
		settings.env    ('KDEDIR', 'include'),
		'/usr/include',
		'/usr/include/kde3',
		'/usr/include/kde',
		'/opt/kde3/include',
		'/opt/kde/include'
		]

    libList	= \
		[
		settings.lookup ('with-kde-libraries'),
		settings.lookup ('prefix', 'lib'),
		settings.env    ('KDEDIR', 'lib'),
		'/usr/lib',
		'/usr/lib/kde3',
		'/usr/lib/kde',
		'/opt/kde3/lib',
		'/opt/kde/lib'
		]

    incDir = rkLocator.locateFile ('kapplication.h', incList)
    libDir = rkLocator.locateFile ('libkio.so',      libList)
    prefix = getPrefix (settings, libDir)


    if incDir is None :
        settings.log ('Checking for KDE3 includes', 'not found')
        sys.exit     (1)

    if libDir is None :
        settings.log ('Checking for KDE3 libraries', 'not found')
        sys.exit     (1)

    if prefix is None :
        settings.log ('Checking for installation location', 'cannot determin')
        sys.exit     (1)

    text   = ''

    settings.log ('Checking for KDE3 includes',  	incDir)
    settings.log ('Checking for KDE3 libraries', 	libDir)
    settings.log ('Checking for installation location', prefix)

    text = text + "kde3IncDir = '%s'\n" % (incDir)
    text = text + "kde3LibDir = '%s'\n" % (libDir)
    text = text + "prefix = '%s'\n"     % (prefix)

    import re
    re_major = re.compile(  '.*KDE_VERSION_MAJOR\\s+([0-9]+)')
    re_minor = re.compile(  '.*KDE_VERSION_MINOR\\s+([0-9]+)')
    re_rel   = re.compile('.*KDE_VERSION_RELEASE\\s+([0-9]+)')

    major = None
    minor = None
    rel   = None

    for line in open(incDir + '/kdeversion.h').readlines() :
        m_major = re_major.match (line)
        if m_major : major = m_major.group(1)
        m_minor = re_minor.match (line)
        if m_minor : minor = m_minor.group(1)
        m_rel   = re_rel  .match (line)
        if m_rel   : rel   = m_rel  .group(1)

    if major is None or minor is None or rel is None :
        settings.log ('Checking for KDE version', 'cannot determin')
        sys.exit     (1)

    kdever = '%s.%s.%s' % (major, minor, rel)
    kdeverno = '%02d%02d%02d' % (int(major), int(minor), int(rel))
    settings.log ('Checking for KDE version', kdever, kdeverno)

    text = text + "kdever = '%s'\n"     % (kdever)
    text = text + "kdeverno = '%s'\n"   % (kdeverno)

    return text

def Help (settings) :

    print "	--with-kde-includes=dir		: Set KDE include directory"
    print "	--with-kde-libraries=dir	: Set KDE library directory"
    print "	--prefix			: Set target prefix directory"
