  <class name="Block" title="Block Methods" super="Item">
    <method name="getNumRows" comment="Number of rows in associated query">
      <return type="number" comment="Number of rows"/>
      <description>
          Returns the number of rows in the query which provides data for
          the block. This is the number of rows returned by the database,
          plus (or minus) subsequent insertions or deletions.
      </description>
    </method>
    <method name="getQueryRow" comment="Get current query row number">
      <return type="number" comment="Row number"/>
      <description>
          Returns the row number (within the rows returned from the database.
          again plus or minus subsequent insertions or deletions) corresponding
          to the current row, that is, the row that has focus (in a form) or
          which is being processed (in a report).
      </description>
    </method>
    <method name="gotoQueryRow" comment="Move to specified row in query">
      <return type="bool" comment="Success"/>
      <arg name="rowNum" type="number" comment="Row number" opt="No"/>
      <description>
          Focus moves to the row specified, where the row number applies to
          the data returned by the database.
      </description>
    </method>
    <method name="getRowValue" comment="Get value from associated query">
      <return type="string" comment="Value"/>
      <arg name="ctrlName" type="string" comment="Column required, as control name" opt="No"/>
      <arg name="rowNum" type="number" comment="Row number" opt="No"/>
      <description>
          This method returns the value for the specified column (identified by
          the corresponding control name) and row, where the row is the
          row number amongst the data returned by the database. This will
          usually be the same as the value displayed in a form (if the
          record is currently displayed). However, it will be different if the
	  user has edited the displayed value but has not saved the change.
      </description>
    </method>
    <method name="setRowValue" comment="Set value in associated query">
      <return type="void" comment=""/>
      <arg name="ctrlName" type="string" comment="Column required, as control name" opt="No"/>
      <arg name="rowNum" type="number" comment="Row number" opt="No"/>
      <arg name="value" type="string" comment="Value" opt="No"/>
      <description>
          The specified value is stored into the specified column (identified
          by control name) and row in the data returned by the database. Note
          that this will be overwritten if the user subsequently changes
          the value displayed in the corresponding control; also, the value
          will not be saved to the database unless the record is saved.
      </description>
    </method>
    <method name="doAction" comment="Perform action in block">
      <return type="bool" comment="Success"/>
      <arg name="action" type="number" comment="Action code, see RekallMain" opt="No"/>
      <description>
          The specified block action is performed, where the action is
          identified by the code argument. Actions are numbers, but are
          best accessed via <emphasis>RekallMain</emphasis> (for instance,
          <emphasis>RekallMain.actNext</emphasis>) or via a block
          object (for instance, <emphasis>block.actNext</emphasis>). In
          practice, it will almost always be more convenient to use
          on of the action methods (for instance,
          <emphasis>block.firstRecord()</emphasis>).
      </description>
    </method>
    <method name="isInQuery" comment="See if query (find) is in progress">
      <return type="bool" comment="Query (find) in progress"/>
      <description>
          Returns a true result if the block is executing a query, eg.,
          the user has clicked the <emphasis>start query</emphasis> tool
          button.
      </description>
    </method>
    <method name="firstRecord" comment="Go to first record">
      <return type="bool" comment="Success"/>
      <description>
          Moves focus to the first record in the block. This is equivalent
          to <emphasis>block.doAction(block.actFirst)</emphasis>.
      </description>
    </method>
    <method name="previousRecord" comment="Go to previous record">
      <return type="bool" comment="Success"/>
      <description>
          Moves focus to the previous record in the block. If focus is
          already at the first record then no action occurs.
      </description>
    </method>
    <method name="nextRecord" comment="Go to next record">
      <return type="bool" comment="Success"/>
      <description>
          Moves focus to the next in the block. If focus is already at the
          last record, then focus moves to a new empty record (which can be
          filled in and inserted into the database). If focus is already at
          this new record then no action takes place.
      </description>
    </method>
    <method name="lastRecord" comment="Go to last record">
      <return type="bool" comment="Success"/>
      <description>
          Moves focus to the last record. This is the last record returned
          by the database, and not a new empty record.
      </description>
    </method>
    <method name="addRecord" comment="Add a new record">
      <return type="bool" comment="Success"/>
      <description>
          Focus is moved to a new, empty record. The record is not actually
          added at this point; it will be inserted into the database when
          subsequently saved.
      </description>
    </method>
    <method name="saveRecord" comment="Save current record">
      <return type="bool" comment="Success"/>
      <description>
          Changes made to the current record are saved to the database.
      </description>
    </method>
    <method name="deleteRecord" comment="Delete current record">
      <return type="bool" comment="Success"/>
      <description>
          The current record is deleted from the database, and focus moves to the
          next record. Note that after a deletion, records are in effect
          renumbered to account for the deleted record.
      </description>
    </method>
    <method name="startQuery" comment="Start query">
      <return type="bool" comment="Success"/>
      <description>
          The block is set into query mode, equivalent to the user clicking
          the <emphasis>start query</emphasis> tool button. All displayed
          controls for the current row are cleared, and the user can
          then enter search terms prior to executing the query.
      </description>
    </method>
    <method name="executeQuery" comment="Execute query">
      <return type="bool" comment="Success"/>
      <description>
          Executes a query started with the <emphasis>startQuery()</emphasis>
          method (or via the <emphasis>start query</emphasis> tool button or
          menu entry).
      </description>
    </method>
    <method name="cancelQuery" comment="Cancel query">
      <return type="bool" comment="Success"/>
      <description>
          Cancels a query. All controls in the current row are returned
          to their state before the query was started.
      </description>
    </method>
    <method name="setUserFilter" comment="Set filter for subsequent queries">
      <return type="void" comment=""/>
      <arg name="filter" type="string" comment="Filter (where) expression" opt="No"/>
      <description><![CDATA[
          Set a filter expression to be used in subsequent queries. This should be
          a valid SQL <i>where</i> expression, and will be used in the query
          in addition to any other filtering expressions.          
      ]]></description>
      <description>
      </description>
    </method>
    <method name="setUserSorting" comment="Set sorting for subsequent queries">
      <return type="void" comment=""/>
      <arg name="sorting" type="string" comment="Sorting (order) expression" opt="No"/>
      <description><![CDATA[
          Set an ordering expression to be used in subsequent queries. This
          should be a valid SQL <i>order by</i> expression, and will be used in
          the query in addition to (and after) any other ordering expressions.          
      ]]></description>
      <description>
      </description>
    </method>
  </class>
