  <class name="Form" title="Form Methods" super="FormBlock">
    <method name="openForm" comment="Open a named form">
      <return type="dict" comment="Field dictionary if modal"/>
      <arg name="formName" type="string" comment="Name of form to be opened" opt="No"/>
      <arg name="parameters" type="dictionary" comment="Parameter dictionary" opt="Yes"/>
      <arg name="key" type="any" comment="Parent/child link value" opt="Yes"/>
      <description><![CDATA[
      <para>
        Opens the named form. The optional second argument is passed as the
        parameter dictionary to the form; if present, the optional third argument
        is used as the parent value to provide master-child linking (in the
        same manner as for form-subform linkage). 
      </para>
      <para>
        The parameters can also include the key <emphasis>__showAs</emphasis> which can be
        set to <emphasis>ShowAsDesign</emphasis> to show the form in design mode (set
        to <emphasis>ShowAsData</emphasis> gives the default behaviour).
      </para>
      <para>
        If the form is modal, then control does not return until the form
        closes. In this case the result is a dictionary, where the keys are
        the (path) names of the controls in the form and the values are the
        values in those controls. This allows modal forms to be used as
        dialog boxes.
      </para>
      ]]></description>
    </method>
    <method name="openReport" comment="Open a named report">
      <return type="bool" comment="Success"/>
      <arg name="reportName" type="string" comment="Name of report to be opened" opt="No"/>
      <arg name="parameters" type="dictionary" comment="Parameter dictionary" opt="Yes"/>
      <arg name="key" type="any" comment="Parent/child link value" opt="Yes"/>
      <description><![CDATA[
      <para>
        Opens the named report. The optional second argument is passed as the
        parameter dictionary to the report; if present, the optional third argument
        is used as the parent value to provide master-child linking (in the
        same manner as for form-subform linkage). 
      </para>
      <para>
        The parameters can also include the key <emphasis>__showAs</emphasis> which can be
        set to <emphasis>ShowAsDesign</emphasis> to show the report in design mode, or
        <emphasis>ShowAsData</emphasis> to show in data view (set
        to <emphasis>ShowAsPrint</emphasis> gives the default behaviour).
      </para>
      <para>
        If the report is modal the control does not return until the report
        is closed.
      </para>
      ]]></description>
    </method>
    <method name="openTextForm" comment="Open an XML form definition">
      <return type="bool" comment="Success"/>
      <arg name="xmlDefn" type="string" comment="XML definition" opt="No"/>
      <arg name="parameters" type="dictionary" comment="Parameter dictionary" opt="Yes"/>
      <description><![CDATA[
      ]]></description>
    </method>
    <method name="openTextReport" comment="Open an XML report definition">
      <return type="bool" comment="Success"/>
      <arg name="xmlDefn" type="string" comment="XML definition" opt="No"/>
      <arg name="parameters" type="dictionary" comment="Parameter dictionary" opt="Yes"/>
      <description><![CDATA[
      ]]></description>
    </method>
    <method name="openTable" comment="Open a named table">
      <return type="bool" comment="Success"/>
      <arg name="formName" type="string" comment="Name of table to be opened" opt="No"/>
      <arg name="parameters" type="dictionary" comment="Parameter dictionary" opt="Yes"/>
      <description><![CDATA[
      ]]></description>
    </method>
    <method name="openQuery" comment="Open a named query">
      <return type="bool" comment="Success"/>
      <arg name="formName" type="string" comment="Name of query to be opened" opt="No"/>
      <arg name="parameters" type="dictionary" comment="Parameter dictionary" opt="Yes"/>
      <description><![CDATA[
      ]]></description>
    </method>
    <method name="openServer" comment="Get low-level connection to server database">
      <return type="pydbi" comment="Connection object"/>
      <arg name="serverName" type="string" comment="Server name" opt="No"/>
      <description><![CDATA[
      ]]></description>
    </method>
    <method name="executeCopier" comment="Execute a copier">
      <return type="number" comment="Number of rows copied"/>
      <arg name="copierName" type="string" comment="Copier name" opt="No"/>
      <arg name="parameters" type="dictionary" comment="Parameter dictionary" opt="Yes"/>
      <description><![CDATA[
      ]]></description>
    </method>
    <method name="getServerList" comment="Get list of servers">
      <return type="list" comment="List of server names"/>
      <description><![CDATA[
      ]]></description>
    </method>
    <method name="getObjectList" comment="Get list of objects on server">
      <return type="list" comment="List of object names"/>
      <arg name="serverName" type="string" comment="Server name" opt="No"/>
      <arg name="objectType" type="string" comment="Object type" opt="No"/>
      <description><![CDATA[
      ]]></description>
    </method>
    <method name="getObjecttext" comment="Get XML definition of object">
      <return type="string" comment="XML definition text"/>
      <arg name="serverName" type="string" comment="Server name" opt="No"/>
      <arg name="objectName" type="string" comment="Object name" opt="No"/>
      <description><![CDATA[
      ]]></description>
    </method>
    <method name="close" comment="Close the form">
      <arg name="rc" type="number" comment="Form exit code" opt="No"/>
      <return type="void" comment=""/>
      <description><![CDATA[
      Closes the form.
      ]]></description>
    </method>
    <method name="closeAll" comment="Close the form">
      <arg name="rc" type="number" comment="Form exit code" opt="No"/>
      <return type="void" comment=""/>
      <description><![CDATA[
      This is identical to <emphasis>close</emphasis>.
      ]]></description>
    </method>
    <method name="findOpenForm" comment="Locate an open form">
      <return type="form" comment="From or None if not open"/>
      <arg name="formName" type="string" comment="Name of form to be found" opt="No"/>
    </method>
    <method name="opener" comment="Get opening form">
      <return type="dict" comment="Opening form or None"/>
      <description>
         Returns the form that opened this form, if that form is still
         open. Note that the result should not be cached, lest the opening
         form be subsequently closed.
      </description>
    </method>
    <method name="getParameter" comment="Get paramater value">
      <return type="string" comment="Paramater value or None"/>
      <arg name="paramater" type="string" comment="Parameter name"/>
      <description>
         Returns the value of the named parameter, as passed to the form
         or set in the form paramater dialog.
      </description>
    </method>
    <method name="parameterDict" comment="Get paramater dictionary">
      <return type="dict" comment="Paramater dictionary"/>
      <description>
         Returns the entire set of parameters as a dictionary; the
         dictionary keys are the parameter names.
      </description>
    </method>
    <method name="getGlobalCtrl" comment="Locate control globally by name">
      <return type="object" comment="Control or None if not found"/>
      <arg name="name" type="string" comment="Control name" opt="No"/>
      <description><![CDATA[
          This method searches for the named control in the form,
          recursing down as needed (except that it does not recurse
          into linked components). Note that if there is more than one
          control with the name name, then the result is not defined
          (any such named control might be returned), and the
          <emphasis>getNamedCtrl</emphasis> method should be used.
      ]]></description>
    </method>
    <method name="getOpener" comment="Get opening form">
      <return type="object" comment="Opening form"/>
      <description><![CDATA[
      If the form was opened by another form (ie., by a call to
      <emphasis>openForm</emphasis> or one of the related method),
      then this method returns the form-level object in that form.
      If opened directly from the list of forms, or from a macro,
      then the result is <emphasis>None</emphasis>.
      ]]></description>
    </method>
  </class>
