  <class name="Object" title="Object Methods">
    <property name="__parent__" access="r" comment="Parent object">
      <description>
        The <emphasis>__parent__</emphasis> property accesses the parent
        object in the object tree, and is equivalent to the
        <emphasis>getParent()</emphasis> method call. The result is
        <emphasis>None</emphasis> if there is no parent.
      </description>
    </property>
    <property name="__block__" access="r" comment="Enclosing block">
      <description>
        The <emphasis>__block__</emphasis> property accesses the 
        enclosing block in the object tree, and is equivalent to the
        <emphasis>getBlock()</emphasis> method call. The result is
        <emphasis>None</emphasis> if there is no such block.
      </description>
    </property>
    <property name="__root__" access="r" comment="Root object">
      <description>
        The <emphasis>__root__</emphasis> property accesses the topmost
        object in the object tree, and is equivalent to the
        <emphasis>getForm()</emphasis> method call within forms.
      </description>
    </property>
    <method name="setEnabled" comment="Enable or disable control">
      <return type="void" comment=""/>
      <arg name="enable" type="bool" comment="True to enable" opt="No"/>
      <description>
	Sets the object to enabled or disabled state. In the disabled
	state, the user cannot, for instance, make changes in data control,
	or click a button.
      </description>
    </method>
    <method name="setVisible" comment="Show or hide control">
      <return type="void" comment=""/>
      <arg name="show" type="bool" comment="True to show" opt="No"/>
      <description>
        Shows (makes visible) or hides the object. Note that when using
        dynamic geometry management, hiding an object may cause the
        layout to adjust to use up the vacated space.
      </description>
    </method>
    <method name="isEnabled" comment="Test if control is enabled">
      <return type="bool" comment="True if enabled"/>
      <description>
         Returns a tre result if the object is enabled.
      </description>
    </method>
    <method name="isVisible" comment="Test if control is visible">
      <return type="bool" comment="True if visible"/>
      <description>
         Returns a tre result if the object is visible.
      </description>
    </method>
    <method name="getElement" comment="Get contol element">
      <return type="string" comment="Element"/>
      <description><![CDATA[
          Returns the control element, ie., the type of the control,
          for instance <i>KBField</i> or <i>KBButton</i>
      ]]></description>
    </method>
    <method name="getName" comment="Get control name">
      <return type="string" comment="Name"/>
      <description>
          Returns the control name. This is equivalent to
          <emphasis>getAttr("name")</emphasis>
      </description>
    </method>
    <method name="setAttr" comment="Set attribute (property)">
      <return type="void" comment=""/>
      <arg name="name" type="string" comment="Attribute name" opt="No"/>
      <arg name="value" type="string" comment="Value to set" opt="No"/>
      <arg name="update" type="number" comment="Non-zero to update control" opt="Yes"/>
      <description>
        Sets the specified attribute to the specified value and, if the
        third (optional) argument is true, applies the change. If serveral
        changes are being made is is more efficient (and may result in less
        visual flicker) if all but the last call are made with a false thrird
        argument. Note also that which <emphasis>setAttr()</emphasis> works
        on such things as visual properties (colours, fonts, etc) it will
        not affect function properties (table names, ordering, etc).
      </description>
    </method>
    <method name="getAttr" comment="Get attribute (property)">
      <return type="string" comment="Attribute value"/>
      <arg name="name" type="string" comment="Attribute name" opt="No"/>
      <description>
        Retrieves the value if the named attribute.
      </description>
    </method>
    <method name="update" comment="Update control for attribute changes">
      <return type="void" comment=""/>
      <description>
         Forces update after changing attribute values. This is equivalent
         to the third argument of the <emphasis>setAttr()</emphasis> method
         being true. See <emphasis>setAttr()</emphasis> for additional
         details.
      </description>
    </method>
    <method name="width" comment="Get control width">
      <return type="number" comment="Width"/>
      <description>
          Returns the control (or, in general, object) width, in pixels.
      </description>
    </method>
    <method name="height" comment="Get control height">
      <return type="number" comment="Height"/>
      <description>
          Returns the control (or, in general, object) height, in pixels.
      </description>
    </method>
    <method name="resize" comment="Resize control">
      <return type="void" comment=""/>
      <arg name="width" type="number" comment="New width" opt="No"/>
      <arg name="height" type="number" comment="New height" opt="No"/>
      <description>
          Resizes the control to the specified width and height. Note that this
          will only work when geometry management is static; using this
          call with dynamic management may have unpredictable effects.
      </description>
    </method>
    <method name="getParent" comment="Get parent object if any">
      <return type="object" comment="Parent or None"/>
      <description>
          Returns the parent object or <emphasis>None</emphasis> if there
          is no parent (ie., the method is being called on a form or
          report object).
      </description>
    </method>
    <method name="getBlock" comment="Get enclosing block if any">
      <return type="block" comment="Enclosing block or None"/>
      <description>
          Returns the immediately enclosing (form or report) block, or
          <emphasis>None</emphasis> is there is no such block
          (ie., the method is being called on a form or report object).
      </description>
    </method>
    <method name="getForm" comment="Get enclosing form if any">
      <return type="form" comment="Enclosing form or None"/>
      <description>
          Returns the enclosing form, or <emphasis>None</emphasis> if
          called on an object in a report. Note that calling
          <emphasis>getForm()</emphasis> on a form object will return
          that form.
      </description>
    </method>
    <method name="lastError" comment="Get last error">
      <return type="string" comment="Text message for last error"/>
      <description>
          Returns a textual message corresponding to the last error
          which occured relating to the object to which the
          <emphasis>lastError()</emphasis> method is applied.
      </description>
    </method>
    <method name="getNamedCtrl" comment="Locate control by name">
      <return type="object" comment="Control or None if not found"/>
      <arg name="name" type="string" comment="Control name relative to this object" opt="No"/>
      <arg name="bool" type="errorOK" comment="Show error dialog if no control found" opt="Yes"/>
      <description>
          When applied to an object, this method locates the object
          which has the specified path name relative to the object. The
          (path) name can comprise multiple parts, separated by the /
          character, where double periods <emphasis>..</emphasis> means the
          parent (as in the manner or <emphasis>Unix</emphasis> filesystem
          path names). <emphasis>getBlock()</emphasis> (without spaces) means 
          the enclosing block, while <emphasis>getRoot()</emphasis> returns
          the outermost form or report block. If the third argument is
          true (the default) and the path name is invalid, an error dialog will
          be shown.
      </description>
    </method>
    <method name="getControls" comment="Get list of data controls">
      <return type="list" comment="List of data controls"/>
      <description>
          Returns a list of all data controls which are immediate
          children of the object to which the method is applied. Note
          that blocks are excluded
      </description>
    </method>
    <method name="getChildren" comment="Get list of all chilren">
      <return type="list" comment="List of children"/>
      <description>
          Returns a list of all immediate children of the object
          to which the method is applied.
      </description>
    </method>
    <method name="getConfig" comment="Get configuration value">
      <return type="string" comment="Configuration value"/>
      <arg name="name" type="string" comment="Configuration name" opt="No"/>
      <description>
          Returns the value of the named configuration entry for the
          object to which the method is applied. Configuration entries are
          used to parameterise linked components.
      </description>
    </method>
    <method name="setFGColor" comment="Set control foreground color">
      <return type="void" comment=""/>
      <arg name="color" type="string" comment="Colour as #XXXXXX" opt="No"/>
      <description>
          Sets the foreground colour of the object.
      </description>
    </method>
    <method name="setBGColor" comment="Set control background color">
      <return type="void" comment=""/>
      <arg name="color" type="string" comment="Colour as #XXXXXX" opt="No"/>
      <description>
          Sets the background colour of the object.
      </description>
    </method>
    <method name="setFont" comment="Set control font">
      <return type="void" comment=""/>
      <arg name="font" type="string" comment="Font description" opt="No"/>
      <description>
          Sets the font of the object. Fonts are described as can be seen
          in the propery dialogs
          <emphasis>family;size;weight;italic</emphasis>; eg.,
	  <emphasis>Arial;10;75;1</emphasis> is 10-point bold-italic
	  Arial.
      </description>
    </method>
  </class>
