#!/bin/sh
#
# Alltests wrapper script for Linux (KDE and QT3)
#
#
prefix=
   gui=
  rapp='--rapp'
   app=rekall
 rootd=
 names=
  path=
   xml=

   tmp=/tmp
 xbase=/tmp
dbhost=192.168.1.66
  skip=


#  Decode arguments. All are of the form --arg or --arg=value
#
while [ $# -gt 0 ]
do
	case $1 in
		--gui)
			# Run with the QT test runner GUI. The
			# RekallApp option is disabled.
			#
			gui='--gui'
			rapp=
			;;

		--rapp)
			# Run with a Rekall application instance.
			# Disabled the QT test runner GUI.
			#
			rapp='--rapp'
			gui=
			;;

		--names)
			# Display test names as they run.
			#
			names='--names'
			;;

		--prefix=*)
			# Specify a prefix. Can be used for testing during
			# builds and to handle testing rekall and rekallqt
			# on the same system.
			#
			prefix=`expr $1 : '--prefix=\(.*\)'`
			;;

		--dbhost=*)
			# Specify a non-standard database host.
			#
			dbhost=`expr $1 : '--dbhost=\(.*\)'`
			;;

		--skip=*)
			# Specify an initial skip list
			#
			skip=`expr $1 : '--skip=\(.*\)'`
			;;

		--app=*)
			# Specify a non-standard application name.
			# Used for testing, for instance, rekalldev.
			#
			app=`expr $1 : '--app=\(.*\)'`
			;;

		--xml=*)
			# Output results to XML file.
			#
			xml=$1
			;;

		--qt3)
			# Run the QT3 build. The prefix is always
			# /usr/local and the applcation name rekallqt.
			#
			prefix=/usr/local
			app=rekallqt
			;;

		*)
			# Anything else is an error.
			#
			echo alltests.sh: $1: bad argument
			exit 1
			;;
	esac

	shift
done


#  If a non-standard prefix is specified then:
#	* set the library path so we get the right libraries
#	* set the test path to the corresponding directory
#	* set the non-default root directory
#
if [ "$prefix" != "" ]
then
	export KDEDIR=$prefix
	export LD_LIBRARY_PATH=$prefix/lib
	export TESTPATH=$prefix/share/apps/$app
	rootd="--rootdir=$prefix"
fi


matched=


if grep -qi 'suse' /etc/issue
then
	matched=yes
	skip=$skip

	[ "$prefix" = "" ] && prefix=/opt/kde3
fi

if grep -qi 'fedora' /etc/issue
then
	matched=yes
	skip=$skip

	[ "$prefix" = "" ] && prefix=/usr
fi

if grep -qi 'ubuntu' /etc/issue
then
	matched=yes
	skip=$skip

	[ "$prefix" = "" ] && prefix=/usr
fi

if grep -qi 'debian' /etc/issue
then
	matched=yes
	skip=$skip

	[ "$prefix" = "" ] && prefix=/usr
fi

if grep -qi 'Linspire' /etc/issue
then
	matched=yes
	skip=$skip

	[ "$prefix" = "" ] && prefix=/usr
fi

if grep -qi 'Freespire' /etc/issue
then
	matched=yes
	skip=$skip

	[ "$prefix" = "" ] && prefix=/usr
fi

if grep -qi 'Xandros' /etc/issue
then
	matched=yes
	skip=$skip

	[ "$prefix" = "" ] && prefix=/usr
fi

rm	-f $xbase/*.dbf $tmp/*.dbf
rm	-f $xbase/*.dbt $tmp/*.dbt
rm	-f $xbase/*.ndx $tmp/*.ndx

cp	$prefix/share/apps/$app/test/driver/__RekallObjects.* $xbase


skip=$skip

if [ "${matched}" != "yes" ]
then
	echo	"alltests.sh: distribution not recognised"
	exit	1
fi

if [ "$DISPLAY" = "" ]
then
	export DISPLAY=192.168.1.66:0.0
fi

[ -f $prefix/bin/rkt_alltests    ] &&
{
	echo $prefix/bin/rkt_alltests		\
		--nocrashhandler		\
		--silent			\
		--tmp=$tmp			\
		--xbase=$xbase			\
		--dbhost=$dbhost		\
		--skip=$skip			\
		$rootd				\
		$gui				\
		$names				\
		$xml				\
		$rapp

	$prefix/bin/rkt_alltests		\
		--nocrashhandler		\
		--silent			\
		--tmp=$tmp			\
		--xbase=$xbase			\
		--dbhost=$dbhost		\
		--skip=$skip			\
		$rootd				\
		$gui				\
		$names				\
		$xml				\
		$rapp				\
	|| exit 1
}

[ -f $prefix/bin/rkt_alltests_rt ] &&
{
	echo $prefix/bin/rkt_alltests_rt	\
		--nocrashhandler		\
		--silent			\
		--tmp=$tmp			\
		--xbase=$xbase			\
		--dbhost=$dbhost		\
		--skip=$skip			\
		$rootd				\
		$gui				\
		$names				\
		$xml				\
		$rapp

	$prefix/bin/rkt_alltests_rt		\
		--nocrashhandler		\
		--silent			\
		--tmp=$tmp			\
		--xbase=$xbase			\
		--dbhost=$dbhost		\
		--skip=$skip			\
		$rootd				\
		$gui				\
		$names				\
		$xml				\
		$rapp				\
	|| exit 1
}

exit	0

