/***************************************************************************
    file	         : api.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qapplication.h>
#include	<qstring.h>
#include	<qmap.h>
#include	<qlabel.h>

#include	"rk_application.h"
#include	"rk_lineedit.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_label.h"
#include	"kb_field.h"
#include	"kb_choice.h"
#include	"kb_memo.h"
#include	"kb_textedit.h"
#include	"kb_richtext.h"
#include	"kb_qtrichtext.h"
#include	"kb_listbox.h"
#include	"kb_dom.h"

#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"
#include	"api.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



KBAPITest::KBAPITest ()
{
}

KBAPITest::KBAPITest
	(	std::string		name
	)
	:
	CppUnit::TestCase (name)
{
}

void	KBAPITest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBAPITest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

void	KBAPITest::openForm
	(	const char	*name
	)
{
	KBDomDocument	doc	;
	QDomElement	info	;
	if (!doc.loadFile ("API.rkl", "test/api", getenv("TESTPATH")))
	{
		fprintf
		(	stderr,
			"validity: %s: cannot open: %s\n",
			"API.rkl",
			doc.lastError().getMessage().latin1()
		)	;
		exit	(1) ;
	}
	DOMITER_BEGIN(doc.documentElement(),"serverinfo",elem)
	{
		info	= elem	;
	}
	DOMITER_END

	KBDBInfo	*dbInfo	= new KBDBInfo     ("noname") ;
	KBServerInfo	*svInfo	= new KBServerInfo (dbInfo, info) ;

	dbInfo->insert	(svInfo);

	KBError	error	;
	m_form	= rekallTestOpenForm (dbInfo, "api", name, error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;
}

void	KBAPITest::closeForm ()
{
	rekallTestCloseForm () ;
}

KBObject
	*KBAPITest::getObject
	(	const char	*path,
		const char	*name,
		const char	*what
	)
{
	KBNode	 *target = m_form->m_form->getNamedNode(path) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("locating %1 as %2").arg(path).arg(what)),
		target != 0
	) 	;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("locating %1 as %2: class is %3").arg(path).arg(what).arg(target->className())),
		target->inherits("KBObject")
	)	;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("locating %1 as %2: class is %3").arg(path).arg(what).arg(target->className())),
		target->isA	(what)
	)	;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("locating %1 as %2").arg(path).arg(what)),
		_STDSTR(QString(name)),
		_STDSTR(target->getName())
	)	;

	return	(KBObject *)target	;
}

KBField
	*KBAPITest::getField
	(	const char	*path,
		const char	*name
	)
{
	return	(KBField *)getObject (path, name, "KBField") ;
}

KBLabel
	*KBAPITest::getLabel
	(	const char	*path,
		const char	*name
	)
{
	return	(KBLabel *)getObject (path, name, "KBLabel") ;
}

KBChoice
	*KBAPITest::getChoice
	(	const char	*path,
		const char	*name
	)
{
	return	(KBChoice *)getObject (path, name, "KBChoice") ;
}

KBMemo
	*KBAPITest::getMemo
	(	const char	*path,
		const char	*name
	)
{
	return	(KBMemo *)getObject (path, name, "KBMemo") ;
}

KBRichText
	*KBAPITest::getRichText
	(	const char	*path,
		const char	*name
	)
{
	return	(KBRichText *)getObject (path, name, "KBRichText") ;
}

KBListBox
	*KBAPITest::getListBox
	(	const char	*path,
		const char	*name
	)
{
	return	(KBListBox *)getObject (path, name, "KBListBox") ;
}

KBFormBlock
	*KBAPITest::getFormBlock
	(	const char	*path,
		const char	*name
	)
{
	KBNode	 *target = m_form->m_form->getNamedNode(name) ;

	CPPUNIT_ASSERT (target != 0) ;
	CPPUNIT_ASSERT (target->isA("KBFormBlock")) ;

	KBFormBlock *fblk = (KBFormBlock *)target ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString(name)),
		_STDSTR(fblk->getName())
	)	;

	return	fblk ;
}

KBControl
	*KBAPITest::getControl
	(	KBObject	*object,
		const char	*what
	)
{
	KBControl *control = KBTestAccessor::getControl (object) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA(what)) ;
	return	  control  ;
}

KBControl
	*KBAPITest::getControl
	(	KBItem		*item,
		const char	*what,
		uint		qrow,
		uint		drow
	)
{
	KBControl *control = KBTestAccessor::getControl (item, qrow) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA(what)) ;
	CPPUNIT_ASSERT (control->getDRow() == drow ) ;
	return	  control  ;
}

KBControl
	*KBAPITest::getControl
	(	KBField		*field
	)
{
	return	getControl (field, "KBCtrlField") ;
}

KBControl
	*KBAPITest::getControl
	(	KBLabel		*label
	)
{
	return	getControl (label, "KBCtrlLabel") ;
}

KBControl
	*KBAPITest::getControl
	(	KBField		*field,
		uint		qrow,
		uint		drow
	)
{
	return	getControl (field, "KBCtrlField", qrow, drow) ;
}

KBControl
	*KBAPITest::getControl
	(	KBChoice	*choice
	)
{
	return	getControl (choice, "KBCtrlChoice") ;
}

KBControl
	*KBAPITest::getControl
	(	KBChoice	*choice,
		uint		qrow,
		uint		drow
	)
{
	return	getControl (choice, "KBCtrlChoice", qrow, drow) ;
}

KBControl
	*KBAPITest::getControl
	(	KBListBox	*listbox
	)
{
	return	getControl (listbox, "KBCtrlListBox") ;
}

KBControl
	*KBAPITest::getControl
	(	KBMemo		*memo
	)
{
	return	getControl (memo, "KBCtrlMemo") ;
}

KBControl
	*KBAPITest::getControl
	(	KBRichText	*richtext
	)
{
	return	getControl (richtext, "KBCtrlRichText") ;
}

KBControl
	*KBAPITest::getControl
	(	KBListBox	*listbox,
		uint		qrow,
		uint		drow
	)
{
	return	getControl (listbox, "KBCtrlListBox", qrow, drow) ;
}

RKLineEdit
	*KBAPITest::getRKLineEdit
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Expecting RKLineEdit, got %1").arg(widget->className())),
		widget->isA("RKLineEdit")
	)	;
	return	(RKLineEdit *)widget ;
}

RKComboBox
	*KBAPITest::getRKComboBox
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Expecting RKComboBox, got %1").arg(widget->className())),
		widget->isA("RKComboBox")
	) ;
	return	(RKComboBox *)widget ;
}

RKListBox
	*KBAPITest::getRKListBox
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Expecting RKListBox, got %1").arg(widget->className())),
		widget->isA("RKListBox")
	)	;
	return	(RKListBox *)widget ;
}

RKTextEdit
	*KBAPITest::getRKTextEdit
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;

	if (widget->isA("KBTextEdit"))
		widget = ((KBTextEdit *)widget)->textEdit() ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Expecting RKTextEdit, got %1").arg(widget->className())),
		widget->inherits("RKTextEdit")
	)	;
	return	(RKTextEdit *)widget ;
}

RKTextBrowser
	*KBAPITest::getRKTextBrowser
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;

	if (widget->isA("KBCtrlRichTextContainer"))
		widget = ((KBCtrlRichTextContainer *)widget)->wrapper() ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Expecting RKTextEdit, got %1").arg(widget->className())),
		widget->inherits("RKTextBrowser")
	)	;
	return	(RKTextBrowser *)widget ;
}

QLabel	*KBAPITest::getQLabel
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Expecting QLabel, got %1").arg(widget->className())),
		widget->isA("QLabel")
	)	;
	return	(QLabel *)widget ;
}

/*  ------------------------------------------------------------------	*/


extern	CppUnit::Test	*modesTestSuite		(const QMap<QString,QString> *) ;
extern	CppUnit::Test	*containerTestSuite	(const QMap<QString,QString> *) ;
extern	CppUnit::Test	*stackTestSuite		(const QMap<QString,QString> *) ;
extern	CppUnit::Test	*tabberTestSuite	(const QMap<QString,QString> *) ;
extern	CppUnit::Test	*buttonTestSuite	(const QMap<QString,QString> *) ;
extern	CppUnit::Test	*labelTestSuite		(const QMap<QString,QString> *) ;
extern	CppUnit::Test	*fieldTestSuite		(const QMap<QString,QString> *) ;
extern	CppUnit::Test	*checkTestSuite		(const QMap<QString,QString> *) ;
extern	CppUnit::Test	*choiceTestSuite	(const QMap<QString,QString> *) ;
extern	CppUnit::Test	*listboxTestSuite	(const QMap<QString,QString> *) ;
extern	CppUnit::Test	*linkTestSuite		(const QMap<QString,QString> *) ;
extern	CppUnit::Test	*nestedTestSuite	(const QMap<QString,QString> *) ;
extern	CppUnit::Test	*globalNodeTestSuite	(const QMap<QString,QString> *) ;
extern	CppUnit::Test	*validityTestSuite	(const QMap<QString,QString> *) ;
extern	CppUnit::Test	*validityNestedTestSuite(const QMap<QString,QString> *) ;
extern	CppUnit::Test	*mapCaseTestSuite	(const QMap<QString,QString> *) ;
extern	CppUnit::Test	*caretFocusTestSuite	(const QMap<QString,QString> *) ;
extern	CppUnit::Test	*treeTestSuite		(const QMap<QString,QString> *) ;
extern	CppUnit::Test	*formatTestSuite	(const QMap<QString,QString> *) ;
extern	CppUnit::Test	*scanTestSuite		(const QMap<QString,QString> *) ;
extern	CppUnit::Test	*recordsTestSuite	(const QMap<QString,QString> *) ;
extern	CppUnit::Test	*subformTestSuite	(const QMap<QString,QString> *) ;
extern	CppUnit::Test	*framesTestSuite	(const QMap<QString,QString> *) ;
extern	CppUnit::Test	*queryTestSuite		(const QMap<QString,QString> *) ;

CppUnit::Test EXPORT_API *testSetup
	(	const QMap<QString,QString> *settings
	)
{
	CppUnit::TestSuite *suite = new CppUnit::TestSuite ("Api") ;

	if (rekallTestSkip ("API")) return suite ;

#if	! __KB_RUNTIME
	suite->addTest (modesTestSuite		(settings))	;
#endif
	suite->addTest (containerTestSuite	(settings))	;
	suite->addTest (stackTestSuite		(settings))	;
	suite->addTest (tabberTestSuite		(settings))	;
	suite->addTest (globalNodeTestSuite	(settings))	;
	suite->addTest (buttonTestSuite		(settings))	;
	suite->addTest (labelTestSuite		(settings))	;
	suite->addTest (fieldTestSuite		(settings))	;
	suite->addTest (checkTestSuite		(settings))	;
	suite->addTest (choiceTestSuite		(settings))	;
	suite->addTest (listboxTestSuite	(settings))	;
	suite->addTest (linkTestSuite		(settings))	;
	suite->addTest (nestedTestSuite		(settings))	;
	suite->addTest (validityTestSuite	(settings))	;
	suite->addTest (validityNestedTestSuite	(settings))	;
	suite->addTest (mapCaseTestSuite	(settings))	;
	suite->addTest (caretFocusTestSuite	(settings))	;
	suite->addTest (treeTestSuite		(settings))	;
	suite->addTest (formatTestSuite		(settings))	;
	suite->addTest (scanTestSuite		(settings))	;
	suite->addTest (recordsTestSuite	(settings))	;
	suite->addTest (subformTestSuite	(settings))	;
	suite->addTest (framesTestSuite		(settings))	;
	suite->addTest (queryTestSuite		(settings))	;
	return	suite	;
}


#include	"kb_libloader.h"
#include	"factory.h"

TESTFACTORY	(KBTestAPIFactory,API,rkt_api)
