/***************************************************************************
    file	         : buttons.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qmainwindow.h>
#include	<qstring.h>
#include	<qmap.h>
#include	<qregexp.h>

#include	"rk_application.h"
#include	"rk_pushbutton.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_button.h"
#include	"kb_sizer.h"
#include	"kb_dom.h"

#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBButtonTest : public CppUnit::TestCase
{
	TestOpenForm		*m_form		;

	KBButton		*getButton	(const char *, const char *) ;
	KBControl		*getControl	(KBButton  *) ;
	RKPushButton		*getWidget	(KBControl  *) ;

public	:

	KBButtonTest	() ;
	KBButtonTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testButtons0	() ;
	void			testButtons1	() ;
	void			testButtons2	() ;
	void			testButtons3	() ;
	void			testButtons4	() ;
	void			testButtons5	() ;
	void			testButtons6	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBButtonTest::KBButtonTest ()
{
}

KBButtonTest::KBButtonTest
	(	std::string		name
	)
	:
	CppUnit::TestCase (name)
{
}

void	KBButtonTest::setUp ()
{
	KBError	error	;
	m_form	= rekallTestOpenForm (0, "api", "Buttons", error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;
}

void	KBButtonTest::tearDown ()
{
	rekallTestCloseForm () ;
}

void	KBButtonTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBButtonTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}


KBButton
	*KBButtonTest::getButton
	(	const char	*name,
		const char	*text
	)
{
	KBNode	 *target = m_form->m_form->getNamedNode(name) ;

	CPPUNIT_ASSERT (target != 0) ;
	CPPUNIT_ASSERT (target->isA("KBButton")) ;

	KBButton *button = (KBButton *)target ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString(name)),
		_STDSTR(button->getName())
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString(text)),
		_STDSTR(button->getText())
	)	;

	return	button	;
}

KBControl
	*KBButtonTest::getControl
	(	KBButton	*button
	)
{
	KBControl *control = KBTestAccessor::getControl (button) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA("KBCtrlButton")) ;
	return	  control  ;
}

RKPushButton
	*KBButtonTest::getWidget
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;
	CPPUNIT_ASSERT (widget->isA("RKPushButton")) ;
	return	(RKPushButton *)widget ;
}

void	KBButtonTest::testButtons0 ()
{
	KBButton *button   = getButton  ("Button_0", "button_0") ;

	CPPUNIT_ASSERT ( button->isVisible()) ;
	CPPUNIT_ASSERT ( button->isEnabled()) ;

	KBControl *control = getControl (button ) ;
	QWidget	  *widget  = getWidget  (control) ;

	CPPUNIT_ASSERT ( widget->isVisible()) ;
	CPPUNIT_ASSERT ( widget->isEnabled()) ;
}

void	KBButtonTest::testButtons1 ()
{
	KBButton *button = getButton ("Button_1", "button_1") ;

	CPPUNIT_ASSERT ( button->isVisible()) ;
	CPPUNIT_ASSERT (!button->isEnabled()) ;

	KBControl *control = getControl (button ) ;
	QWidget	  *widget  = getWidget  (control) ;

	CPPUNIT_ASSERT ( widget->isVisible()) ;
	CPPUNIT_ASSERT (!widget->isEnabled()) ;
}

void	KBButtonTest::testButtons2 ()
{
	KBButton *button = getButton ("Button_2", "button_2") ;

	CPPUNIT_ASSERT (!button->isVisible()) ;
	CPPUNIT_ASSERT ( button->isEnabled()) ;

	KBControl *control = getControl (button ) ;
	QWidget	  *widget  = getWidget  (control) ;

	CPPUNIT_ASSERT (!widget->isVisible()) ;
	CPPUNIT_ASSERT ( widget->isEnabled()) ;
}

void	KBButtonTest::testButtons3 ()
{
	KBButton *button = getButton ("Button_3", "button_3") ;

	CPPUNIT_ASSERT (!button->isVisible()) ;
	CPPUNIT_ASSERT (!button->isEnabled()) ;

	KBControl *control = getControl (button ) ;
	QWidget	  *widget  = getWidget  (control) ;

	CPPUNIT_ASSERT (!widget->isVisible()) ;
	CPPUNIT_ASSERT (!widget->isEnabled()) ;
}

void	KBButtonTest::testButtons4 ()
{
	KBButton *button   = getButton  ("Button_0", "button_0") ;

	CPPUNIT_ASSERT ( button->isVisible()) ;
	CPPUNIT_ASSERT ( button->isEnabled()) ;

	KBControl *control = getControl (button ) ;
	QWidget	  *widget  = getWidget  (control) ;

	CPPUNIT_ASSERT ( widget->isVisible()) ;
	CPPUNIT_ASSERT ( widget->isEnabled()) ;

	button->setVisible (false) ;
	button->setEnabled (false) ;

	CPPUNIT_ASSERT (!button->isVisible()) ;
	CPPUNIT_ASSERT (!button->isEnabled()) ;
	CPPUNIT_ASSERT (!widget->isVisible()) ;
	CPPUNIT_ASSERT (!widget->isEnabled()) ;
}

void	KBButtonTest::testButtons5 ()
{
	KBButton *button   = getButton  ("Button_3", "button_3") ;

	CPPUNIT_ASSERT (!button->isVisible()) ;
	CPPUNIT_ASSERT (!button->isEnabled()) ;

	KBControl *control = getControl (button ) ;
	QWidget	  *widget  = getWidget  (control) ;

	CPPUNIT_ASSERT (!widget->isVisible()) ;
	CPPUNIT_ASSERT (!widget->isEnabled()) ;

	button->setVisible (true) ;
	button->setEnabled (true) ;

	CPPUNIT_ASSERT ( button->isVisible()) ;
	CPPUNIT_ASSERT ( button->isEnabled()) ;
	CPPUNIT_ASSERT ( widget->isVisible()) ;
	CPPUNIT_ASSERT ( widget->isEnabled()) ;
}

void	KBButtonTest::testButtons6 ()
{
	KBButton	*button  = getButton  ("Button_4", "button_4") ;
	KBControl	*control = getControl (button ) ;
	RKPushButton	*widget  = getWidget  (control) ;

	button->setText ("button_104") ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("button_104")),
		_STDSTR(button->getText())
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("button_104")),
		_STDSTR(widget->text   ())
	)	;

	CPPUNIT_ASSERT
	(	button->setKBProperty ("text", "button_204")
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("button_204")),
		_STDSTR(button->getText())
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("button_204")),
		_STDSTR(widget->text   ())
	)	;
}


CppUnit::Test
	*KBButtonTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Button") ;

	if (rekallTestSkip ("Buttons")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;


	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBButtonTest> ("noDisplay", &KBButtonTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBButtonTest> ("withGUI",   &KBButtonTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBButtonTest>
			(	"buttons0",
				&KBButtonTest::testButtons0
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBButtonTest>
			(	"buttons1",
				&KBButtonTest::testButtons1
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBButtonTest>
			(	"buttons2",
				&KBButtonTest::testButtons2
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBButtonTest>
			(	"buttons3",
				&KBButtonTest::testButtons3
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBButtonTest>
			(	"buttons4",
				&KBButtonTest::testButtons4
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBButtonTest>
			(	"buttons5",
				&KBButtonTest::testButtons5
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBButtonTest>
			(	"buttons6",
				&KBButtonTest::testButtons6
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_API *buttonTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBButtonTest::suite (__settings) ;
}
