/***************************************************************************
    file	         : checks.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qapplication.h>
#include	<qstring.h>
#include	<qmap.h>

#include	"rk_application.h"
#include	"rk_checkbox.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_check.h"
#include	"kb_sizer.h"
#include	"kb_dom.h"
#include	"kb_ctrl.h"

#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBCheckTest : public CppUnit::TestCase
{
	KBDBInfo		*m_dbInfo	;
	TestOpenForm		*m_form		;

	KBCheck			*getCheck	(const char *) ;
	KBControl		*getControl	(KBCheck    *) ;
	KBControl		*getControl	(KBCheck    *, uint, uint) ;
	RKCheckBox		*getWidget	(KBControl  *) ;

public	:

	KBCheckTest	() ;
	KBCheckTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testChecks0	() ;
	void			testChecks1	() ;
	void			testChecks2	() ;
	void			testChecks3	() ;
	void			testChecks4	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBCheckTest::KBCheckTest ()
{
}

KBCheckTest::KBCheckTest
	(	std::string		name
	)
	:
	CppUnit::TestCase (name)
{
}

void	KBCheckTest::setUp ()
{
	KBError	 error	;

	m_dbInfo = rekallDBInfo ("API.rkl", "test/api") ;
	m_form	 = rekallTestOpenForm (m_dbInfo, "api", "Checks", error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;
}

void	KBCheckTest::tearDown ()
{
	rekallTestCloseForm () ;
	delete	m_dbInfo ;
}

void	KBCheckTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBCheckTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}


KBCheck
	*KBCheckTest::getCheck
	(	const char	*name
	)
{
	KBNode	 *target = m_form->m_form->getNamedNode(name) ;

	CPPUNIT_ASSERT (target != 0) ;
	CPPUNIT_ASSERT (target->isA("KBCheck")) ;

	KBCheck *check = (KBCheck *)target ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString(name)),
		_STDSTR(check->getName())
	)	;

	return	check	;
}

KBControl
	*KBCheckTest::getControl
	(	KBCheck		*check
	)
{
	KBControl *control = KBTestAccessor::getControl (check) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA("KBCtrlCheck")) ;
	return	  control  ;
}

KBControl
	*KBCheckTest::getControl
	(	KBCheck		*check,
		uint		qrow,
		uint		drow
	)
{
	KBControl *control = KBTestAccessor::getControl (check, qrow) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA("KBCtrlCheck")) ;
	CPPUNIT_ASSERT (control->getDRow() == drow ) ;
	return	  control  ;
}

RKCheckBox
	*KBCheckTest::getWidget
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;
	CPPUNIT_ASSERT (widget->isA("RKCheckBox")) ;
	return	(RKCheckBox *)widget ;
}

void	KBCheckTest::testChecks0 ()
{
	KBCheck    *check0   = getCheck  ("Check_0") ;

	CPPUNIT_ASSERT(!check0->isChecked (0)) ;
	CPPUNIT_ASSERT(!check0->isChecked (1)) ;
	CPPUNIT_ASSERT( check0->isChecked (2)) ;
}

void	KBCheckTest::testChecks1 ()
{
	KBCheck    *check0   = getCheck  ("Check_0") ;

	rekallGUIPause ("Checks1") ;

	check0->setChecked (0,  true) ;
	check0->setChecked (1,  true) ;
	check0->setChecked (2, false) ;

	rekallGUIPause ("Checks1") ;

	CPPUNIT_ASSERT( check0->isChecked (0)) ;
	CPPUNIT_ASSERT( check0->isChecked (1)) ;
	CPPUNIT_ASSERT(!check0->isChecked (2)) ;
}

void	KBCheckTest::testChecks2 ()
{
	KBCheck    *check0   = getCheck  ("Check_0") ;

	check0->setChecked (0,  true) ;
	check0->setChecked (1, false) ;
	check0->setChecked (2,  true) ;

	CPPUNIT_ASSERT( check0->isChecked (0)) ;
	CPPUNIT_ASSERT(!check0->isChecked (1)) ;
	CPPUNIT_ASSERT( check0->isChecked (2)) ;
}

void	KBCheckTest::testChecks3 ()
{
	KBCheck    *nullint0 = getCheck  ("Nullint_0") ;

	CPPUNIT_ASSERT(!nullint0->isChecked (0)) ;
	CPPUNIT_ASSERT(!nullint0->isChecked (1)) ;
	CPPUNIT_ASSERT(!nullint0->isChecked (2)) ;
	CPPUNIT_ASSERT( nullint0->getValue  (0).isNull ()) ;
	CPPUNIT_ASSERT( nullint0->getValue  (0).isEmpty()) ;
	CPPUNIT_ASSERT(!nullint0->getValue  (0).isTrue ()) ;
}

void	KBCheckTest::testChecks4 ()
{
	KBCheck    *nullint1 = getCheck  ("Nullint_1") ;

	CPPUNIT_ASSERT(!nullint1->isChecked (0)) ;
	CPPUNIT_ASSERT(!nullint1->isChecked (1)) ;
	CPPUNIT_ASSERT(!nullint1->isChecked (2)) ;
	CPPUNIT_ASSERT(!nullint1->getValue  (0).isNull ()) ;
	CPPUNIT_ASSERT(!nullint1->getValue  (0).isEmpty()) ;
	CPPUNIT_ASSERT(!nullint1->getValue  (0).isTrue ()) ;
}

CppUnit::Test
	*KBCheckTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Check") ;

	if (rekallTestSkip ("Checks")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;


	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBCheckTest> ("noDisplay", &KBCheckTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBCheckTest> ("withGUI",   &KBCheckTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBCheckTest>
			(	"checks0",
				&KBCheckTest::testChecks0
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBCheckTest>
			(	"checks1",
				&KBCheckTest::testChecks1
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBCheckTest>
			(	"checks2",
				&KBCheckTest::testChecks2
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBCheckTest>
			(	"checks3",
				&KBCheckTest::testChecks3
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBCheckTest>
			(	"checks4",
				&KBCheckTest::testChecks4
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_API *checkTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBCheckTest::suite (__settings) ;
}
