/***************************************************************************
    file	         : choices.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qapplication.h>
#include	<qstring.h>
#include	<qmap.h>

#include	"rk_application.h"
#include	"rk_combobox.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_choice.h"
#include	"kb_ctrl.h"
#include	"kb_dom.h"

#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"

#include	"api.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBChoiceTest : public KBAPITest
{
public	:

	KBChoiceTest	() ;
	KBChoiceTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			testChoices0	() ;
	void			testChoices1	() ;
	void			testChoices2	() ;
	void			testChoices3	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;


static	const char *stringList1 [] =
{
	"",
	"A",
	"B",
	"C",
	"D"
}	;

static	const char *stringList2 [] =
{
	"",
	"1",
	"2",
	"3",
	"4"
}	;

KBChoiceTest::KBChoiceTest ()
{
}

KBChoiceTest::KBChoiceTest
	(	std::string		name
	)
	:
	KBAPITest	(name)
{
}

void	KBChoiceTest::setUp ()
{
	openForm ("Choices")	;
}

void	KBChoiceTest::tearDown ()
{
	rekallTestCloseForm() ;
}


void	KBChoiceTest::testChoices0 ()
{
	KBChoice	*choice0  = getChoice    ("Choice_0", "Choice_0") ;
	KBControl	*control0 = getControl	 (choice0 ) ;
	RKComboBox	*combo0	  = getRKComboBox(control0) ;

	QStringList	strings	= choice0->getValues () ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	"Unexpected entry count",
		5,
		(int)strings.count()
	)	;

	for (uint idx1 = 0 ; idx1 < 5 ; idx1 += 1)
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected text at offset %1").arg(idx1)),
			_STDSTR(QString(stringList1[idx1])),
			_STDSTR(strings[idx1])
		)	;

	for (uint idx2 = 0 ; idx2 < 5 ; idx2 += 1)
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected control value at offset %1").arg(idx2)),
			_STDSTR(QString(stringList1[idx2])),
			_STDSTR(combo0->text(idx2))
		)	;
}

void	KBChoiceTest::testChoices1 ()
{
	KBChoice	*choice1  = getChoice    ("Choice_1", "Choice_1") ;
	KBControl	*control1 = getControl   (choice1 ) ;
	RKComboBox	*combo1	  = getRKComboBox(control1) ;

	QStringList	strings	  = choice1->getValues () ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	"Unexpected entry count",
		4,
		(int)strings.count()
	)	;

	for (uint idx1 = 0 ; idx1 < 4 ; idx1 += 1)
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected text at offset %1").arg(idx1)),
			_STDSTR(QString(stringList1[idx1+1])),
			_STDSTR(strings[idx1])
		)	;

	for (uint idx2 = 0 ; idx2 < 4 ; idx2 += 1)
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected control value at offset %1").arg(idx2)),
			_STDSTR(QString(stringList1[idx2+1])),
			_STDSTR(combo1->text(idx2))
		)	;
}

void	KBChoiceTest::testChoices2 ()
{
	KBChoice	*choice0  = getChoice    ("Choice_0", "Choice_0") ;
	KBControl	*control0 = getControl   (choice0 ) ;
	RKComboBox	*combo0	  = getRKComboBox(control0) ;

	QStringList	strings	  ;

	for (uint idx0 = 1 ; idx0 < 5 ; idx0 += 1)
		strings.append (stringList2[idx0]) ;
	choice0->setValues (strings) ;

	strings	  = choice0->getValues () ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	"Unexpected entry count",
		5,
		(int)strings.count()
	)	;

	for (uint idx1 = 0 ; idx1 < 5 ; idx1 += 1)
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected text at offset %1").arg(idx1)),
			_STDSTR(QString(stringList2[idx1])),
			_STDSTR(strings[idx1])
		)	;

	for (uint idx2 = 0 ; idx2 < 5 ; idx2 += 1)
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected control value at offset %1").arg(idx2)),
			_STDSTR(QString(stringList2[idx2])),
			_STDSTR(combo0->text(idx2))
		)	;
}

void	KBChoiceTest::testChoices3 ()
{
	KBChoice	*choice1  = getChoice    ("Choice_1", "Choice_1") ;
	KBControl	*control1 = getControl   (choice1 ) ;
	RKComboBox	*combo1	  = getRKComboBox(control1) ;

	QStringList	strings	  ;

	for (uint idx0 = 1 ; idx0 < 5 ; idx0 += 1)
		strings.append (stringList2[idx0]) ;
	choice1->setValues (strings) ;

	strings	  = choice1->getValues () ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	"Unexpected entry count",
		4,
		(int)strings.count()
	)	;

	for (uint idx1 = 0 ; idx1 < 4 ; idx1 += 1)
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected text at offset %1").arg(idx1)),
			_STDSTR(QString(stringList2[idx1+1])),
			_STDSTR(strings[idx1])
		)	;

	for (uint idx2 = 0 ; idx2 < 4 ; idx2 += 1)
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected control value at offset %1").arg(idx2)),
			_STDSTR(QString(stringList2[idx2+1])),
			_STDSTR(combo1->text(idx2))
		)	;
}

CppUnit::Test
	*KBChoiceTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Choices") ;

	if (rekallTestSkip ("Choices")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;


	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBChoiceTest> ("noDisplay", &KBChoiceTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBChoiceTest> ("withGUI",   &KBChoiceTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBChoiceTest>
			(	"choices0",
				&KBChoiceTest::testChoices0
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBChoiceTest>
			(	"choices1",
				&KBChoiceTest::testChoices1
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBChoiceTest>
			(	"choices2",
				&KBChoiceTest::testChoices2
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBChoiceTest>
			(	"choices3",
				&KBChoiceTest::testChoices3
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_API *choiceTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBChoiceTest::suite (__settings) ;
}
