/***************************************************************************
    file	         : containers.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qmainwindow.h>
#include	<qstring.h>
#include	<qmap.h>
#include	<qregexp.h>

#include	"rk_application.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_sizer.h"
#include	"kb_dom.h"

#include	"tester.h"
#include	"tester_kbase.h"


#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBContainerTest : public CppUnit::TestCase
{
	TestOpenForm		*m_form		;

public	:

	KBContainerTest	() ;
	KBContainerTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testContainers1	() ;
	void			testContainers2	() ;
	void			testContainers3	() ;
	void			testContainers4	() ;
	void			testContainers5	() ;
	void			testContainers6	() ;
	void			testContainers7	() ;
	void			testContainers8	() ;
	void			testContainers9	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBContainerTest::KBContainerTest ()
{
}

KBContainerTest::KBContainerTest
	(	std::string		name
	)
	:
	CppUnit::TestCase (name)
{
}

void	KBContainerTest::setUp ()
{
	KBError	error	;
	m_form	= rekallTestOpenForm (0, "api", "Containers", error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;
}

void	KBContainerTest::tearDown ()
{
	rekallTestCloseForm () ;
}

void	KBContainerTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBContainerTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

void	KBContainerTest::testContainers1 ()
{
	KBNode	*target	= m_form->m_form->getNamedNode("Container_1") ;

	CPPUNIT_ASSERT (target != 0) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Container_1")),
		_STDSTR(target->getName())
	)	;
}

void	KBContainerTest::testContainers2 ()
{
	KBNode	*target	= m_form->m_form->getNamedNode("Container_1/Button_1") ;

	CPPUNIT_ASSERT (target != 0) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Button_1")),
		_STDSTR(target->getName())
	)	;
}

void	KBContainerTest::testContainers3 ()
{
	KBNode	*target	= m_form->m_form->getNamedNode("Container_1/Button_1/..") ;

	CPPUNIT_ASSERT (target != 0) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Container_1")),
		_STDSTR(target->getName())
	)	;
}

void	KBContainerTest::testContainers4 ()
{
	KBNode	*target	= m_form->m_form->getNamedNode("Container_1/Button_0") ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(target == 0 ? QString::null : "Got: " + target->getName()),
		target == 0
	)	;
}

void	KBContainerTest::testContainers5 ()
{
	KBNode	*target	= m_form->m_form->getNamedNode("Container_0/Button_0") ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(target == 0 ? QString::null : "Got: " + target->getName()),
		target == 0
	)	;
}

void	KBContainerTest::testContainers6 ()
{
	KBNode	*target	= m_form->m_form->getNamedNode("getRoot()") ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(target == 0 ? QString("No target") : "Got: " + target->getName()),
		target == m_form->m_form
	)	;
}

void	KBContainerTest::testContainers7 ()
{
	KBNode	*target	= m_form->m_form->getNamedNode("getBlock()") ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(target == 0 ? QString::null : "Got: " + target->getName()),
		target == 0
	)	;
}

void	KBContainerTest::testContainers8 ()
{
	KBNode	*target	= m_form->m_form->getNamedNode("Container_1/getRoot()") ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(target == 0 ? QString("No target") : "Got: " + target->getName()),
		target == m_form->m_form
	)	;
}

void	KBContainerTest::testContainers9 ()
{
	KBNode	*target	= m_form->m_form->getNamedNode("Container_1/getBlock()") ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(target == 0 ? QString("No target") : "Got: " + target->getName()),
		target == m_form->m_form
	)	;
}


CppUnit::Test
	*KBContainerTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Container") ;

	if (rekallTestSkip ("Containers")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;

	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBContainerTest> ("noDisplay", &KBContainerTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBContainerTest> ("withGUI",   &KBContainerTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBContainerTest>
			(	"containers1",
				&KBContainerTest::testContainers1
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBContainerTest>
			(	"containers2",
				&KBContainerTest::testContainers2
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBContainerTest>
			(	"containers3",
				&KBContainerTest::testContainers3
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBContainerTest>
			(	"containers4",
				&KBContainerTest::testContainers4
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBContainerTest>
			(	"containers5",
				&KBContainerTest::testContainers5
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBContainerTest>
			(	"containers6",
				&KBContainerTest::testContainers6
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBContainerTest>
			(	"containers7",
				&KBContainerTest::testContainers7
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBContainerTest>
			(	"containers8",
				&KBContainerTest::testContainers8
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBContainerTest>
			(	"containers9",
				&KBContainerTest::testContainers9
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_API *containerTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBContainerTest::suite (__settings) ;
}
