/***************************************************************************
    file	         : frames.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qstring.h>
#include	<qmap.h>
#include	<qlabel.h>

#include	"rk_application.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_label.h"
#include	"kb_memo.h"
#include	"kb_richtext.h"
#include	"kb_dom.h"

#include	"rk_textedit.h"
#include	"rk_textbrowser.h"


#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"

#include	"api.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif

#define	CHECK_FRAME(widget,shape,shadow,width) \
	\
	CPPUNIT_ASSERT_EQUAL_MESSAGE															\
	(	_STDSTR(QString("Frame   : got %1, expected %2").arg(frameShape ((int)(widget)->frameShape ())).arg(frameShape ((int)(shape )))),	\
		(int)(shape),																\
		(int)(widget)->frameShape ()														\
	)	;																	\
	CPPUNIT_ASSERT_EQUAL_MESSAGE															\
	(	_STDSTR(QString("Shadow  : got %1, expected %2").arg(frameShadow((int)(widget)->frameShadow())).arg(frameShadow((int)(shadow)))),	\
		(int)(shadow),																\
		(int)(widget)->frameShadow()														\
	)	;																	\
	CPPUNIT_ASSERT_EQUAL_MESSAGE															\
	(	_STDSTR(QString("Width   : got %1, expected %2").arg((int)(widget)->frameWidth ()).arg((int)(width))),					\
		(int)(width),																\
		(int)(widget)->frameWidth ()														\
	)	; 


class	KBFrameTest : public KBAPITest
{
public	:

	KBFrameTest	() ;
	KBFrameTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;


	void			testFrames0	() ;
	void			testFrames1	() ;
	void			testFrames2	() ;
	void			testFrames3	() ;
	void			testFrames4	() ;
	void			testFrames5	() ;
	void			testFrames6	() ;
	void			testFrames7	() ;
	void			testFrames8	() ;
	void			testFrames9	() ;
	void			testFrames10	() ;
	void			testFrames11	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;


static	QString	frameShadow
	(	int		value
	)
{
	switch (value & QFrame::MShadow)
	{
		case QFrame::Plain	: return "Plain"  ;
		case QFrame::Raised	: return "Raised" ;
		case QFrame::Sunken	: return "Sunken" ;
		default			: break	 ;
	}

	QString	s ;
	s.sprintf("%04x", value & QFrame::MShadow) ;
	return	s ;
}

static	QString	frameShape
	(	int		value
	)
{
	switch (value & QFrame::MShape)
	{
		case QFrame::NoFrame		: return "NoFrame"		;
		case QFrame::Box		: return "Box"			;
		case QFrame::Panel		: return "Panel"		;
		case QFrame::WinPanel		: return "WinPanel"		;
		case QFrame::HLine		: return "HLine"		;
		case QFrame::VLine		: return "VLine"		;
		case QFrame::StyledPanel	: return "StyledPanel"		;
		case QFrame::PopupPanel		: return "PopupPanel"		;
		case QFrame::MenuBarPanel	: return "MenuBarPanel"		;
		case QFrame::ToolBarPanel	: return "ToolBarPanel"		;
		case QFrame::LineEditPanel	: return "LineEditPanel"	;
		case QFrame::TabWidgetPanel	: return "TabWidgetPanel"	;
		case QFrame::GroupBoxPanel	: return "GroupBoxPanel"	;
		default				: break	 ;
	}

	QString	s ;
	s.sprintf("%04x", value & QFrame::MShape) ;
	return	s ;
}

KBFrameTest::KBFrameTest ()
{
}

KBFrameTest::KBFrameTest
	(	std::string		name
	)
	:
	KBAPITest (name)
{
}

void	KBFrameTest::setUp ()
{
	openForm ("Frames_Align") ;
}

void	KBFrameTest::tearDown ()
{
	rekallTestCloseForm() ;
}

void	KBFrameTest::testFrames0 ()
{
	KBLabel   *label   = getLabel   ("Label_0", "Label_0") ;
	KBControl *control = getControl (label  ) ;
	QLabel	  *qlabel  = getQLabel  (control) ;

	CHECK_FRAME(qlabel,0,0,0) ;
}

void	KBFrameTest::testFrames1 ()
{
	KBLabel *label = getLabel ("Label_1", "Label_1") ;
	KBControl *control = getControl (label  ) ;
	QLabel	  *qlabel  = getQLabel  (control) ;

	CHECK_FRAME(qlabel,QFrame::Panel,QFrame::Raised,4) ;
}

void	KBFrameTest::testFrames2 ()
{
	KBLabel *label = getLabel ("Label_2", "Label_2") ;
	KBControl *control = getControl (label  ) ;
	QLabel	  *qlabel  = getQLabel  (control) ;

	CHECK_FRAME(qlabel,QFrame::WinPanel,QFrame::Sunken,2) ;
}

void	KBFrameTest::testFrames3 ()
{
	KBMemo    *memo    = getMemo      ("Memo_0", "Memo_0") ;
	KBControl *control = getControl   (memo   ) ;
	RKTextEdit*qtexted = getRKTextEdit(control) ;

	CHECK_FRAME(qtexted,0,QFrame::Plain,0) ;
}

void	KBFrameTest::testFrames4 ()
{
	KBMemo    *memo     = getMemo      ("Memo_1", "Memo_1") ;
	KBControl *control  = getControl   (memo   ) ;
	RKTextEdit*qtexted  = getRKTextEdit(control) ;

	CHECK_FRAME(qtexted,QFrame::Box,QFrame::Raised,2) ;
}

void	KBFrameTest::testFrames5 ()
{
	KBMemo     *memo     = getMemo      ("Memo_2", "Memo_2") ;
	KBControl  *control  = getControl   (memo   ) ;
	RKTextEdit *qtexted  = getRKTextEdit(control) ;

	CHECK_FRAME(qtexted,QFrame::Box,QFrame::Sunken,4) ;
}

void	KBFrameTest::testFrames6 ()
{
	KBRichText *rt      = getRichText  ("RichText_0", "RichText_0") ;
	KBControl  *control = getControl   (rt     ) ;
	RKTextBrowser *qtextbr = getRKTextBrowser(control) ;

	CHECK_FRAME(qtextbr,0,QFrame::Plain,0) ;
}

void	KBFrameTest::testFrames7 ()
{
	KBRichText *rt       = getRichText  ("RichText_1", "RichText_1") ;
	KBControl  *control  = getControl   (rt     ) ;
	RKTextBrowser *qtextbr  = getRKTextBrowser(control) ;

	CHECK_FRAME(qtextbr,QFrame::StyledPanel,QFrame::Raised,1) ;
}

void	KBFrameTest::testFrames8 ()
{
	KBRichText *rt       = getRichText  ("RichText_2", "RichText_2") ;
	KBControl  *control  = getControl   (rt     ) ;
	RKTextBrowser *qtextbr  = getRKTextBrowser(control) ;

	CHECK_FRAME(qtextbr,QFrame::StyledPanel,QFrame::Sunken,1) ;
}

void	KBFrameTest::testFrames9 ()
{
	KBField    *field    = getField     ("Field_0", "Field_0") ;
	KBControl  *control  = getControl   (field  ) ;
	RKLineEdit *lineed   = getRKLineEdit(control) ;

	CHECK_FRAME(lineed,0,QFrame::Plain,0) ;
}

void	KBFrameTest::testFrames10 ()
{
	KBField    *field    = getField     ("Field_1", "Field_1") ;
	KBControl  *control  = getControl   (field  ) ;
	RKLineEdit *lineed   = getRKLineEdit(control) ;

	CHECK_FRAME(lineed,QFrame::Box,QFrame::Raised,4) ;
}

void	KBFrameTest::testFrames11 ()
{
	KBField    *field    = getField     ("Field_2", "Field_2") ;
	KBControl  *control  = getControl   (field  ) ;
	RKLineEdit *lineed   = getRKLineEdit(control) ;

	CHECK_FRAME(lineed,QFrame::Panel,QFrame::Plain,1) ;
}

CppUnit::Test
	*KBFrameTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Frames") ;

	if (rekallTestSkip ("Frames")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;


	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBFrameTest> ("noDisplay", &KBFrameTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBFrameTest> ("withGUI",   &KBFrameTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBFrameTest>
			(	"frames0",
				&KBFrameTest::testFrames0
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBFrameTest>
			(	"frames1",
				&KBFrameTest::testFrames1
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBFrameTest>
			(	"frames2",
				&KBFrameTest::testFrames2
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBFrameTest>
			(	"frames3",
				&KBFrameTest::testFrames3
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBFrameTest>
			(	"frames4",
				&KBFrameTest::testFrames4
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBFrameTest>
			(	"frames5",
				&KBFrameTest::testFrames5
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBFrameTest>
			(	"frames6",
				&KBFrameTest::testFrames6

		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBFrameTest>
			(	"frames7",
				&KBFrameTest::testFrames7
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBFrameTest>
			(	"frames8",
				&KBFrameTest::testFrames8
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBFrameTest>
			(	"frames9",
				&KBFrameTest::testFrames9

		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBFrameTest>
			(	"frames10",
				&KBFrameTest::testFrames10
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBFrameTest>
			(	"frames11",
				&KBFrameTest::testFrames11
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_API *framesTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBFrameTest::suite (__settings) ;
}
