/***************************************************************************
    file	         : global.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qmainwindow.h>
#include	<qstring.h>
#include	<qmap.h>
#include	<qregexp.h>

#include	"rk_application.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_dom.h"

#include	"tester.h"
#include	"tester_kbase.h"


#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBGlobalNodeTest : public CppUnit::TestCase
{
	TestOpenForm		*m_form		;

public	:

	KBGlobalNodeTest	() ;
	KBGlobalNodeTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testMissing	() ;
	void			testContainer1	() ;
	void			testContainer2	() ;
	void			testButton1	() ;
	void			testButton2	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBGlobalNodeTest::KBGlobalNodeTest ()
{
}

KBGlobalNodeTest::KBGlobalNodeTest
	(	std::string		name
	)
	:
	CppUnit::TestCase (name)
{
}

void	KBGlobalNodeTest::setUp ()
{
	KBError	error	;
	m_form	= rekallTestOpenForm (0, "api", "Containers", error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;
}

void	KBGlobalNodeTest::tearDown ()
{
	rekallTestCloseForm () ;
}

void	KBGlobalNodeTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBGlobalNodeTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

void	KBGlobalNodeTest::testMissing ()
{
	KBNode	*target	= m_form->m_form->getGlobalNode("Missing") ;
	CPPUNIT_ASSERT (target == 0) ;
}

void	KBGlobalNodeTest::testContainer1 ()
{
	KBNode	*target	= m_form->m_form->getGlobalNode("Container_1") ;

	CPPUNIT_ASSERT (target != 0) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Container_1")),
		_STDSTR(target->getName())
	)	;
}

void	KBGlobalNodeTest::testContainer2 ()
{
	KBNode	*target	= m_form->m_form->getGlobalNode("Container_2") ;

	CPPUNIT_ASSERT (target != 0) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Container_2")),
		_STDSTR(target->getName())
	)	;
}


void	KBGlobalNodeTest::testButton1 ()
{
	KBNode	*target	= m_form->m_form->getGlobalNode("Button_1") ;

	CPPUNIT_ASSERT (target != 0) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Button_1")),
		_STDSTR(target->getName())
	)	;
}

void	KBGlobalNodeTest::testButton2 ()
{
	KBNode	*target	= m_form->m_form->getGlobalNode("Button_2") ;

	CPPUNIT_ASSERT (target != 0) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Button_2")),
		_STDSTR(target->getName())
	)	;
}


CppUnit::Test
	*KBGlobalNodeTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("GlobalNode") ;

	if (rekallTestSkip ("Global")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;

	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBGlobalNodeTest> ("noDisplay", &KBGlobalNodeTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBGlobalNodeTest> ("withGUI",   &KBGlobalNodeTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBGlobalNodeTest>
			(	"missing",
				&KBGlobalNodeTest::testMissing
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBGlobalNodeTest>
			(	"container1",
				&KBGlobalNodeTest::testContainer1
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBGlobalNodeTest>
			(	"container2",
				&KBGlobalNodeTest::testContainer2
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBGlobalNodeTest>
			(	"button1",
				&KBGlobalNodeTest::testButton1
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBGlobalNodeTest>
			(	"button2",
				&KBGlobalNodeTest::testButton2
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_API *globalNodeTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBGlobalNodeTest::suite (__settings) ;
}
