/***************************************************************************
    file	         : labels.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qstring.h>
#include	<qmap.h>
#include	<qlabel.h>

#include	"rk_application.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_label.h"
#include	"kb_dom.h"

#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"

#include	"api.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBLabelTest : public KBAPITest
{
public	:

	KBLabelTest	() ;
	KBLabelTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;


	void			testLabels0	() ;
	void			testLabels1	() ;
	void			testLabels2	() ;
	void			testLabels3	() ;
	void			testLabels4	() ;
	void			testLabels5	() ;
	void			testLabels6	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBLabelTest::KBLabelTest ()
{
}

KBLabelTest::KBLabelTest
	(	std::string		name
	)
	:
	KBAPITest (name)
{
}

void	KBLabelTest::setUp ()
{
	openForm ("Labels") ;
}

void	KBLabelTest::tearDown ()
{
	rekallTestCloseForm () ;
}

void	KBLabelTest::testLabels0 ()
{
	KBLabel *label   = getLabel  ("Label_0", "Label_0") ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("label_0")),
		_STDSTR(label->getText())
	)	;

	CPPUNIT_ASSERT ( label->isVisible()) ;
	CPPUNIT_ASSERT ( label->isEnabled()) ;

	KBControl *control = getControl (label  ) ;
	QLabel	  *qlabel  = getQLabel  (control) ;

	CPPUNIT_ASSERT ( qlabel->isVisible()) ;
	CPPUNIT_ASSERT ( qlabel->isEnabled()) ;
}

void	KBLabelTest::testLabels1 ()
{
	KBLabel *label = getLabel ("Label_1", "Label_1") ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("label_1")),
		_STDSTR(label->getText())
	)	;

	CPPUNIT_ASSERT ( label->isVisible()) ;
	CPPUNIT_ASSERT (!label->isEnabled()) ;

	KBControl *control = getControl (label ) ;
	QLabel	  *qlabel  = getQLabel  (control) ;

	CPPUNIT_ASSERT ( qlabel->isVisible()) ;
	CPPUNIT_ASSERT (!qlabel->isEnabled()) ;
}

void	KBLabelTest::testLabels2 ()
{
	KBLabel *label = getLabel ("Label_2", "Label_2") ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("label_2")),
		_STDSTR(label->getText())
	)	;

	CPPUNIT_ASSERT (!label->isVisible()) ;
	CPPUNIT_ASSERT ( label->isEnabled()) ;

	KBControl *control = getControl (label ) ;
	QLabel	  *qlabel  = getQLabel  (control) ;

	CPPUNIT_ASSERT (!qlabel->isVisible()) ;
	CPPUNIT_ASSERT ( qlabel->isEnabled()) ;
}

void	KBLabelTest::testLabels3 ()
{
	KBLabel *label = getLabel ("Label_3", "Label_3") ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("label_3")),
		_STDSTR(label->getText())
	)	;

	CPPUNIT_ASSERT (!label->isVisible()) ;
	CPPUNIT_ASSERT (!label->isEnabled()) ;

	KBControl *control = getControl (label ) ;
	QLabel	  *qlabel  = getQLabel  (control) ;

	CPPUNIT_ASSERT (!qlabel->isVisible()) ;
	CPPUNIT_ASSERT (!qlabel->isEnabled()) ;
}

void	KBLabelTest::testLabels4 ()
{
	KBLabel *label   = getLabel  ("Label_0", "Label_0") ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("label_0")),
		_STDSTR(label->getText())
	)	;

	CPPUNIT_ASSERT ( label->isVisible()) ;
	CPPUNIT_ASSERT ( label->isEnabled()) ;

	KBControl *control = getControl (label ) ;
	QLabel	  *qlabel  = getQLabel  (control) ;

	CPPUNIT_ASSERT ( qlabel->isVisible()) ;
	CPPUNIT_ASSERT ( qlabel->isEnabled()) ;

	label->setVisible (false) ;
	label->setEnabled (false) ;

	CPPUNIT_ASSERT (!label ->isVisible()) ;
	CPPUNIT_ASSERT (!label ->isEnabled()) ;
	CPPUNIT_ASSERT (!qlabel->isVisible()) ;
	CPPUNIT_ASSERT (!qlabel->isEnabled()) ;
}

void	KBLabelTest::testLabels5 ()
{
	KBLabel *label   = getLabel  ("Label_3", "Label_3") ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("label_3")),
		_STDSTR(label->getText())
	)	;

	CPPUNIT_ASSERT (!label->isVisible()) ;
	CPPUNIT_ASSERT (!label->isEnabled()) ;

	KBControl *control = getControl (label ) ;
	QLabel	  *qlabel  = getQLabel  (control) ;

	CPPUNIT_ASSERT (!qlabel->isVisible()) ;
	CPPUNIT_ASSERT (!qlabel->isEnabled()) ;

	label->setVisible (true) ;
	label->setEnabled (true) ;

	CPPUNIT_ASSERT ( label ->isVisible()) ;
	CPPUNIT_ASSERT ( label ->isEnabled()) ;
	CPPUNIT_ASSERT ( qlabel->isVisible()) ;
	CPPUNIT_ASSERT ( qlabel->isEnabled()) ;
}

void	KBLabelTest::testLabels6 ()
{
	KBLabel		*label   = getLabel   ("Label_4", "Label_4") ;
	KBControl	*control = getControl (label  ) ;
	QLabel		*qlabel  = getQLabel  (control) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("label_4")),
		_STDSTR(label->getText())
	)	;

	label->setText ("label_104") ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("label_104")),
		_STDSTR(label->getText())
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("label_104")),
		_STDSTR(qlabel->text   ())
	)	;

	CPPUNIT_ASSERT
	(	label->setKBProperty ("text", "label_204")
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("label_204")),
		_STDSTR(label->getText())
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("label_204")),
		_STDSTR(qlabel->text   ())
	)	;
}

CppUnit::Test
	*KBLabelTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Label") ;

	if (rekallTestSkip ("Labels")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;


	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBLabelTest> ("noDisplay", &KBLabelTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBLabelTest> ("withGUI",   &KBLabelTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBLabelTest>
			(	"labels0",
				&KBLabelTest::testLabels0
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBLabelTest>
			(	"labels1",
				&KBLabelTest::testLabels1
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBLabelTest>
			(	"labels2",
				&KBLabelTest::testLabels2
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBLabelTest>
			(	"labels3",
				&KBLabelTest::testLabels3
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBLabelTest>
			(	"labels4",
				&KBLabelTest::testLabels4
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBLabelTest>
			(	"labels5",
				&KBLabelTest::testLabels5
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBLabelTest>
			(	"labels6",
				&KBLabelTest::testLabels6
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_API *labelTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBLabelTest::suite (__settings) ;
}
