/***************************************************************************
    file	         : links.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qapplication.h>
#include	<qstring.h>
#include	<qmap.h>

#include	"rk_application.h"
#include	"rk_combobox.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_link.h"
#include	"kb_dom.h"

#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBLinkTest : public CppUnit::TestCase
{
	TestOpenForm		*m_form		;

	KBLink			*getLink	(const char *) ;
	KBControl		*getControl	(KBLink    *) ;
	KBControl		*getControl	(KBLink    *, uint, uint) ;
	RKComboBox		*getWidget	(KBControl  *) ;

public	:

	KBLinkTest	() ;
	KBLinkTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testLinks0	() ;
	void			testLinks1	() ;
	void			testLinks2	() ;
	void			testLinks3	() ;
	void			testLinks5	() ;
	void			testLinks6	() ;
	void			testLinks7	() ;
	void			testLinks8	() ;
	void			testLinks9	() ;
	void			testLinks10	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

static	const char *values_n[] =
{
	"1",
	"2",
	"3",
	"4",
	"5",
	"6",
	"7",
	"8",
	"9"
}	;

static	const char *strings_n[] =
{
	"One",
	"Two",
	"Three",
	"Four",
	"Five",
	"Six",
	"Seven",
	"Eight",
	"Nine"
}	;

KBLinkTest::KBLinkTest ()
{
}

KBLinkTest::KBLinkTest
	(	std::string		name
	)
	:
	CppUnit::TestCase (name)
{
}

void	KBLinkTest::setUp ()
{
	KBDomDocument	doc	;
	QDomElement	info	;
	if (!doc.loadFile ("API.rkl", "test/api", getenv("TESTPATH")))
	{
		fprintf
		(	stderr,
			"links: %s: cannot open: %s\n",
			"API.xml",
			doc.lastError().getMessage().latin1()
		)	;
		exit	(1) ;
	}
	DOMITER_BEGIN(doc.documentElement(),"serverinfo",elem)
	{
		info	= elem	;
	}
	DOMITER_END

	KBDBInfo	*dbInfo	= new KBDBInfo     ("noname") ;
	KBServerInfo	*svInfo	= new KBServerInfo (dbInfo, info) ;

	dbInfo->insert	(svInfo);

	KBError	error	;
	m_form	= rekallTestOpenForm (dbInfo, "api", "Links", error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;
}

void	KBLinkTest::tearDown ()
{
	rekallTestCloseForm () ;
}

void	KBLinkTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBLinkTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

KBLink
	*KBLinkTest::getLink
	(	const char	*name
	)
{
	KBNode	 *target = m_form->m_form->getNamedNode(name) ;

	CPPUNIT_ASSERT (target != 0) ;
	CPPUNIT_ASSERT (target->isA("KBLink")) ;

	KBLink *link = (KBLink *)target ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString(name)),
		_STDSTR(link->getName())
	)	;

	return	link	;
}

KBControl
	*KBLinkTest::getControl
	(	KBLink		*link
	)
{
	KBControl *control = KBTestAccessor::getControl (link) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA("KBCtrlLink")) ;
	return	  control  ;
}

KBControl
	*KBLinkTest::getControl
	(	KBLink		*link,
		uint		qrow,
		uint		drow
	)
{
	KBControl *control = KBTestAccessor::getControl (link, qrow) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA("KBCtrlLink")) ;
	CPPUNIT_ASSERT (control->getDRow() == drow ) ;
	return	  control  ;
}

RKComboBox
	*KBLinkTest::getWidget
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;
	CPPUNIT_ASSERT (widget->isA("RKComboBox")) ;
	return	(RKComboBox *)widget ;
}

void	KBLinkTest::testLinks0 ()
{
	KBLink *link0  = getLink  ("Link_0") ;
	KBLink *link1  = getLink  ("Link_1") ;
	KBLink *link2  = getLink  ("Link_2") ;
	KBLink *link3  = getLink  ("Link_3") ;

	CPPUNIT_ASSERT ( link0->isVisible(0)) ;
	CPPUNIT_ASSERT ( link0->isEnabled(0)) ;
	CPPUNIT_ASSERT ( link1->isVisible(0)) ;
	CPPUNIT_ASSERT (!link1->isEnabled(0)) ;
	CPPUNIT_ASSERT (!link2->isVisible(0)) ;
	CPPUNIT_ASSERT ( link2->isEnabled(0)) ;
	CPPUNIT_ASSERT (!link3->isVisible(0)) ;
	CPPUNIT_ASSERT (!link3->isEnabled(0)) ;

	KBControl  *control0 = getControl (link0  ) ;
	RKComboBox *widget0  = getWidget  (control0) ;
	KBControl  *control1 = getControl (link1  ) ;
	RKComboBox *widget1  = getWidget  (control1) ;
	KBControl  *control2 = getControl (link2  ) ;
	RKComboBox *widget2  = getWidget  (control2) ;
	KBControl  *control3 = getControl (link3  ) ;
	RKComboBox *widget3  = getWidget  (control3) ;

	CPPUNIT_ASSERT ( widget0->isVisible()) ;
	CPPUNIT_ASSERT ( widget0->isEnabled()) ;
	CPPUNIT_ASSERT ( widget1->isVisible()) ;
	CPPUNIT_ASSERT (!widget1->isEnabled()) ;
	CPPUNIT_ASSERT (!widget2->isVisible()) ;
	CPPUNIT_ASSERT ( widget2->isEnabled()) ;
	CPPUNIT_ASSERT (!widget3->isVisible()) ;
	CPPUNIT_ASSERT (!widget3->isEnabled()) ;

	KBControl  *drow00   = getControl (link0, 0, 0) ;
	KBControl  *drow10   = getControl (link1, 0, 0) ;
	KBControl  *drow20   = getControl (link2, 0, 0) ;
	KBControl  *drow30   = getControl (link3, 0, 0) ;

	CPPUNIT_ASSERT (drow00 == control0) ;
	CPPUNIT_ASSERT (drow10 == control1) ;
	CPPUNIT_ASSERT (drow20 == control2) ;
	CPPUNIT_ASSERT (drow30 == control3) ;
}

void	KBLinkTest::testLinks1 ()
{
	KBLink *link   = getLink  ("Link_0") ;
	uint	drows	 = m_form->m_form->rowsInBlock() ;

	for (uint row = 0 ; row < drows ; row += 1)
	{
		KBControl  *control = getControl (link, row, row) ;
		RKComboBox  *widget  = getWidget  (control) ;

		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected widget value: row %1").arg(row)),
			(int)(row + 1),
			(int)widget->currentItem()
		)	;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected control text: row %1").arg(row)),
			_STDSTR(QString::number(row + 1)),
			_STDSTR(control->getValue().getRawText())
		)	;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected link text: row %1").arg(row)),
			_STDSTR(QString::number(row + 1)),
			_STDSTR(link  ->getValue(row).getRawText())
		)	;
	}

}

void	KBLinkTest::testLinks2 ()
{
	KBLink *link   = getLink  ("Link_1") ;
	uint	drows	 = m_form->m_form->rowsInBlock() ;

	for (uint row = 0 ; row < drows ; row += 1)
	{
		KBControl  *control = getControl (link, row, row) ;
		RKComboBox *widget  = getWidget  (control) ;

		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected widget value: row %1").arg(row)),
			(int)(row + 1),
			(int)widget->currentItem()
		)	;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected control text: row %1").arg(row)),
			_STDSTR(QString(values_n[row])),
			_STDSTR(control->getValue().getRawText())
		)	;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected link text: row %1").arg(row)),
			_STDSTR(QString(values_n[row])),
			_STDSTR(link  ->getValue(row).getRawText())
		)	;
	}

}

void	KBLinkTest::testLinks3 ()
{
	KBLink *link   = getLink  ("Link_2") ;
	uint	drows	 = m_form->m_form->rowsInBlock() ;

	for (uint row = 0 ; row < drows ; row += 1)
	{
		KBControl  *control = getControl (link, row, row) ;
		RKComboBox *widget  = getWidget  (control) ;

		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected widget value: row %1").arg(row)),
			(int)(row + 1),
			(int)widget->currentItem()
		)	;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected control text: row %1").arg(row)),
			_STDSTR(QString::number(row + 1)),
			_STDSTR(control->getValue().getRawText())
		)	;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected link text: row %1").arg(row)),
			_STDSTR(QString::number(row + 1)),
			_STDSTR(link  ->getValue(row).getRawText())
		)	;
	}

}

void	KBLinkTest::testLinks5 ()
{
	KBLink *link   = getLink  ("Link_0") ;
	int	width	 = m_form->m_widget->width () ;
	int	height	 = m_form->m_widget->height() ;

	rekallGUIPause	   ("testLinks5") ;
	uint	drows1	 = m_form->m_form->rowsInBlock() ;

	m_form->m_widget->resize (width, height + 20 * 2) ;
	rekallGUIPause	   ("testLinks5") ;

	uint	drows2	 = m_form->m_form->rowsInBlock() ;

//	fprintf
//	(	stderr,
//		"KBLinkTest::testLinks2: %d->%d\n",
//		drows1,
//		drows2
//	)	;

	for (uint row = 0 ; row < drows2 ; row += 1)
	{
		KBControl  *control = getControl (link, row, row) ;
		RKComboBox *widget  = getWidget  (control) ;

		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected widget value: row %1").arg(row)),
			(int)(row + 1),
			(int)widget->currentItem()
		)	;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected control text: row %1").arg(row)),
			_STDSTR(QString::number(row + 1)),
			_STDSTR(control->getValue().getRawText())
		)	;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected link text: row %1").arg(row)),
			_STDSTR(QString::number(row + 1)),
			_STDSTR(link  ->getValue(row).getRawText())
		)	;
	}
}

void	KBLinkTest::testLinks6 ()
{
	KBLink *link0  = getLink  ("Link_0") ;
	KBLink *link1  = getLink  ("Link_1") ;
	KBLink *link2  = getLink  ("Link_2") ;
	KBLink *link3  = getLink  ("Link_3") ;

	link0->setVisible (0, false) ;
	link0->setEnabled (0, false) ;
	link1->setVisible (1, false) ;
	link1->setEnabled (1,  true) ;
	link2->setVisible (2,  true) ;
	link2->setEnabled (2, false) ;
	link3->setVisible (3, true ) ;
	link3->setEnabled (3, true ) ;

	CPPUNIT_ASSERT (!link0->isVisible(0)) ;
	CPPUNIT_ASSERT (!link0->isEnabled(0)) ;
	CPPUNIT_ASSERT (!link1->isVisible(1)) ;
	CPPUNIT_ASSERT ( link1->isEnabled(1)) ;
	CPPUNIT_ASSERT ( link2->isVisible(2)) ;
	CPPUNIT_ASSERT (!link2->isEnabled(2)) ;
	CPPUNIT_ASSERT ( link3->isVisible(3)) ;
	CPPUNIT_ASSERT ( link3->isEnabled(3)) ;

	KBControl  *control0 = getControl (link0, 0, 0) ;
	RKComboBox *widget0  = getWidget  (control0) ;
	KBControl  *control1 = getControl (link1, 1, 1) ;
	RKComboBox *widget1  = getWidget  (control1) ;
	KBControl  *control2 = getControl (link2, 2, 2) ;
	RKComboBox *widget2  = getWidget  (control2) ;
	KBControl  *control3 = getControl (link3, 3, 3) ;
	RKComboBox *widget3  = getWidget  (control3) ;

	CPPUNIT_ASSERT (!widget0->isVisible()) ;
	CPPUNIT_ASSERT (!widget0->isEnabled()) ;
	CPPUNIT_ASSERT (!widget1->isVisible()) ;
	CPPUNIT_ASSERT ( widget1->isEnabled()) ;
	CPPUNIT_ASSERT ( widget2->isVisible()) ;
	CPPUNIT_ASSERT (!widget2->isEnabled()) ;
	CPPUNIT_ASSERT ( widget3->isVisible()) ;
	CPPUNIT_ASSERT ( widget3->isEnabled()) ;
}

void	KBLinkTest::testLinks7 ()
{
	KBLink     *link0    = getLink    ("Link_0") ;
	KBControl  *control0 = getControl (link0, 0, 0) ;
	RKComboBox *combo0   = getWidget  (control0) ;

	QStringList strings  = link0->getDisplayList (0) ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	"Unexpected entry count",
		10,
		(int)strings.count()
	)	;

	for (uint idx1 = 1 ; idx1 < 10 ; idx1 += 1)
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected text at offset %1").arg(idx1)),
			_STDSTR(QString(strings_n[idx1-1])),
			_STDSTR(strings[idx1])
		)	;

	for (uint idx2 = 1 ; idx2 < 10 ; idx2 += 1)
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected control value at offset %1").arg(idx2)),
			_STDSTR(QString(strings_n[idx2-1])),
			_STDSTR(combo0->text(idx2))
		)	;
}

void	KBLinkTest::testLinks8 ()
{
	KBLink     *link0    = getLink    ("Link_0") ;
	KBControl  *control0 = getControl (link0, 0, 0) ;
	RKComboBox *combo0   = getWidget  (control0) ;

	CPPUNIT_ASSERT(link0->currentItem(0) == 1) ;

	link0->setCurrentItem(0, 2) ;
	CPPUNIT_ASSERT(link0->currentItem(0) == 2) ;

	link0->setCurrentItem(0, 0) ;
	CPPUNIT_ASSERT(link0->currentItem(0) == 0) ;
}


void	KBLinkTest::testLinks9 ()
{
	KBLink     *link0    = getLink    ("Link_0") ;
	KBControl  *control0 = getControl (link0, 0, 0) ;
	RKComboBox *combo0   = getWidget  (control0) ;

	CPPUNIT_ASSERT(link0->currentItem(0) == 1) ;

	rekallGUIPause     ("testLinks9") ;
	CPPUNIT_ASSERT(link0->setCurrentDisplay(0, "Two")) ;
	rekallGUIPause     ("testLinks9") ;
	CPPUNIT_ASSERT(link0->currentItem(0) == 2) ;

	CPPUNIT_ASSERT(link0->setCurrentDisplay(0, "")) ;
	CPPUNIT_ASSERT(link0->currentItem(0) == 0) ;
}


void	KBLinkTest::testLinks10 ()
{
	KBLink     *link0    = getLink    ("Link_0") ;
	KBControl  *control0 = getControl (link0, 0, 0) ;
	RKComboBox *combo0   = getWidget  (control0) ;

	CPPUNIT_ASSERT( link0->setCurrentDisplay(0, "One")) ;
	CPPUNIT_ASSERT(!link0->setCurrentDisplay(0, "Not")) ;
	CPPUNIT_ASSERT( link0->setCurrentDisplay(0, "Two")) ;
}


CppUnit::Test
	*KBLinkTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Link") ;

	if (rekallTestSkip ("Links")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;


	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBLinkTest> ("noDisplay", &KBLinkTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBLinkTest> ("withGUI",   &KBLinkTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBLinkTest>
			(	"links0",
				&KBLinkTest::testLinks0
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBLinkTest>
			(	"links1",
				&KBLinkTest::testLinks1
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBLinkTest>
			(	"links2",
				&KBLinkTest::testLinks2
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBLinkTest>
			(	"links3",
				&KBLinkTest::testLinks3
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBLinkTest>
			(	"links5",
				&KBLinkTest::testLinks5
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBLinkTest>
			(	"links6",
				&KBLinkTest::testLinks6
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBLinkTest>
			(	"links7",
				&KBLinkTest::testLinks7
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBLinkTest>
			(	"links8",
				&KBLinkTest::testLinks8
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBLinkTest>
			(	"links9",
				&KBLinkTest::testLinks9
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBLinkTest>
			(	"links10",
				&KBLinkTest::testLinks10
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_API *linkTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBLinkTest::suite (__settings) ;
}
