/***************************************************************************
    file	         : mapcase.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qapplication.h>
#include	<qstring.h>
#include	<qmap.h>

#include	"rk_application.h"
#include	"rk_lineedit.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_field.h"
#include	"kb_memo.h"
#include	"kb_dom.h"
#include	"kb_textedit.h"

#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBMapCaseTest : public CppUnit::TestCase
{
	TestOpenForm		*m_form		;

	KBField			*getField	(const char *) ;
	KBMemo			*getMemo	(const char *) ;
	KBControl		*getControl	(KBItem     *, const char *) ;
	KBControl		*getControl	(KBItem     *, const char *, uint, uint) ;
	RKLineEdit		*getLineEdit	(KBControl  *) ;
	KBTextEdit		*getTextEdit	(KBControl  *) ;

public	:

	KBMapCaseTest	() ;
	KBMapCaseTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testMapCase0	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBMapCaseTest::KBMapCaseTest ()
{
}

KBMapCaseTest::KBMapCaseTest
	(	std::string		name
	)
	:
	CppUnit::TestCase (name)
{
}

void	KBMapCaseTest::setUp ()
{
	KBDomDocument	doc	;
	QDomElement	info	;
	if (!doc.loadFile ("API.rkl", "test/api", getenv("TESTPATH")))
	{
		fprintf
		(	stderr,
			"fields: %s: cannot open: %s\n",
			"API.xml",
			doc.lastError().getMessage().latin1()
		)	;
		exit	(1) ;
	}
	DOMITER_BEGIN(doc.documentElement(),"serverinfo",elem)
	{
		info	= elem	;
	}
	DOMITER_END

	KBDBInfo	*dbInfo	= new KBDBInfo     ("noname") ;
	KBServerInfo	*svInfo	= new KBServerInfo (dbInfo, info) ;

	dbInfo->insert	(svInfo);

	KBError	error	;
	m_form	= rekallTestOpenForm (dbInfo, "api", "MapCase", error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;
}

void	KBMapCaseTest::tearDown ()
{
	rekallTestCloseForm () ;
}

void	KBMapCaseTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBMapCaseTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

KBField
	*KBMapCaseTest::getField
	(	const char	*name
	)
{
	KBNode	 *target = m_form->m_form->getNamedNode(name) ;

	CPPUNIT_ASSERT (target != 0) ;
	CPPUNIT_ASSERT (target->isA("KBField")) ;

	KBField *field = (KBField *)target ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString(name)),
		_STDSTR(field->getName())
	)	;

	return	field	;
}

KBMemo
	*KBMapCaseTest::getMemo
	(	const char	*name
	)
{
	KBNode	 *target = m_form->m_form->getNamedNode(name) ;

	CPPUNIT_ASSERT (target != 0) ;
	CPPUNIT_ASSERT (target->isA("KBMemo")) ;

	KBMemo   *memo   = (KBMemo *)target ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString(name)),
		_STDSTR(memo->getName())
	)	;

	return	memo ;
}

KBControl
	*KBMapCaseTest::getControl
	(	KBItem		*item,
		const char	*isa
	)
{
	KBControl *control = KBTestAccessor::getControl (item) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA(isa)) ;
	return	  control  ;
}

KBControl
	*KBMapCaseTest::getControl
	(	KBItem		*item,
		const char	*isa,
		uint		qrow,
		uint		drow
	)
{
	KBControl *control = KBTestAccessor::getControl (item, qrow) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA(isa)) ;
	CPPUNIT_ASSERT (control->getDRow() == drow ) ;
	return	  control  ;
}

RKLineEdit
	*KBMapCaseTest::getLineEdit
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;
	CPPUNIT_ASSERT (widget->isA("RKLineEdit")) ;
	return	(RKLineEdit *)widget ;
}

KBTextEdit
	*KBMapCaseTest::getTextEdit
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;
	CPPUNIT_ASSERT (widget->isA("KBTextEdit")) ;
	return	(KBTextEdit *)widget ;
}

void	KBMapCaseTest::testMapCase0 ()
{
	KBField    *field1    = getField  ("Field_1") ;
	KBField    *field2    = getField  ("Field_2") ;
	KBMemo     *memo1     = getMemo   ("Memo_1" ) ;
	KBMemo     *memo2     = getMemo   ("Memo_2" ) ;


	KBControl  *controlF1 = getControl (field1,  "KBCtrlField") ;
	KBControl  *controlF2 = getControl (field2,  "KBCtrlField") ;
	RKLineEdit *widgetF1  = getLineEdit(controlF1) ;
	RKLineEdit *widgetF2  = getLineEdit(controlF2) ;

	KBControl  *controlM1 = getControl (memo1,   "KBCtrlMemo" ) ;
	KBControl  *controlM2 = getControl (memo2,   "KBCtrlMemo" ) ;
	KBTextEdit *widgetM1  = getTextEdit(controlM1) ;
	KBTextEdit *widgetM2  = getTextEdit(controlM2) ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("Field_1: Unexpected initial text")),
		_STDSTR(QString("One")),
		_STDSTR(controlF1->getValue().getRawText())
	)	;
	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("Field_2: Unexpected initial text")),
		_STDSTR(QString("One")),
		_STDSTR(controlF2->getValue().getRawText())
	)	;
	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("Memo_1: Unexpected initial text")),
		_STDSTR(QString("One")),
		_STDSTR(controlM1->getValue().getRawText())
	)	;
	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("Memo_2: Unexpected initial text")),
		_STDSTR(QString("One")),
		_STDSTR(controlM1->getValue().getRawText())
	)	;

	QFocusEvent fout (QEvent::FocusOut) ;
	KBTestAccessor::eventFilter (controlF1, widgetF1, &fout) ;
	KBTestAccessor::eventFilter (controlF2, widgetF2, &fout) ;
	KBTestAccessor::eventFilter (controlM1, widgetM1->textEdit(), &fout) ;
	KBTestAccessor::eventFilter (controlM2, widgetM2->textEdit(), &fout) ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("Field_1: Unexpected mapped text")),
		_STDSTR(QString("ONE")),
		_STDSTR(controlF1->getValue().getRawText())
	)	;
	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("Field_2: Unexpected mapped text")),
		_STDSTR(QString("one")),
		_STDSTR(controlF2->getValue().getRawText())
	)	;
	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("Memo_1: Unexpected imapped text")),
		_STDSTR(QString("ONE")),
		_STDSTR(controlM1->getValue().getRawText())
	)	;
	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("Memo_2: Unexpected imapped text")),
		_STDSTR(QString("one")),
		_STDSTR(controlM2->getValue().getRawText())
	)	;
}


CppUnit::Test
	*KBMapCaseTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Field") ;

	if (rekallTestSkip ("MapCase")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;


	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBMapCaseTest> ("noDisplay", &KBMapCaseTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBMapCaseTest> ("withGUI",   &KBMapCaseTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBMapCaseTest>
			(	"mapCase0",
				&KBMapCaseTest::testMapCase0
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_API *mapCaseTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBMapCaseTest::suite (__settings) ;
}
