/***************************************************************************
    file	         : modes.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qmainwindow.h>
#include	<qstring.h>
#include	<qmap.h>
#include	<qregexp.h>

#include	"rk_application.h"
#include	"rk_pushbutton.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_button.h"
#include	"kb_dom.h"
#include	"kb_sizer.h"

#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBModesTest : public CppUnit::TestCase
{
	QString			m_name		;
	TestOpenForm		*m_form		;

	void			openForm	(const char *, KB::ShowAs) ;

public	:

	KBModesTest	() ;
	KBModesTest	(std::string name, const char *) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testData	() ;
	void			testDesign	() ;
	void			testDataDesign	() ;
	void			testDesignData	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBModesTest::KBModesTest ()
{
}

KBModesTest::KBModesTest
	(	std::string		name,
		const char		*form
	)
	:
	CppUnit::TestCase (name),
	m_name		  (form)
{
}

void	KBModesTest::setUp ()
{
}

void	KBModesTest::tearDown ()
{
	rekallTestCloseForm () ;
}

void	KBModesTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBModesTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

void	KBModesTest::openForm
	(	const char	*name,
		KB::ShowAs	showAs
	)
{
	KBDomDocument	doc	;
	QDomElement	info	;
	if (!doc.loadFile ("API.rkl", "test/api", getenv("TESTPATH")))
	{
		fprintf
		(	stderr,
			"fields: %s: cannot open: %s\n",
			"API.xml",
			doc.lastError().getMessage().latin1()
		)	;
		exit	(1) ;
	}
	DOMITER_BEGIN(doc.documentElement(),"serverinfo",elem)
	{
		info	= elem	;
	}
	DOMITER_END

	KBDBInfo	*dbInfo	= new KBDBInfo     ("noname") ;
	KBServerInfo	*svInfo	= new KBServerInfo (dbInfo, info) ;

	dbInfo->insert	(svInfo);

	KBError	error	;
	m_form	= rekallTestOpenFormMode
			(	dbInfo,
				"api",
				name,
				showAs,
				error
			)	;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;
}

void	KBModesTest::testData ()
{
	openForm (m_name, KB::ShowAsData) ;
	rekallGUIPause ("modesTestData") ;
}

void	KBModesTest::testDesign ()
{
	openForm (m_name, KB::ShowAsDesign) ;
	rekallGUIPause ("modesTestDesign") ;
}

void	KBModesTest::testDataDesign ()
{
	openForm (m_name, KB::ShowAsData) ;
	rekallGUIPause ("modesTestDataDesign") ;
	m_form->m_form->showAs (KB::ShowAsDesign) ;
	rekallGUIPause ("modesTestDataDesign") ;
}

void	KBModesTest::testDesignData ()
{
	openForm (m_name, KB::ShowAsDesign) ;
	rekallGUIPause ("modesTestDesignData") ;
	m_form->m_form->showAs (KB::ShowAsData) ;
	rekallGUIPause ("modesTestDesignData") ;
}

CppUnit::Test
	*KBModesTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Button") ;

	if (rekallTestSkip ("Modes")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;


	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBModesTest> ("noDisplay", &KBModesTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBModesTest> ("withGUI",   &KBModesTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		static	const char *forms[] =
		{
			"Buttons",
			"Containers",
			"Fields",
			"Format",
			"Labels",
			"Links",
			"Nested",
			"Stack",
			"Tabber",
			"Trees",
			0
		}	;

		for (const char **cp = &forms[0] ; *cp != 0 ; cp += 1)
		{
		s->addTest
		(	new CppUnit::TestCaller<KBModesTest>
			(	_STDSTR(QString("data_%1").arg(*cp)),
				&KBModesTest::testData,
				new KBModesTest (*cp, *cp)
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBModesTest>
			(	_STDSTR(QString("design_%1").arg(*cp)),
				&KBModesTest::testDesign,
				new KBModesTest (*cp, *cp)
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBModesTest>
			(	_STDSTR(QString("dataDesign_%1").arg(*cp)),
				&KBModesTest::testDataDesign,
				new KBModesTest (*cp, *cp)
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBModesTest>
			(	_STDSTR(QString("designData_%1").arg(*cp)),
				&KBModesTest::testDesignData,
				new KBModesTest (*cp, *cp)
		)	)	;
		}
	}

	return s ;
}

CppUnit::Test EXPORT_API *modesTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBModesTest::suite (__settings) ;
}
