/***************************************************************************
    file	         : query.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qapplication.h>
#include	<qstring.h>
#include	<qmap.h>

#include	"rk_application.h"
#include	"rk_lineedit.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_field.h"
#include	"kb_sizer.h"
#include	"kb_dom.h"

#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"
#include	"api.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBQueryTest : public KBAPITest
{
	KBDBInfo		*m_dbInfo	;
	QString			m_formName	;
	TestOpenForm		*m_form		;

	KBQryBase		*m_query	;
	QString			m_sql		;

public	:

	KBQueryTest	() ;
	KBQueryTest	(std::string name, const char *) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testQuery0	() ;
	void			testQuery1	() ;
	void			testQuery2	() ;
	void			testQuery3	() ;
	void			testQuery4	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBQueryTest::KBQueryTest ()
	:
	KBAPITest	()
{
}

KBQueryTest::KBQueryTest
	(	std::string		name,
		const char		*formName
	)
	:
	KBAPITest	(name),
	m_formName	(formName)
{
}

void	KBQueryTest::setUp ()
{
	KBError	 error	;

	m_dbInfo = rekallDBInfo ("API.rkl", "test/api") ;
	m_form	 = rekallTestOpenForm (m_dbInfo, "api", m_formName, error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;

	m_query	= m_form->m_form->getQuery() ;
	CPPUNIT_ASSERT	(m_query != 0) ;

	m_sql	= m_query->getSQLText (m_form->m_form->getQryLevel(), false) ;
}

void	KBQueryTest::tearDown ()
{
	rekallTestCloseForm () ;
	delete	m_dbInfo ;
}

void	KBQueryTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBQueryTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

void	KBQueryTest::testQuery0 ()
{
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("select 0, TableA.id, TableA.t from TableA")),
		_STDSTR(m_sql)
	)	;
}

void	KBQueryTest::testQuery1 ()
{
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("select 0, TableA.id, TableA.t from TableA, TableB where TableA.id = TableB.id")),
		_STDSTR(m_sql)
	)	;
}

void	KBQueryTest::testQuery2 ()
{
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("select 0, TableA.id, TableA.t from TableA, TableB where TableA.id = TableB.id")),
		_STDSTR(m_sql)
	)	;
}

void	KBQueryTest::testQuery3 ()
{
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("select 0, TableA.id, TableA.t from TableA left outer join TableB on TableA.id = TableB.id")),
		_STDSTR(m_sql)
	)	;
}

void	KBQueryTest::testQuery4 ()
{
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("select 0, TableA.id, TableA.t from TableA right outer join TableB on TableA.id = TableB.id")),
		_STDSTR(m_sql)
	)	;
}

CppUnit::Test
	*KBQueryTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Query") ;

	if (rekallTestSkip ("Query")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;


	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBQueryTest> ("noDisplay", &KBQueryTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBQueryTest>
			(	"withGUI",
				&KBQueryTest::withGUI
		)	)	;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBQueryTest>
			(	"querySQLSimple",
				&KBQueryTest::testQuery0,
				new KBQueryTest ("querySQLSimple", "SQLBlock_Simple")
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBQueryTest>
			(	"querySQLJoinWhere",
				&KBQueryTest::testQuery1,
				new KBQueryTest ("querySQLJoinWhere", "SQLBlock_JoinWhere")
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBQueryTest>
			(	"querySQLJoinInner",
				&KBQueryTest::testQuery2,
				new KBQueryTest ("querySQLJoinInner", "SQLBlock_JoinInner")
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBQueryTest>
			(	"querySQLJoinLeft",
				&KBQueryTest::testQuery3,
				new KBQueryTest ("querySQLJoinLeft", "SQLBlock_JoinLeft")
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBQueryTest>
			(	"querySQLJoinRight",
				&KBQueryTest::testQuery4,
				new KBQueryTest ("querySQLJoinRight", "SQLBlock_JoinRight")
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_API *queryTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBQueryTest::suite (__settings) ;
}
