/***************************************************************************
    file	         : stack.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qmainwindow.h>
#include	<qstring.h>
#include	<qmap.h>
#include	<qregexp.h>

#include	"rk_application.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_stack.h"
#include	"kb_sizer.h"
#include	"kb_dom.h"

#include	"tester.h"
#include	"tester_kbase.h"


#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBStackTest : public CppUnit::TestCase
{
	TestOpenForm		*m_form		;


public	:

	KBStackTest	() ;
	KBStackTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testStack1	() ;
	void			testStack2	() ;
	void			testStack3	() ;
	void			testStack4	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBStackTest::KBStackTest ()
{
}

KBStackTest::KBStackTest
	(	std::string		name
	)
	:
	CppUnit::TestCase (name)
{
}

void	KBStackTest::setUp ()
{
	KBError	error	;
	m_form	= rekallTestOpenForm (0, "api", "Stack", error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;
}

void	KBStackTest::tearDown ()
{
	rekallTestCloseForm () ;
}

void	KBStackTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBStackTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

void	KBStackTest::testStack1 ()
{
	KBNode	*stack	= m_form->m_form->getNamedNode("Stack") ;

	CPPUNIT_ASSERT (stack != 0) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Stack")),
		_STDSTR(stack->getName())
	)	;
}

void	KBStackTest::testStack2 ()
{
	KBNode	*page1 = m_form->m_form->getNamedNode("Stack/Page_1") ;
	KBNode	*page2 = m_form->m_form->getNamedNode("Stack/Page_2") ;

	CPPUNIT_ASSERT (page1 != 0) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Page_1")),
		_STDSTR(page1->getName())
	)	;

	CPPUNIT_ASSERT (page2 != 0) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Page_2")),
		_STDSTR(page2->getName())
	)	;
}

void	KBStackTest::testStack3 ()
{
	KBNode	*node	= m_form->m_form->getNamedNode("Stack") ;
	CPPUNIT_ASSERT (node  != 0) ;

	KBStack *stack	= node->isStack() ;
	CPPUNIT_ASSERT (stack != 0) ;

	KBNode  *page   = stack->currentPage() ;

	CPPUNIT_ASSERT (page != 0) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Page_1")),
		_STDSTR(page->getName())
	)	;

	KBNode	*button	= page->getNamedNode ("Button_1") ;
	CPPUNIT_ASSERT (button != 0) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Button_1")),
		_STDSTR(button->getName())
	)	;
}

void	KBStackTest::testStack4 ()
{
	KBNode	*node	= m_form->m_form->getNamedNode("Stack") ;
	CPPUNIT_ASSERT (node  != 0) ;

	KBStack *stack	= node->isStack() ;
	CPPUNIT_ASSERT (stack != 0) ;

	KBNode	*node2 = m_form->m_form->getNamedNode("Stack/Page_2") ;
	CPPUNIT_ASSERT (node2 != 0) ;

	KBStackPage *page2 = node2->isStackPage() ;
	CPPUNIT_ASSERT (page2 != 0) ;

	page2->setCurrent() ;

	KBNode  *page   = stack->currentPage() ;

	CPPUNIT_ASSERT (page != 0) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Page_2")),
		_STDSTR(page->getName())
	)	;

	KBNode	*button	= page->getNamedNode ("Button_2") ;
	CPPUNIT_ASSERT (button != 0) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Button_2")),
		_STDSTR(button->getName())
	)	;
}


CppUnit::Test
	*KBStackTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Container") ;

	if (rekallTestSkip ("Stack")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;

	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBStackTest> ("noDisplay", &KBStackTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBStackTest> ("withGUI",   &KBStackTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBStackTest>
			(	"stack1",
				&KBStackTest::testStack1
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBStackTest>
			(	"stack2",
				&KBStackTest::testStack2
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBStackTest>
			(	"stack3",
				&KBStackTest::testStack3
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBStackTest>
			(	"stack4",
				&KBStackTest::testStack4
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_API *stackTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBStackTest::suite (__settings) ;
}
