/***************************************************************************
    file	         : trees.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qapplication.h>
#include	<qstring.h>
#include	<qmap.h>
#include	<qlistview.h>

#include	"rk_application.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_tree.h"
#include	"kb_ctrl.h"
#include	"kb_dom.h"

#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBTreeTest : public CppUnit::TestCase
{
	TestOpenForm		*m_form		;

	KBTree			*getTree	(const char *) ;
	KBControl		*getControl	(KBTree    *) ;
	KBControl		*getControl	(KBTree    *, uint, uint) ;
	QListView		*getWidget	(KBControl  *) ;

public	:

	KBTreeTest	() ;
	KBTreeTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testTree0	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBTreeTest::KBTreeTest ()
{
}

KBTreeTest::KBTreeTest
	(	std::string		name
	)
	:
	CppUnit::TestCase (name)
{
}

void	KBTreeTest::setUp ()
{
	KBDomDocument	doc	;
	QDomElement	info	;
	if (!doc.loadFile ("API.rkl", "test/api", getenv("TESTPATH")))
	{
		fprintf
		(	stderr,
			"trees: %s: cannot open: %s\n",
			"API.xml",
			doc.lastError().getMessage().latin1()
		)	;
		exit	(1) ;
	}
	DOMITER_BEGIN(doc.documentElement(),"serverinfo",elem)
	{
		info	= elem	;
	}
	DOMITER_END

	KBDBInfo	*dbInfo	= new KBDBInfo     ("noname") ;
	KBServerInfo	*svInfo	= new KBServerInfo (dbInfo, info) ;

	dbInfo->insert	(svInfo);

	KBError	error	;
	m_form	= rekallTestOpenForm (dbInfo, "api", "Trees", error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;
}

void	KBTreeTest::tearDown ()
{
	rekallTestCloseForm () ;
}

void	KBTreeTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBTreeTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

KBTree
	*KBTreeTest::getTree
	(	const char	*name
	)
{
	KBNode	 *target = m_form->m_form->getNamedNode(name) ;

	CPPUNIT_ASSERT (target != 0) ;
	CPPUNIT_ASSERT (target->isA("KBTree")) ;

	KBTree *tree = (KBTree *)target ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString(name)),
		_STDSTR(tree->getName())
	)	;

	return	tree	;
}

KBControl
	*KBTreeTest::getControl
	(	KBTree		*tree
	)
{
	KBControl *control = KBTestAccessor::getControl (tree) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA("KBCtrlTree")) ;
	return	  control  ;
}

KBControl
	*KBTreeTest::getControl
	(	KBTree		*tree,
		uint		qrow,
		uint		drow
	)
{
	KBControl *control = KBTestAccessor::getControl (tree, qrow) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA("KBCtrlTree")) ;
	CPPUNIT_ASSERT (control->getDRow() == drow) ;
	return	  control  ;
}

QListView
	*KBTreeTest::getWidget
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;
	CPPUNIT_ASSERT (widget->isA("QListView")) ;
	return	(QListView *)widget ;
}

void	KBTreeTest::testTree0 ()
{
	KBTree *tree1   = getTree  ("Tree_1") ;
}


CppUnit::Test
	*KBTreeTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Tree") ;

	if (rekallTestSkip ("Trees")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;


	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBTreeTest> ("noDisplay", &KBTreeTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBTreeTest> ("withGUI",   &KBTreeTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBTreeTest>
			(	"testTree0",
				&KBTreeTest::testTree0
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_API *treeTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBTreeTest::suite (__settings) ;
}
