/***************************************************************************
    file	         : validityNested.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qapplication.h>
#include	<qstring.h>
#include	<qmap.h>

#include	"rk_application.h"
#include	"rk_lineedit.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_field.h"
#include	"kb_dom.h"

#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"
#include	"api.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBValidityNestedTest : public KBAPITest
{
public	:

	KBValidityNestedTest	() ;
	KBValidityNestedTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			testValidityNested1	() ;
	void			testValidityNested2	() ;
	void			testValidityNested3	() ;
	void			testValidityNested4	() ;
	void			testValidityNested5	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBValidityNestedTest::KBValidityNestedTest ()
	:
	KBAPITest ()
{
}

KBValidityNestedTest::KBValidityNestedTest
	(	std::string		name
	)
	:
	KBAPITest (name)
{
}

void	KBValidityNestedTest::setUp ()
{
	openForm ("ValidityNested") ;

	KBField    *field1   = getField   ("Field_t_1", "Field_t_1") ;
	KBField    *field1s  = getField   ("block/Field_t_1_Sub", "Field_t_1_Sub") ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("No")),
		_STDSTR(field1 ->getAttrVal("nullok"))
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("No")),
		_STDSTR(field1s->getAttrVal("nullok"))
	)	;

}

void	KBValidityNestedTest::tearDown ()
{
	rekallTestCloseForm() ;
}

void	KBValidityNestedTest::testValidityNested1 ()
{
//	KBField    *field1   = getField   ("Field_t_1", "Field_t_1") ;
//	KBField    *field1s  = getField   ("block/Field_t_1_Sub", "Field_t_1_Sub") ;

	QPtrList<KBItem> items  ;
	bool	   anySet	;

	anySet	= m_form->m_form->invalidControls (0, items, false) ;
	CPPUNIT_ASSERT_EQUAL
	(	(unsigned int)0,
		(unsigned int)items.count()
	)	;
	CPPUNIT_ASSERT
	(	anySet
	)	;
}

void	KBValidityNestedTest::testValidityNested2 ()
{
	KBField    *field1   = getField   ("Field_t_1", "Field_t_1") ;
//	KBField    *field1s  = getField   ("block/Field_t_1_Sub", "Field_t_1_Sub") ;

	field1->setValue(0, KBValue()) ;

	QPtrList<KBItem> items  ;
	bool	   anySet	;

	anySet	= m_form->m_form->invalidControls (0, items, false) ;
	CPPUNIT_ASSERT_EQUAL
	(	(unsigned int)1,
		(unsigned int)items.count()
	)	;
	CPPUNIT_ASSERT
	(	anySet
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Field_t_1")),
		_STDSTR(items.at(0)->getName())
	)	;
}

void	KBValidityNestedTest::testValidityNested3 ()
{
	KBField    *field1   = getField   ("Field_t_1", "Field_t_1") ;
//	KBField    *field1s  = getField   ("block/Field_t_1_Sub", "Field_t_1_Sub") ;

	field1->setValue(0, KBValue()) ;

	QPtrList<KBItem> items  ;
	bool	   anySet	;

	anySet	= m_form->m_form->invalidControls (0, items, true) ;
	CPPUNIT_ASSERT_EQUAL
	(	(unsigned int)1,
		(unsigned int)items.count()
	)	;
	CPPUNIT_ASSERT
	(	anySet
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Field_t_1")),
		_STDSTR(items.at(0)->getName())
	)	;
}

void	KBValidityNestedTest::testValidityNested4 ()
{
//	KBField    *field1   = getField   ("Field_t_1", "Field_t_1") ;
	KBField    *field1s  = getField   ("block/Field_t_1_Sub", "Field_t_1_Sub") ;

	field1s->setValue(0, KBValue()) ;

	QPtrList<KBItem> items  ;
	bool	   anySet	;

	anySet	= m_form->m_form->invalidControls (0, items, false) ;
	CPPUNIT_ASSERT_EQUAL
	(	(unsigned int)0,
		(unsigned int)items.count()
	)	;
	CPPUNIT_ASSERT
	(	anySet
	)	;
}

void	KBValidityNestedTest::testValidityNested5 ()
{
//	KBField    *field1   = getField   ("Field_t_1", "Field_t_1") ;
	KBField    *field1s  = getField   ("block/Field_t_1_Sub", "Field_t_1_Sub") ;

	field1s->setValue(0, KBValue()) ;

	QPtrList<KBItem> items  ;
	bool	   anySet	;

	anySet	= m_form->m_form->invalidControls (0, items, true) ;
	CPPUNIT_ASSERT_EQUAL
	(	(unsigned int)1,
		(unsigned int)items.count()
	)	;
	CPPUNIT_ASSERT
	(	anySet
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Field_t_1_Sub")),
		_STDSTR(items.at(0)->getName())
	)	;
}

CppUnit::Test
	*KBValidityNestedTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Field") ;

	if (rekallTestSkip ("ValidityNested")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;


	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBValidityNestedTest> ("noDisplay", &KBValidityNestedTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBValidityNestedTest> ("withGUI",   &KBValidityNestedTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBValidityNestedTest>
			(	"validityNested1",
				&KBValidityNestedTest::testValidityNested1
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBValidityNestedTest>
			(	"validityNested2",
				&KBValidityNestedTest::testValidityNested2
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBValidityNestedTest>
			(	"validityNested3",
				&KBValidityNestedTest::testValidityNested3
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBValidityNestedTest>
			(	"validityNested4",
				&KBValidityNestedTest::testValidityNested4
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBValidityNestedTest>
			(	"validityNested5",
				&KBValidityNestedTest::testValidityNested5
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_API *validityNestedTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBValidityNestedTest::suite (__settings) ;
}
