/***************************************************************************
    file	         : api_report.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qmainwindow.h>
#include	<qstring.h>
#include	<qmap.h>
#include	<qregexp.h>

#include	"rk_application.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_report.h"
#include	"kb_dom.h"
#include	"kb_script.h"

#include	"tester.h"
#include	"tester_kbase.h"


#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif




class	KBAPIReportScriptTest : public CppUnit::TestCase
{
	QString			m_name		;
	QDomElement		m_elem		;

	KBDBInfo		*m_dbInfo	;
	TestOpenReport		*m_report		;

	QString			trimWhiteSpace	(QString) ;

public	:

	KBAPIReportScriptTest	() ;
	KBAPIReportScriptTest	(std::string name, const QString &, const QDomElement &) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testAPI		() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBAPIReportScriptTest::KBAPIReportScriptTest ()
{
}

KBAPIReportScriptTest::KBAPIReportScriptTest
	(	std::string		ident,
		const QString		&name,
		const QDomElement	&elem
	)
	:
	CppUnit::TestCase (ident),
	m_name		  (name),
	m_elem		  (elem)
{
}

QString	KBAPIReportScriptTest::trimWhiteSpace
	(	QString		text
	)
{
	int	idx ;
	for (idx = 0 ; idx < (int)text.length() ; idx += 1)
	{
		if ( text[idx] == '\n')
		{	idx	+= 1 ;
			break	;
		}

		if (!text[idx].isSpace())
			break	;
	}

	text	= text.mid(idx) ;
	for (idx = text.length() - 1 ; idx >= 0 ; idx -= 1)
		if ((text[idx] != ' ') && (text[idx] != '\n'))
			break	;
	text	= text.mid(0, idx + 1) ;

//	fprintf
//	(	stderr,
//		"[[[\n%s]]]\n",
//		text.latin1()
//	)	;

	return	text	;
}

void	KBAPIReportScriptTest::setUp ()
{
	KBError	 error	;
	m_dbInfo = rekallDBInfo ("API_Script.rkl", "test/api_script") ;
	m_report = rekallTestOpenReport (m_dbInfo, "api_script", m_name, error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_report != 0
	)	;
}

void	KBAPIReportScriptTest::tearDown ()
{
	rekallTestCloseReport () ;
	delete	m_dbInfo ;
}

void	KBAPIReportScriptTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBAPIReportScriptTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

void	KBAPIReportScriptTest::testAPI ()
{
	QString	script	;
	QString	result	;
	QString	regexp	;

	DOMITER_ELEM_BEGIN(m_elem,child)
	{
		if (child.tagName() == "script")
		{	script	= trimWhiteSpace (child.text()) ;
			regexp	= child.attribute("regexp") ;
			continue;
		}
		if (child.tagName() == "result")
		{	result	= trimWhiteSpace (child.text()) ;
			continue;
		}
	}
	DOMITER_END

	KBEvent		event	(m_report->m_report, "executeScript", script) ;
	KBValue		resval	;
	KBScriptError	*rc	= event.execute	(resval, 0, 0) ;

	if (rc != 0)
	{
		QString	err	= rc->error().getMessage() ;
		delete	rc	;
		CPPUNIT_FAIL	(_STDSTR(err)) ;
		return	;
	}

	QString	res	= trimWhiteSpace(resval.getRawText()) ;

	if (!regexp.isEmpty())
		res.replace (QRegExp(regexp), "") ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(m_elem.attribute("id")),
		_STDSTR(result),
		_STDSTR(res   )
	)	;
}

static	void	addAPITests
	(	CppUnit::TestSuite	*suite,
		const QString		&spec
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite(_STDSTR(spec)) ;
	suite->addTest (s) ;

	if (rekallTestSkip (spec))
		return	;

	KBDomDocument	doc	;
	if (!doc.loadFile ("Api_" + spec + ".xml", "test/api_script", getenv("TESTPATH")))
	{
		fprintf
		(	stderr,
			"api_script: %s: cannot open: Api_%s.xml\n",
			spec.latin1(),
			doc.lastError().getMessage().latin1()
		)	;
		exit	(1) ;
	}

	QString name = doc.documentElement().attribute("name") ;

	DOMITER_BEGIN(doc.documentElement(),"test",child)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBAPIReportScriptTest>
			(	_STDSTR(child.attribute("id")),
				&KBAPIReportScriptTest::testAPI,
				new KBAPIReportScriptTest (_STDSTR(child.attribute("id")), name, child)
			)
		)	;
	}
	DOMITER_END
}

CppUnit::Test
	*KBAPIReportScriptTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("APIReportScript") ;

	if (rekallTestSkip ("APIReportScript")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;


	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBAPIReportScriptTest> ("noDisplay", &KBAPIReportScriptTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBAPIReportScriptTest> ("withGUI",   &KBAPIReportScriptTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		addAPITests (s, "Event_Report"		) ;
	}
	return s ;
}

CppUnit::Test EXPORT_API *reportTestSuite
	(	const QMap<QString,QString> *settings
	)
{

	return	KBAPIReportScriptTest::suite (settings) ;
}

