/***************************************************************************
    file	         : dom.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qmap.h>

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_file.h"
#include	"kb_locator.h"
#include	"kb_dblink.h"
#include	"kb_dom.h"

#include	"tester.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif


class	KBDOMTest : public CppUnit::TestCase
{
public	:

	KBDOMTest	() ;
	KBDOMTest	(std::string name) ;


	void			testDOM1	() ;
	void			testDOM2	() ;
	void			testDOM3	() ;
	void			testDOM4	() ;
	void			testDOM5	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

static	const QMap<QString,QString> *settings	;


KBDOMTest::KBDOMTest ()
{
}

KBDOMTest::KBDOMTest
	(	std::string	name
	)
	:
	CppUnit::TestCase (name)
{
}

void	KBDOMTest::testDOM1 ()
{
	KBDomDocument doc ;

	if (doc.loadFile ("dom1.xml", "test/common", 0))
		CPPUNIT_FAIL("Open unexpectedly succeeded") ;
}

void	KBDOMTest::testDOM2 ()
{
	KBDomDocument doc ;

	if (doc.loadFile ("dom2.xml", "test/common", 0))
		CPPUNIT_FAIL("Open unexpectedly succeeded") ;
}

void	KBDOMTest::testDOM3 ()
{
	KBDomDocument doc ;

	if (!doc.loadFile ("dom3.xml", "test/common", 0))
		CPPUNIT_FAIL
		(	_STDSTR
			(	QString("%1: %2")
					.arg(doc.lastError().getMessage())
					.arg(doc.lastError().getDetails())
		)	)	;
}

void	KBDOMTest::testDOM4 ()
{
	KBDomDocument doc ;

	if (doc.loadFile ("dom3.xml", 0, 0))
		CPPUNIT_FAIL("Open unexpectedly succeeded") ;
}

void	KBDOMTest::testDOM5 ()
{
	const char *location	= getenv("TESTPATH") ;
        QString	   path		;

	if (location == 0)
		path	= locateDir("appdata", "test/common/dom3.xml") ;
	else	path	= location ;

	KBDomDocument doc ;

	if (doc.loadFile (path, 0, 0))
		CPPUNIT_FAIL
		(	_STDSTR
			(	QString("%1: %2")
					.arg(doc.lastError().getMessage())
					.arg(doc.lastError().getDetails())
		)	)	;
}

CppUnit::Test
	*KBDOMTest::suite
	(	const QMap<QString,QString> *__settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("DOM") ;

	if (rekallTestSkip("DOM")) return s ;

	settings = __settings ;
	s->addTest (new CppUnit::TestCaller<KBDOMTest> ("DOM1",   &KBDOMTest::testDOM1)) ;
	s->addTest (new CppUnit::TestCaller<KBDOMTest> ("DOM2",   &KBDOMTest::testDOM2)) ;
	s->addTest (new CppUnit::TestCaller<KBDOMTest> ("DOM3",   &KBDOMTest::testDOM3)) ;
	s->addTest (new CppUnit::TestCaller<KBDOMTest> ("DOM4",   &KBDOMTest::testDOM4)) ;
	s->addTest (new CppUnit::TestCaller<KBDOMTest> ("DOM5",   &KBDOMTest::testDOM5)) ;
	return s ;
}

CppUnit::Test EXPORT_API *domTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBDOMTest::suite (__settings) ;
}

