/***************************************************************************
    file	         : parseSelect.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qfile.h>
#include	<qstring.h>
#include	<qmap.h>

#include	"kb_dblink.h"
#include	"kb_select.h"
#include	"kb_basequery.h"
#include	"kb_dom.h"

#include	"tester.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif

class	KBParseSelectTest : public CppUnit::TestCase
{
	QDomElement		m_elem		;

public	:

	KBParseSelectTest	() ;
	KBParseSelectTest	(std::string name, const QDomElement &) ;


	void			parseSelect	() ;

	static CppUnit::Test	*suite		() ;
}	;


KBParseSelectTest::KBParseSelectTest ()
{
}

KBParseSelectTest::KBParseSelectTest
	(	std::string		name,
		const QDomElement	&elem
	)
	:
	CppUnit::TestCase 	(name),
	m_elem			(elem)
{
}

void	KBParseSelectTest::parseSelect ()
{
	KBBaseSelect	select	;
	QString		id	= m_elem.attribute("id"    ) ;
	QString		test	= m_elem.attribute("test"  ) ;
	QString		expect	= m_elem.attribute("expect") ;

	DOMITER_ELEM_BEGIN(m_elem,child)
	{
		if (child.tagName() == "test"  )
		{
			test	= child.text() ;
			continue  ;
		}
		if (child.tagName() == "expect")
		{
			expect	= child.text() ;
			continue  ;
		}
	}
	DOMITER_END

	if (expect.isEmpty())
		 expect = test.simplifyWhiteSpace() ;

	select.parse (test, 0) ;
	QString	 result	= select.getQueryText ((KBDBLink *)0) ;

	result	= result.simplifyWhiteSpace() ;
	test	= test  .simplifyWhiteSpace() ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("%1: %2").arg(id).arg(test)),
		_STDSTR(expect),
		_STDSTR(result)
	)	;
}

static	void	addSQLTests
	(	CppUnit::TestSuite	*suite,
		const char		*spec
	)
{
	KBDomDocument	doc	;
	if (!doc.loadFile (spec, "test/common", getenv("TESTPATH")))
	{
		fprintf
		(	stderr,
			"sql: %s: cannot open: %s\n",
			spec,
			doc.lastError().getMessage().latin1()
		)	;
		exit	(1) ;
	}

	DOMITER_BEGIN(doc.documentElement(),"parseSelect",elem)
	{
		suite->addTest
		(	new CppUnit::TestCaller<KBParseSelectTest>
			(	_STDSTR(elem.attribute("id")),
				&KBParseSelectTest::parseSelect,
				new KBParseSelectTest (_STDSTR(elem.attribute("id")), elem)
		)	)	;
	}
	DOMITER_END
}

CppUnit::Test
	*KBParseSelectTest::suite ()
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("parseSelect") ;

	if (rekallTestSkip("parseSelect")) return s ;

	addSQLTests (s, "select.xml") ;
	return s ;
}

CppUnit::Test EXPORT_API *parseSelectTestSuite
	(	const QMap<QString,QString> *
	)
{
	return	KBParseSelectTest::suite () ;
}

