/***************************************************************************
    file	         : sql_select.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qfile.h>
#include	<qstring.h>
#include	<qmap.h>

#include	"kb_dblink.h"
#include	"kb_select.h"
#include	"kb_basequery.h"
#include	"kb_dom.h"

#include	"tester.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif

class	KBSQLSelectTest : public CppUnit::TestCase
{
	void			parseSelect	(const QDomElement &) ;
	void			buildSelect	(const QDomElement &) ;
	void			evaluate	(const QDomElement &) ;

	QDomElement		m_elem		;

public	:

	KBSQLSelectTest	() ;
	KBSQLSelectTest	(std::string name, const QDomElement &) ;


	void			testSQL		() ;

	static CppUnit::Test	*suite		() ;
}	;


KBSQLSelectTest::KBSQLSelectTest ()
{
}

KBSQLSelectTest::KBSQLSelectTest
	(	std::string		name,
		const QDomElement	&elem
	)
	:
	CppUnit::TestCase 	(name),
	m_elem			(elem)
{
}

void	KBSQLSelectTest::buildSelect
	(	const QDomElement	&elem
	)
{
	KBSelect select	;
	QString	 id	= elem.attribute("id"    ) ;
	QString	 expect	= elem.attribute("expect") ;

	DOMITER_ELEM_BEGIN(elem, child)
	{
		if (child.nodeName() == "table")
		{
			select.appendTable(child.attribute("name"), child.attribute("alias")) ;
			continue ;
		}
		if (child.nodeName() == "expr")
		{
			select.appendExpr (child.attribute("value"), child.attribute("alias")) ;
			continue ;
		}
		if (child.nodeName() == "where")
		{
			select.appendWhere(child.attribute("expr")) ;
			continue ;
		}
		if (child.nodeName() == "group")
		{
			select.appendGroup(child.attribute("expr")) ;
			continue ;
		}
		if (child.nodeName() == "having")
		{
			select.appendHaving(child.attribute("expr")) ;
			continue ;
		}
		if (child.nodeName() == "order")
		{
			select.appendOrder(child.attribute("expr")) ;
			continue ;
		}
	}
	DOMITER_END

	QString	 result	= select.getQueryText (false, false, 0) ;

	result	= result.simplifyWhiteSpace() ;
	expect	= expect.simplifyWhiteSpace() ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(id),
		_STDSTR(expect),
		_STDSTR(result)
	)	;
}

void	KBSQLSelectTest::parseSelect
	(	const QDomElement	&elem
	)
{
	KBSelect select	;
	QString	 id	= elem.attribute("id"    ) ;
	QString	 test	= elem.attribute("test"  ) ;
	QString	 expect	= elem.attribute("expect") ;

	DOMITER_ELEM_BEGIN(elem,child)
	{
		if (child.tagName() == "test"  )
		{
			test	= elem.text() ;
			continue  ;
		}
		if (child.tagName() == "expect")
		{
			expect	= elem.text() ;
			continue  ;
		}
	}
	DOMITER_END

	if (expect.isEmpty())
		 expect = test.simplifyWhiteSpace() ;

	select.parseQuery (test) ;
	QString	 result	= select.getQueryText (false, false, 0) ;

	result	= result.simplifyWhiteSpace() ;
	test	= test  .simplifyWhiteSpace() ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("%1: %2").arg(id).arg(test)),
		_STDSTR(expect),
		_STDSTR(result)
	)	;
}

void	KBSQLSelectTest::evaluate
	(	const QDomElement	&elem
	)
{
	if (elem.nodeName() == "parseSelect")
	{
		parseSelect (elem) ;
		return	;
	}

	if (elem.nodeName() == "buildSelect")
	{
		buildSelect (elem) ;
		return	;
	}
}

void	KBSQLSelectTest::testSQL ()
{
	evaluate (m_elem) ;
}

static	void	addSQLTests
	(	CppUnit::TestSuite	*suite,
		const char		*spec
	)
{
	KBDomDocument	doc	;
	if (!doc.loadFile (spec, "test/common", getenv("TESTPATH")))
	{
		fprintf
		(	stderr,
			"sql: %s: cannot open: %s\n",
			spec,
			doc.lastError().getMessage().latin1()
		)	;
		exit	(1) ;
	}

	QDomElement  docElem	= doc.documentElement() ;
	for (QDomNode node = docElem.firstChild	() ;
			     !node  .isNull	() ;
		      node = node   .nextSibling() )
	{
		QDomElement elem = node.toElement () ;
		if (elem.isNull()) continue ;

		suite->addTest
		(	new CppUnit::TestCaller<KBSQLSelectTest>
			(	_STDSTR(elem.attribute("id")),
				&KBSQLSelectTest::testSQL,
				new KBSQLSelectTest (_STDSTR(elem.attribute("id")), elem)
		)	)	;
	}
}

CppUnit::Test
	*KBSQLSelectTest::suite ()
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("SQLSelect") ;
	if (rekallTestSkip("SQLSelect")) return s ;
	addSQLTests (s, "select.xml") ;
	return s ;
}

CppUnit::Test EXPORT_API *sqlSelectTestSuite
	(	const QMap<QString,QString> *
	)
{
	return	KBSQLSelectTest::suite () ;
}

