/***************************************************************************
    file	         : buttons.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qmainwindow.h>
#include	<qstring.h>
#include	<qmap.h>
#include	<qregexp.h>

#include	"rk_application.h"
#include	"rk_pushbutton.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_button.h"
#include	"kb_sizer.h"
#include	"kb_dom.h"

#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBCompLinkButtonTest : public CppUnit::TestCase
{
	QString			m_formName	;
	KBDBInfo		*m_dbInfo	;
	TestOpenForm		*m_form		;
	KBButton		*m_button0	;
	KBButton		*m_button1	;
	KBButton		*m_button2	;
	KBButton		*m_button3	;
	KBButton		*m_button4	;

	KBButton		*getButton	(const char *, const char *) ;
	KBControl		*getControl	(KBButton  *) ;
	RKPushButton		*getWidget	(KBControl  *) ;

public	:

	KBCompLinkButtonTest		() ;
	KBCompLinkButtonTest		(std::string name, const char *) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testButtons0	() ;
	void			testButtons1	() ;
	void			testButtons2	() ;
	void			testButtons3	() ;
	void			testButtons4	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBCompLinkButtonTest::KBCompLinkButtonTest ()
{
}

KBCompLinkButtonTest::KBCompLinkButtonTest
	(	std::string		name,
		const char		*formName
	)
	:
	CppUnit::TestCase (name),
	m_formName	  (formName)
{
}

void	KBCompLinkButtonTest::setUp ()
{
	KBError	error	;

	m_dbInfo = rekallDBInfo ("Complink.rkl", "test/complink") ;
	m_form	 = rekallTestOpenForm (m_dbInfo, "complink", m_formName, error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;

	m_button0 = getButton ("Button_0", "First"   ) ;
	m_button1 = getButton ("Button_1", "Previous") ;
	m_button2 = getButton ("Button_2", "Next"    ) ;
	m_button3 = getButton ("Button_3", "Last"    ) ;
	m_button4 = getButton ("Button_4", "Add"     ) ;

	m_form->m_form->gotoQRow (2) ;
	CPPUNIT_ASSERT (m_form->m_form->getCurQRow() == 2) ;
}

void	KBCompLinkButtonTest::tearDown ()
{
	rekallTestCloseForm () ;
	delete	m_dbInfo ;
}

void	KBCompLinkButtonTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBCompLinkButtonTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}


KBButton
	*KBCompLinkButtonTest::getButton
	(	const char	*name,
		const char	*text
	)
{
	KBNode	 *target = m_form->m_form->getNamedNode(name) ;

	CPPUNIT_ASSERT (target != 0) ;
	CPPUNIT_ASSERT (target->isA("KBCompLink")) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString(name)),
		_STDSTR(target->getName())
	)	;

	KBButton *button = 0 ;
	LITER
	(	KBNode,
		target->getChildren(),
		n,
		if (n->isA("KBButton"))
		{
			button	= (KBButton *)n ;
			break	;
		}
	)

	CPPUNIT_ASSERT (button != 0) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString(text)),
		_STDSTR(button->getText())
	)	;

	return	button	;
}

KBControl
	*KBCompLinkButtonTest::getControl
	(	KBButton	*button
	)
{
	KBControl *control = KBTestAccessor::getControl (button) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA("KBControl")) ;
	return	  control  ;
}

RKPushButton
	*KBCompLinkButtonTest::getWidget
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;
	CPPUNIT_ASSERT (widget->isA("RKPushButton")) ;
	return	(RKPushButton *)widget ;
}

void	KBCompLinkButtonTest::testButtons0 ()
{
	bool	evRc	;
	KBEvent	*event	= KBTestAccessor::findEvent(m_button0, "onclick") ;

	CPPUNIT_ASSERT (event != 0) ;

	KBTestAccessor::eventHook (m_button0, *event, 0, (KBValue *)0, evRc) ;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Expected row %1 but at %2").arg(0).arg(m_form->m_form->getCurQRow())),
		m_form->m_form->getCurQRow() == 0
	)	;
}

void	KBCompLinkButtonTest::testButtons1 ()
{
	bool	evRc	;
	KBEvent	*event	= KBTestAccessor::findEvent(m_button1, "onclick") ;

	CPPUNIT_ASSERT (event != 0) ;

	KBTestAccessor::eventHook (m_button0, *event, 0, (KBValue *)0, evRc) ;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Expected row %1 but at %2").arg(1).arg(m_form->m_form->getCurQRow())),
		m_form->m_form->getCurQRow() == 1
	)	;
}

void	KBCompLinkButtonTest::testButtons2 ()
{
	bool	evRc	;
	KBEvent	*event	= KBTestAccessor::findEvent(m_button2, "onclick") ;

	CPPUNIT_ASSERT (event != 0) ;

	KBTestAccessor::eventHook (m_button0, *event, 0, (KBValue *)0, evRc) ;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Expected row %1 but at %2").arg(3).arg(m_form->m_form->getCurQRow())),
		m_form->m_form->getCurQRow() == 3
	)	;
}

void	KBCompLinkButtonTest::testButtons3 ()
{
	bool	evRc	;
	KBEvent	*event	= KBTestAccessor::findEvent(m_button3, "onclick") ;

	CPPUNIT_ASSERT (event != 0) ;

	KBTestAccessor::eventHook (m_button0, *event, 0, (KBValue *)0, evRc) ;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Expected row %1 but at %2").arg(8).arg(m_form->m_form->getCurQRow())),
		m_form->m_form->getCurQRow() == 8
	)	;
}

void	KBCompLinkButtonTest::testButtons4 ()
{
	bool	evRc	;
	KBEvent	*event	= KBTestAccessor::findEvent(m_button4, "onclick") ;

	CPPUNIT_ASSERT (event != 0) ;

	KBTestAccessor::eventHook (m_button0, *event, 0, (KBValue *)0, evRc) ;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Expected row %1 but at %2").arg(9).arg(m_form->m_form->getCurQRow())),
		m_form->m_form->getCurQRow() == 9
	)	;
}

CppUnit::Test
	*KBCompLinkButtonTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Button") ;

	if (rekallTestSkip ("API")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;


	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBCompLinkButtonTest> ("noDisplay", &KBCompLinkButtonTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBCompLinkButtonTest> ("withGUI",   &KBCompLinkButtonTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBCompLinkButtonTest>
			(	"buttonsPY0",
				&KBCompLinkButtonTest::testButtons0,
				new KBCompLinkButtonTest ("buttonsPY0", "ButtonsPY")
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBCompLinkButtonTest>
			(	"buttonsPY1",
				&KBCompLinkButtonTest::testButtons1,
				new KBCompLinkButtonTest ("buttonsPY1", "ButtonsPY")
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBCompLinkButtonTest>
			(	"buttonsPY2",
				&KBCompLinkButtonTest::testButtons2,
				new KBCompLinkButtonTest ("buttonsPY2", "ButtonsPY")
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBCompLinkButtonTest>
			(	"buttonsPY3",
				&KBCompLinkButtonTest::testButtons3,
				new KBCompLinkButtonTest ("buttonsPY3", "ButtonsPY")
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBCompLinkButtonTest>
			(	"buttonsPY4",
				&KBCompLinkButtonTest::testButtons4,
				new KBCompLinkButtonTest ("buttonsPY4", "ButtonsPY")
		)	)	;

		s->addTest
		(	new CppUnit::TestCaller<KBCompLinkButtonTest>
			(	"buttonsKJS0",
				&KBCompLinkButtonTest::testButtons0,
				new KBCompLinkButtonTest ("buttonsKJS0", "ButtonsKJS")
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBCompLinkButtonTest>
			(	"buttonsKJS1",
				&KBCompLinkButtonTest::testButtons1,
				new KBCompLinkButtonTest ("buttonsKJS1", "ButtonsKJS")
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBCompLinkButtonTest>
			(	"buttonsKJS2",
				&KBCompLinkButtonTest::testButtons2,
				new KBCompLinkButtonTest ("buttonsKJS2", "ButtonsKJS")
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBCompLinkButtonTest>
			(	"buttonsKJS3",
				&KBCompLinkButtonTest::testButtons3,
				new KBCompLinkButtonTest ("buttonsKJS3", "ButtonsKJS")
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBCompLinkButtonTest>
			(	"buttonsKJS4",
				&KBCompLinkButtonTest::testButtons4,
				new KBCompLinkButtonTest ("buttonsKJS4", "ButtonsKJS")
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_API *buttonTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBCompLinkButtonTest::suite (__settings) ;
}
