/***************************************************************************
    file	         : el32.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qapplication.h>
#include	<qstring.h>
#include	<qmap.h>

#include	"tester.h"
#include	"tester_kbase.h"

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	"std.h"
#include	"eli.h"
#include	"interp.h"
#include	"syn.h"
#include	"code.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif




class	KBEL32Test : public CppUnit::TestCase
{
public	:

	KBEL32Test	() ;
	KBEL32Test	(std::string name)	;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			testEL32_1	() ;
	void			testEL32_2	() ;
	void			testEL32_3	() ;
	void			testEL32_4	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBEL32Test::KBEL32Test ()
{
}

KBEL32Test::KBEL32Test
	(	std::string		name
	)
	:
	CppUnit::TestCase (name)
{
}

void	KBEL32Test::setUp ()
{
	extern	void	el_initialize (uint, uint, bool) ;
	el_initialize (1024 * 64, 4096, true) ;
}

void	KBEL32Test::tearDown ()
{
}

void	KBEL32Test::testEL32_1 ()
{
	const char *script =
			"public fn ()\n"
			"{	return	12345	;\n"
			"}\n"
			;

	CBUFF	*cbuff  = el_compile (0, 0, 0, script, 0) ;
	CPPUNIT_ASSERT (cbuff != 0) ;
	CPPUNIT_ASSERT (el_loadtext (cbuff)) ;

	VALUE	res	= el_vexec   ("", "fn", 0, 0) ;

	CPPUNIT_ASSERT (res.tag->tag == V_NUM) ;
	CPPUNIT_ASSERT (res.val.num  == 12345) ;

	free	((void *)cbuff) ;
}

void	KBEL32Test::testEL32_2 ()
{
	const char *script =
			"public fn ()\n"
			"{	return	123.45	;\n"
			"}\n"
			;

	CBUFF	*cbuff  = el_compile (0, 0, 0, script, 0) ;
	CPPUNIT_ASSERT (cbuff != 0) ;
	CPPUNIT_ASSERT (el_loadtext (cbuff)) ;

	VALUE	res	= el_vexec   ("", "fn", 0, 0) ;

	CPPUNIT_ASSERT (res.tag->tag == V_DBL ) ;
	CPPUNIT_ASSERT (res.val.dbl  == 123.45) ;

	free	((void *)cbuff) ;
}

void	KBEL32Test::testEL32_3 ()
{
	const char *script =
			"public fn ()\n"
			"{	return	\"123.45\" ;\n"
			"}\n"
			;

	CBUFF	*cbuff  = el_compile (0, 0, 0, script, 0) ;
	CPPUNIT_ASSERT (cbuff != 0) ;
	CPPUNIT_ASSERT (el_loadtext (cbuff)) ;

	VALUE	res	= el_vexec   ("", "fn", 0, 0) ;

	CPPUNIT_ASSERT (res.tag->tag == V_STR) ;
	CPPUNIT_ASSERT (qstrcmp (res.val.str->text, "123.45") == 0) ;

	free	((void *)cbuff) ;
}

void	KBEL32Test::testEL32_4 ()
{
	const char *script =
			"public fn (a, b)\n"
			"{	return	a * b + 5 ;\n"
			"}\n"
			;

	CBUFF	*cbuff  = el_compile (0, 0, 0, script, 0) ;
	CPPUNIT_ASSERT (cbuff != 0) ;
	CPPUNIT_ASSERT (el_loadtext (cbuff)) ;

	VALUE	args[2]	;
	args[0]	= 2	;
	args[1]	= 3	;

	VALUE	res	= el_vexec   ("", "fn", 2, args) ;

	CPPUNIT_ASSERT (res.tag->tag == V_NUM) ;
	CPPUNIT_ASSERT (res.val.num  == 11   ) ;

	free	((void *)cbuff) ;
}

CppUnit::Test EXPORT_API *testSetup
	(	const QMap<QString,QString> *settings
	)
{
	CppUnit::TestSuite *suite = new CppUnit::TestSuite ("EL32") ;

	suite->addTest
	(	new CppUnit::TestCaller<KBEL32Test>
		(	"testEL32_1",
			&KBEL32Test::testEL32_1
		)
	)	;
	suite->addTest
	(	new CppUnit::TestCaller<KBEL32Test>
		(	"testEL32_2",
			&KBEL32Test::testEL32_2
		)
	)	;
	suite->addTest
	(	new CppUnit::TestCaller<KBEL32Test>
		(	"testEL32_3",
			&KBEL32Test::testEL32_3
		)
	)	;
	suite->addTest
	(	new CppUnit::TestCaller<KBEL32Test>
		(	"testEL32_4",
			&KBEL32Test::testEL32_3
		)
	)	;

	return	suite	;
}


#include	"kb_libloader.h"
#include	"factory.h"

TESTFACTORY	(KBTestEL32Factory,EL32,rkt_el32)
