/***************************************************************************
    file	         : attr.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	"kb_classes.h"
#include	"kb_node.h"
#include	"kb_attr.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif

class	KBAttrTest : public CppUnit::TestCase
{
	KBNode			*m_anode	;
	KBNode			*m_bnode	;
	QDict<QString>		m_adict		;

public	:

	KBAttrTest	() ;
	KBAttrTest	(std::string name) ;

	virtual	void		setUp			() ;
	virtual	void		tearDown		() ;

	void			attrStrName		() ;
	void			attrStrFlags		() ;
	void			attrStrRequired		() ;
	void			attrStrOwner		() ;
	void			attrStrOwnerName	() ;
	void			attrEscapeText		() ;
	void			attrEscapeTextNL	() ;
	void			attrStrPrintAttr	() ;
	void			attrStrValue1		() ;
	void			attrStrValue2		() ;
	void			attrStrValue3		() ;
	void			attrStrValue4		() ;
	void			attrStrReplicate	() ;
	void			attrStrSetValue1	() ;

	void			attrIntValue1		() ;
	void			attrIntValue2		() ;
	void			attrIntValue3		() ;
	void			attrIntValue4		() ;
	void			attrIntSetValue1	() ;

	void			attrUIntValue1		() ;
	void			attrUIntValue2		() ;
	void			attrUIntValue3		() ;
	void			attrUIntSetValue1	() ;

	void			attrBoolValue1		() ;
	void			attrBoolValue2		() ;
	void			attrBoolValue3		() ;
	void			attrBoolValue4		() ;
	void			attrBoolValue5		() ;
	void			attrBoolValue6		() ;
	void			attrBoolValue7		() ;
	void			attrBoolSetValue1	() ;

	static CppUnit::Test	*suite			(const QMap<QString,QString> *) ;
}	;

KBAttrTest::KBAttrTest ()
{
	m_adict.setAutoDelete (true) ;
}

KBAttrTest::KBAttrTest
	(	std::string	name
	)
	:
	CppUnit::TestCase (name)
{
	m_adict.setAutoDelete (true) ;
}

void	KBAttrTest::setUp ()
{
	m_anode	= new KBNode (0, "KBANode") ;
	m_bnode	= new KBNode (0, "KBBNode") ;

	m_adict.insert ("v_Empty", new QString(""   )) ;
	m_adict.insert ("v_0",     new QString("0"  )) ;
	m_adict.insert ("v_42",    new QString("42" )) ;
	m_adict.insert ("v_Yes",   new QString("Yes")) ;
	m_adict.insert ("v_No",    new QString("No" )) ;
}

void	KBAttrTest::tearDown ()
{
	delete	m_anode	;
	m_anode	= 0	;
	delete	m_bnode	;
	m_bnode	= 0	;

	m_adict.clear()	;
}

void	KBAttrTest::attrStrName ()
{
	KBAttrStr attr (m_anode, "str1", "str1_value", KAF_NOTES) ;
	CPPUNIT_ASSERT (attr.getName () == "str1") ;
}

void	KBAttrTest::attrStrFlags ()
{
	KBAttrStr attr (m_anode, "str1", "str1_value", KAF_NOTES) ;
	CPPUNIT_ASSERT (attr.getFlags() == KAF_NOTES) ;
}

void	KBAttrTest::attrStrRequired ()
{
	KBAttrStr attr (m_anode, "str1", "str1_value", KAF_REQD) ;
	CPPUNIT_ASSERT (attr.required()) ;
}

void	KBAttrTest::attrStrOwner ()
{
	KBAttrStr attr (m_anode, "str1", "str1_value", KAF_REQD) ;
	CPPUNIT_ASSERT (attr.getOwner() == m_anode) ;
}

void	KBAttrTest::attrStrOwnerName ()
{
	KBAttrStr attr (m_anode, "str1", "str1_value", KAF_REQD) ;
	CPPUNIT_ASSERT (qstrcmp(attr.getOwnerName(), "KBNode") == 0) ;
}

void	KBAttrTest::attrEscapeText ()
{
	QString	  text	= KBAttr::escapeText("12<>&\"\'\n", false) ;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Got [%1]").arg(text)),
		text == "12&lt;&gt;&amp;&quot;&#039;\n"
	)	;
}

void	KBAttrTest::attrEscapeTextNL()
{
	QString	  text	= KBAttr::escapeText("12<>&\"\'\n", true) ;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Got [%1]").arg(text)),
		text == "12&lt;&gt;&amp;&quot;&#039;&#010;"
	)	;
}

void	KBAttrTest::attrStrPrintAttr()
{
	KBAttrStr attr (m_anode, "str1", "str1_value", KAF_REQD) ;
	QString	  text ;
	QString	  null ;
	attr.printAttr (text, null, 0, false) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Got [%1]").arg(text)),
		text == " str1=\"str1_value\""
	)	;
}

void	KBAttrTest::attrStrValue1 ()
{
	KBAttrStr attr (m_anode, "str1", "str1_value", KAF_NOTES) ;
	CPPUNIT_ASSERT (attr.getValue() == "str1_value") ;
}

void	KBAttrTest::attrStrValue2 ()
{
	KBAttrStr attr (m_anode, "v_0", m_adict, KAF_NOTES) ;
	CPPUNIT_ASSERT (attr.getValue() == "0") ;
}

void	KBAttrTest::attrStrValue3 ()
{
	KBAttrStr attr (m_anode, "v_Empty", m_adict, KAF_NOTES) ;
	CPPUNIT_ASSERT (attr.getValue() == "") ;
}

void	KBAttrTest::attrStrValue4 ()
{
	KBAttrStr attr (m_anode, "v_None", m_adict, KAF_NOTES) ;
	CPPUNIT_ASSERT (attr.getValue() == "") ;
}

void	KBAttrTest::attrStrReplicate ()
{
	KBAttrStr att1 (m_anode, "str1", "str1_value", KAF_NOTES) ;
	KBAttrStr att2 (m_bnode, "str1", m_anode,      KAF_NOTES) ;
	CPPUNIT_ASSERT (att2.getValue() == "str1_value") ;
}

void	KBAttrTest::attrStrSetValue1 ()
{
	KBAttrStr attr (m_anode, "str1", "", KAF_NOTES) ;
	CPPUNIT_ASSERT (attr.getValue() == "") ;
	attr.setValue  ("one_two") ;
	CPPUNIT_ASSERT (attr.getValue() == "one_two") ;
}

void	KBAttrTest::attrIntValue1 ()
{
	KBAttrInt attr (m_anode, "int1", "", KAF_NOTES) ;
	CPPUNIT_ASSERT (attr.getValue   () == "") ;
	CPPUNIT_ASSERT (attr.getIntValue() ==  0) ;
}

void	KBAttrTest::attrIntValue2 ()
{
	KBAttrInt attr (m_anode, "int2", "42", KAF_NOTES) ;
	CPPUNIT_ASSERT (attr.getValue   () == "42") ;
	CPPUNIT_ASSERT (attr.getIntValue() ==   42) ;
}

void	KBAttrTest::attrIntValue3 ()
{
	KBAttrInt attr (m_anode, "int3", 99, KAF_NOTES) ;
	CPPUNIT_ASSERT (attr.getValue   () == "99") ;
	CPPUNIT_ASSERT (attr.getIntValue() ==   99) ;
}

void	KBAttrTest::attrIntValue4 ()
{
	KBAttrInt attr (m_anode, "int4", "-66", KAF_NOTES) ;
	CPPUNIT_ASSERT (attr.getValue   () == "-66") ;
	CPPUNIT_ASSERT (attr.getIntValue() ==  -66) ;
}

void	KBAttrTest::attrIntSetValue1 ()
{
	KBAttrInt attr (m_anode, "int5", "43", KAF_NOTES) ;
	CPPUNIT_ASSERT (attr.getIntValue() == 43) ;
	attr.setValue  (44) ;
	CPPUNIT_ASSERT (attr.getIntValue() == 44) ;
}

void	KBAttrTest::attrUIntValue1 ()
{
	KBAttrUInt attr (m_anode, "uint1", "", KAF_NOTES) ;
	CPPUNIT_ASSERT (attr.getValue   () == "") ;
	CPPUNIT_ASSERT (attr.getIntValue() ==  0) ;
}

void	KBAttrTest::attrUIntValue2 ()
{
	KBAttrUInt attr (m_anode, "uint2", "42", KAF_NOTES) ;
	CPPUNIT_ASSERT (attr.getValue   () == "42") ;
	CPPUNIT_ASSERT (attr.getIntValue() ==   42) ;
}

void	KBAttrTest::attrUIntValue3 ()
{
	KBAttrUInt attr (m_anode, "uint3", 99, KAF_NOTES) ;
	CPPUNIT_ASSERT (attr.getValue   () == "99") ;
	CPPUNIT_ASSERT (attr.getIntValue() ==   99) ;
}

void	KBAttrTest::attrUIntSetValue1 ()
{
	KBAttrUInt attr (m_anode, "uint4", "55", KAF_NOTES) ;
	CPPUNIT_ASSERT (attr.getIntValue() == 55) ;
	attr.setValue  (56) ;
	CPPUNIT_ASSERT (attr.getIntValue() == 56) ;
}

void	KBAttrTest::attrBoolValue1 ()
{
	KBAttrBool attr (m_anode, "bool1", QString(""), KAF_NOTES) ;
	CPPUNIT_ASSERT  (attr.getValue    () == ""   ) ;
	CPPUNIT_ASSERT  (attr.getBoolValue() == false) ;
}

void	KBAttrTest::attrBoolValue2 ()
{
	KBAttrBool attr (m_anode, "bool2", QString("No"), KAF_NOTES) ;
	CPPUNIT_ASSERT  (attr.getValue    () == "No" ) ;
	CPPUNIT_ASSERT  (attr.getBoolValue() == false) ;
}

void	KBAttrTest::attrBoolValue3 ()
{
	KBAttrBool attr (m_anode, "bool3", QString("Yes"), KAF_NOTES) ;
	CPPUNIT_ASSERT  (attr.getValue    () == "Yes") ;
	CPPUNIT_ASSERT  (attr.getBoolValue() == true ) ;
}

void	KBAttrTest::attrBoolValue4 ()
{
	KBAttrBool attr (m_anode, "bool4", false, KAF_NOTES) ;
	CPPUNIT_ASSERT  (attr.getValue    () == "No" ) ;
	CPPUNIT_ASSERT  (attr.getBoolValue() == false) ;
}

void	KBAttrTest::attrBoolValue5 ()
{
	KBAttrBool attr (m_anode, "bool5", true, KAF_NOTES) ;
	CPPUNIT_ASSERT  (attr.getValue    () == "Yes") ;
	CPPUNIT_ASSERT  (attr.getBoolValue() == true ) ;
}

void	KBAttrTest::attrBoolValue6 ()
{
	KBAttrBool attr (m_anode, "bool6", "No",  KAF_NOTES) ;
	CPPUNIT_ASSERT  (attr.getValue    () == "No" ) ;
	CPPUNIT_ASSERT  (attr.getBoolValue() == false) ;
}

void	KBAttrTest::attrBoolValue7 ()
{
	KBAttrBool attr (m_anode, "bool7", "Yes", KAF_NOTES) ;
	CPPUNIT_ASSERT  (attr.getValue    () == "Yes") ;
	CPPUNIT_ASSERT  (attr.getBoolValue() == true ) ;
}

void	KBAttrTest::attrBoolSetValue1 ()
{
	KBAttrBool attr (m_anode, "bool8", QString("No"), KAF_NOTES) ;
	CPPUNIT_ASSERT (attr.getBoolValue() == false) ;
	attr.setValue  (true ) ;
	CPPUNIT_ASSERT (attr.getBoolValue() == true ) ;
	attr.setValue  (false) ;
	CPPUNIT_ASSERT (attr.getBoolValue() == false) ;
}

CppUnit::Test
	*KBAttrTest::suite
	(	const QMap<QString,QString> *
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Attr") ;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrStrName", 
			&KBAttrTest::attrStrName 
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrStrFlags",
			&KBAttrTest::attrStrFlags
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrStrRequired",
			&KBAttrTest::attrStrRequired
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrStrOwner",
			&KBAttrTest::attrStrOwner
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrStrOwnerName",
			&KBAttrTest::attrStrOwnerName
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrEscapeText",
			&KBAttrTest::attrEscapeText
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrEscapeTextNL",
			&KBAttrTest::attrEscapeTextNL
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrStrPrintAttr",
			&KBAttrTest::attrStrPrintAttr
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrStrValue1",
			&KBAttrTest::attrStrValue1
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrStrValue2",
			&KBAttrTest::attrStrValue2
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrStrValue3",
			&KBAttrTest::attrStrValue3
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrStrValue4",
			&KBAttrTest::attrStrValue4
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrStrReplicate",
			&KBAttrTest::attrStrReplicate
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrStrSetValue1",
			&KBAttrTest::attrStrSetValue1
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrIntValue1",
			&KBAttrTest::attrIntValue1
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrIntValue2",
			&KBAttrTest::attrIntValue2
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrIntValue3",
			&KBAttrTest::attrIntValue3
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrIntValue4",
			&KBAttrTest::attrIntValue4
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrIntSetValue1",
			&KBAttrTest::attrIntSetValue1
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrUIntValue1",
			&KBAttrTest::attrUIntValue1
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrUIntValue2",
			&KBAttrTest::attrUIntValue2
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrUIntValue3",
			&KBAttrTest::attrUIntValue3
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrUIntSetValue1",
			&KBAttrTest::attrUIntSetValue1
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrBoolValue1",
			&KBAttrTest::attrBoolValue1
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrBoolValue2",
			&KBAttrTest::attrBoolValue2
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrBoolValue3",
			&KBAttrTest::attrBoolValue3
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrBoolValue4",
			&KBAttrTest::attrBoolValue4
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrBoolValue5",
			&KBAttrTest::attrBoolValue5
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrBoolValue6",
			&KBAttrTest::attrBoolValue6
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrBoolValue7",
			&KBAttrTest::attrBoolValue7
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBAttrTest>
		(	"attrBoolSetValue1",
			&KBAttrTest::attrBoolSetValue1
	)	)	;
	return s ;
}

CppUnit::Test EXPORT_API *attrTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBAttrTest::suite (__settings) ;
}
