/***************************************************************************
    file	         : skin.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qstring.h>
#include	<qmap.h>

#include	"kb_classes.h"
#include	"kb_dom.h"
#include	"kb_skin.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif

class	KBSkinTest : public CppUnit::TestCase
{
	KBSkin			*m_skinA	;
	KBSkin			*m_skinB	;

public	:

	KBSkinTest	() ;
	KBSkinTest	(std::string name) ;

	virtual	void		setUp			() ;
	virtual	void		tearDown		() ;

	void			skinTest1		() ;
	void			skinTest2		() ;

	static CppUnit::Test	*suite			(const QMap<QString,QString> *) ;
}	;

struct	SkinElement
{	const char	*m_name		;
	const char	*m_fgColor	;
	const char	*m_bgColor	;
	const char	*m_font		;
}	;

static	SkinElement	elements[] =
{
{	"elem_1",	"0xff0000",	"0x0000ff",	"Lucida Grande:13:50:1"	},
{	0,		0,		0,		0			}
}	;

KBSkinTest::KBSkinTest ()
{
	m_skinA	= 0 ;
	m_skinB	= 0 ;
}

KBSkinTest::KBSkinTest
	(	std::string	name
	)
	:
	CppUnit::TestCase (name)
{
	m_skinA	= 0 ;
	m_skinB	= 0 ;
}

void	KBSkinTest::setUp ()
{
	m_skinA	= new KBSkin ;
	for (SkinElement *se = &elements[0] ; se->m_name != 0 ; se += 1)
		m_skinA->add (se->m_name, se->m_fgColor, se->m_bgColor, se->m_font) ;


	KBDomDocument	doc	;
	if (!doc.loadFile ("TestSkin.xml", "test/kbase", getenv("TESTPATH")))
		CPPUNIT_FAIL (_STDSTR(doc.lastError().getMessage())) ;

	m_skinB	= new KBSkin (doc.documentElement()) ;
}

void	KBSkinTest::tearDown ()
{
	delete	m_skinA	;
	m_skinA	= 0 	;
	delete	m_skinB	;
	m_skinB	= 0 	;
}

void	KBSkinTest::skinTest1 ()
{
	CPPUNIT_ASSERT (m_skinA->fgColor("elem_1") == "0xff0000") ;
	CPPUNIT_ASSERT (m_skinA->bgColor("elem_1") == "0x0000ff") ;
	CPPUNIT_ASSERT (m_skinA->font   ("elem_1") == "Lucida Grande:13:50:1") ;
}

void	KBSkinTest::skinTest2 ()
{
	if (m_skinB == 0)
	{
		CPPUNIT_FAIL ("No skin for skinTest2") ;
		return	;
	}

	CPPUNIT_ASSERT (m_skinB->fgColor("First" ) == "0x0000ff") ;
	CPPUNIT_ASSERT (m_skinB->bgColor("First" ) == "0xffff00") ;
	CPPUNIT_ASSERT (m_skinB->font   ("First" ) == "Lucida Grande:13:25:1") ;
	CPPUNIT_ASSERT (m_skinB->fgColor("Second") == "0xff0000") ;
	CPPUNIT_ASSERT (m_skinB->bgColor("Second") == "0x00ffff") ;
	CPPUNIT_ASSERT (m_skinB->font   ("Second") == "Lucida Grande:13:75:0") ;
	CPPUNIT_ASSERT (m_skinB->fgColor("Missed").isEmpty()) ;
	CPPUNIT_ASSERT (m_skinB->bgColor("Missed").isEmpty()) ;
	CPPUNIT_ASSERT (m_skinB->font   ("Missed").isEmpty()) ;
}


CppUnit::Test
	*KBSkinTest::suite
	(	const QMap<QString,QString> *__settings
	)

{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Skin") ;
	s->addTest
	(	new CppUnit::TestCaller<KBSkinTest>
		(	"skinTest1", 
			&KBSkinTest::skinTest1
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBSkinTest>
		(	"skinTest2", 
			&KBSkinTest::skinTest2
	)	)	;
	return s ;
}

CppUnit::Test EXPORT_API *skinTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBSkinTest::suite (__settings) ;
}
