/***************************************************************************
    file	         : navigate.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#define		private	public

#include	<qmainwindow.h>
#include	<qstring.h>
#include	<qmap.h>
#include	<qregexp.h>

#if		__KB_PLAT_WIN32
#define		PAUSE	_sleep (1)
#else
#include	<unistd.h>
#define		PAUSE	usleep(100000)
#endif

#include	"rk_application.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_button.h"
#include	"kb_dom.h"
#include	"kb_field.h"
#include	"kb_ascii.h"

#include	"accessor.h"
#include	"tester.h"
#include	"tester_kbase.h"

#include	"api_if.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_REPORTS	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_REPORTS
#endif



class	KBMiscNavigateTest : public KBAPIIF
{
	KBDBInfo		*m_dbInfo	;
	TestOpenForm		*m_form		;

	KBField			*field_id	;
	KBField			*field_t	;
	KBField			*field_n	;

	QString			nullToEmpty	(const QString &) ;

public	:

	KBMiscNavigateTest	() ;
	KBMiscNavigateTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testNavigate0	() ;
	void			testNavigate1	() ;
	void			testNavigate2	() ;
	void			testNavigate3	() ;
	void			testNavigate4	() ;
	void			testNavigate5	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBMiscNavigateTest::KBMiscNavigateTest ()
{
}

KBMiscNavigateTest::KBMiscNavigateTest
	(	std::string		name
	)
	:
	KBAPIIF	(name)
{
}

void	KBMiscNavigateTest::setUp ()
{
	KBError	 error	;

	m_dbInfo = rekallDBInfo ("Misc.rkl", "test/misc") ;
	m_form	 = rekallTestOpenForm (m_dbInfo, "misc", "Navigate", error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;

	m_docroot	= m_form->m_form	;

	field_id	= getField ("id", "id") ;
	field_t		= getField ("t",  "t" ) ;
	field_n		= getField ("n",  "n" ) ;

	KBControl	*c_id_0	= getControl	(field_id, 0, 0) ;
	RKLineEdit	*e_id_0	= getRKLineEdit	(c_id_0) ;

	e_id_0->setActiveWindow() ;
	for (uint idx = 0 ; idx < 10 ; idx += 1)
		if (!e_id_0->isActiveWindow())
		{	qApp->syncX() ;
			PAUSE ;
			qApp->processEvents() ;
		}

	CPPUNIT_ASSERT
	(	e_id_0->isActiveWindow()
	)	;

	QMouseEvent me (QEvent::MouseButtonPress, QPoint(2,2), 0, 0) ;
	me.spont = true ;
	qApp->notify (e_id_0, &me) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus in unexpected item: %1").arg(KBTestAccessor::currentItem(m_form->m_form)->getName())),
		KBTestAccessor::currentItem(m_form->m_form) == field_id
	)	;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus at unexpected row: %1").arg(KBTestAccessor::currentQRow(m_form->m_form))),
		KBTestAccessor::currentQRow(m_form->m_form) == 0
	)	;
}

void	KBMiscNavigateTest::tearDown ()
{
	rekallTestCloseForm ()	;
	delete	m_dbInfo	;
}

void	KBMiscNavigateTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBMiscNavigateTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

QString	KBMiscNavigateTest::nullToEmpty
	(	const QString	&text
	)
{
	return	text.isNull() ? QString("") : text ;
}

void	KBMiscNavigateTest::testNavigate0 ()
{
	QKeyEvent key (QEvent::KeyRelease, Qt::Key_Menu, 0, 0) ;
	m_form->m_form->KBNavigator::keyStroke (field_id, &key) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus in unexpected item: %1").arg(KBTestAccessor::currentItem(m_form->m_form)->getName())),
		KBTestAccessor::currentItem(m_form->m_form) == field_id
	)	;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus at unexpected row: %1").arg(KBTestAccessor::currentQRow(m_form->m_form))),
		KBTestAccessor::currentQRow(m_form->m_form) == 0
	)	;
}

void	KBMiscNavigateTest::testNavigate1 ()
{
	QKeyEvent key (QEvent::KeyRelease, Qt::Key_Tab, 0, 0) ;
	m_form->m_form->KBNavigator::keyStroke (field_id, &key) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus in unexpected item: %1").arg(KBTestAccessor::currentItem(m_form->m_form)->getName())),
		KBTestAccessor::currentItem(m_form->m_form) == field_t
	)	;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus at unexpected row: %1").arg(KBTestAccessor::currentQRow(m_form->m_form))),
		KBTestAccessor::currentQRow(m_form->m_form) == 0
	)	;
}

void	KBMiscNavigateTest::testNavigate2 ()
{
	QKeyEvent key (QEvent::KeyRelease, Qt::Key_Down, 0, 0) ;
	m_form->m_form->KBNavigator::keyStroke (field_id, &key) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus in unexpected item: %1").arg(KBTestAccessor::currentItem(m_form->m_form)->getName())),
		KBTestAccessor::currentItem(m_form->m_form) == field_id
	)	;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus at unexpected row: %1").arg(KBTestAccessor::currentQRow(m_form->m_form))),
		KBTestAccessor::currentQRow(m_form->m_form) == 1
	)	;
}

void	KBMiscNavigateTest::testNavigate3 ()
{
	QKeyEvent key (QEvent::KeyRelease, Qt::Key_BackTab, 0, 0) ;
	m_form->m_form->KBNavigator::keyStroke (field_t, &key) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus in unexpected item: %1").arg(KBTestAccessor::currentItem(m_form->m_form)->getName())),
		KBTestAccessor::currentItem(m_form->m_form) == field_id
	)	;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus at unexpected row: %1").arg(KBTestAccessor::currentQRow(m_form->m_form))),
		KBTestAccessor::currentQRow(m_form->m_form) == 0
	)	;
}

void	KBMiscNavigateTest::testNavigate4 ()
{
	QKeyEvent key (QEvent::KeyRelease, Qt::Key_Up, 0, 0) ;
	m_form->m_form->KBNavigator::keyStroke (field_id, &key) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus in unexpected item: %1").arg(KBTestAccessor::currentItem(m_form->m_form)->getName())),
		KBTestAccessor::currentItem(m_form->m_form) == field_id
	)	;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus at unexpected row: %1").arg(KBTestAccessor::currentQRow(m_form->m_form))),
		KBTestAccessor::currentQRow(m_form->m_form) == 0
	)	;
}

void	KBMiscNavigateTest::testNavigate5 ()
{
	KBControl  *ctrl_t_0	= getControl	(field_t, 0, 0) ;
	RKLineEdit *le_t_0	= getRKLineEdit	(ctrl_t_0)	;

	KBField	   *status	= getField	("status/status", "status") ;
	KBControl  *ctrl_status	= getControl	(status,  0, 0) ;
	RKLineEdit *le_status	= getRKLineEdit	(ctrl_status  )	;

	QMouseEvent me (QEvent::MouseButtonPress, QPoint(2,2), 0, 0) ;
	me.spont = true ;
	qApp->notify (le_t_0, &me) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus in unexpected item: %1").arg(KBTestAccessor::currentItem(m_form->m_form)->getName())),
		KBTestAccessor::currentItem(m_form->m_form) == field_t
	)	;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus at unexpected row: %1").arg(KBTestAccessor::currentQRow(m_form->m_form))),
		KBTestAccessor::currentQRow(m_form->m_form) == 0
	)	;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(nullToEmpty(le_t_0->text())),
		_STDSTR(QString("One"))
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(nullToEmpty(le_status->text())),
		_STDSTR(QString(""))
	)	;

	le_t_0->setText ("OneOne") ;

	QKeyEvent key (QEvent::KeyRelease, Qt::Key_Enter, 0, Qt::ControlButton) ;
	m_form->m_form->KBNavigator::keyStroke (field_t, &key) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(nullToEmpty(le_t_0->text())),
		_STDSTR(QString("OneOne"))
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(nullToEmpty(le_status->text())),
		_STDSTR(QString("actSave"))
	)	;
}

CppUnit::Test
	*KBMiscNavigateTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("navigate") ;

	if (rekallTestSkip ("navigate")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;

	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBMiscNavigateTest> ("noDisplay", &KBMiscNavigateTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBMiscNavigateTest> ("withGUI",   &KBMiscNavigateTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBMiscNavigateTest>
			(	"navigate0",
				&KBMiscNavigateTest::testNavigate0
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBMiscNavigateTest>
			(	"navigate1",
				&KBMiscNavigateTest::testNavigate1
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBMiscNavigateTest>
			(	"navigate2",
				&KBMiscNavigateTest::testNavigate2
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBMiscNavigateTest>
			(	"navigate3",
				&KBMiscNavigateTest::testNavigate3
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBMiscNavigateTest>
			(	"navigate4",
				&KBMiscNavigateTest::testNavigate4
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBMiscNavigateTest>
			(	"navigate5",
				&KBMiscNavigateTest::testNavigate5
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_REPORTS *miscNavigateTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBMiscNavigateTest::suite (__settings) ;
}
