/***************************************************************************
    file	         : kbprop.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qstring.h>
#include	<qmap.h>

#include	"rk_application.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_dom.h"
#include	"kb_ascii.h"
#include	"kb_formcopier.h"

#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBKBPropTest : public CppUnit::TestCase
{
	TestOpenForm		*m_form		;

	KBNode			*getNode	(const char *, const char *) ;

	void			checkHasAll	(const QStringList &, const char **) ;

public	:

	KBKBPropTest	() ;
	KBKBPropTest	(std::string name)	;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testLabel	() ;
	void			testButton	() ;
	void			testField	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBKBPropTest::KBKBPropTest ()
{
}

KBKBPropTest::KBKBPropTest
	(	std::string		name
	)
	:
	CppUnit::TestCase (name)
{
}

void	KBKBPropTest::setUp ()
{
	KBError	error	;
	m_form	= rekallTestOpenFormMode (0, "nodes", "Nodes", KB::ShowAsDesign, error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;
}

void	KBKBPropTest::tearDown ()
{
	rekallTestCloseForm () ;
}

void	KBKBPropTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBKBPropTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

KBNode	*KBKBPropTest::getNode
	(	const char	*name,
		const char	*type
	)
{
	KBNode	 *target = m_form->m_form->getNamedNode(name) ;

	CPPUNIT_ASSERT (target != 0) ;
	CPPUNIT_ASSERT (target->isA(type)) ;

	return	target	;
}

void	KBKBPropTest::checkHasAll
	(	const QStringList	&props,
		const char		**pl
	)
{
	uint		count	= 0 ;
	const char	*p	;

	while ((p = pl[count]) != 0)
	{
		CPPUNIT_ASSERT_MESSAGE
		(	_STDSTR(QString("Missing property: %1").arg(p)),
			props.findIndex(p) >= 0
		)	;
		count	+= 1 ;
	}

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Expected %1 props, got %2").arg(count).arg(props.count())),
		count == props.count()
	)	;
}

void	KBKBPropTest::testLabel ()
{
	KBNode 	*node	= getNode ("label", "KBLabel") ;
	QStringList	props	;
	node->enumKBProperty	(props) ;

	static	const char *pl[] =
	{
		"text",
		"visible",
		"enabled",
		"__parent__",
		"__block__",
		"__root__",
		"name",
		"element",
		0
	}	;
	checkHasAll (props, pl) ;
}

void	KBKBPropTest::testButton ()
{
	KBNode 	*node	= getNode ("button", "KBButton") ;
	QStringList	props	;
	node->enumKBProperty	(props) ;

	static	const char *pl[] =
	{
		"text",
		"visible",
		"enabled",
		"__parent__",
		"__block__",
		"__root__",
		"name",
		"element",
		0
	}	;
	checkHasAll (props, pl) ;
}

void	KBKBPropTest::testField ()
{
	KBNode 	*node	= getNode ("field", "KBField") ;
	QStringList	props	;
	node->enumKBProperty	(props) ;

	static	const char *pl[] =
	{
		"value",
		"visible",
		"enabled",
		"__parent__",
		"__block__",
		"__root__",
		"name",
		"element",
		0
	}	;
	checkHasAll (props, pl) ;
}

CppUnit::Test
	*KBKBPropTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Label") ;

	if (rekallTestSkip ("CopyPaste")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;


	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBKBPropTest> ("noDisplay", &KBKBPropTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBKBPropTest> ("withGUI",   &KBKBPropTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBKBPropTest>
			(	"testLabel",
				&KBKBPropTest::testLabel
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBKBPropTest>
			(	"testButton",
				&KBKBPropTest::testButton
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBKBPropTest>
			(	"testField",
				&KBKBPropTest::testField
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_API *kbPropTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBKBPropTest::suite (__settings) ;
}
