/***************************************************************************
    file	         : popups.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#define		private	public

#include	<qmainwindow.h>
#include	<qstring.h>
#include	<qmap.h>
#include	<qregexp.h>

#if		__KB_PLAT_WIN32
#define		PAUSE	_sleep (1)
#else
#include	<unistd.h>
#define		PAUSE	usleep(100000)
#endif

#include	"rk_application.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_button.h"
#include	"kb_dom.h"
#include	"kb_field.h"
#include	"kb_ascii.h"
#include	"kb_macro.h"
#include	"kb_test.h"

#include	"accessor.h"
#include	"tester.h"
#include	"tester_kbase.h"

#include	"api_if.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_REPORTS	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_REPORTS
#endif



class	KBTestPopupsTest : public KBAPIIF
{
	KBDBInfo		*m_dbInfo	;
	TestOpenForm		*m_form		;
	KBForm			*m_docroot	;

	KBTest			*getTest	(const QString &) ;

public	:

	KBTestPopupsTest	() ;
	KBTestPopupsTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testPopups0	() ;
	void			testPopups1	() ;
	void			testPopups2	() ;
	void			testPopups3	() ;
	void			testPopups4	() ;
	void			testPopups5	() ;
	void			testPopups6	() ;
	void			testPopups7	() ;
	void			testPopups8	() ;
	void			testPopups9	() ;
	void			testPopups10	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBTestPopupsTest::KBTestPopupsTest ()
{
}

KBTestPopupsTest::KBTestPopupsTest
	(	std::string		name
	)
	:
	KBAPIIF	(name)
{
}

KBTest	*KBTestPopupsTest::getTest
	(	const QString		&name
	)
{
	LITER
	(	KBTest,
		m_docroot->getTests(),
		t,
		if (t->getName() == name) return t ;
	)

	CPPUNIT_FAIL (_STDSTR(QString("Cannot find test '%1'").arg(name))) ;
	return	0 ;
}

void	KBTestPopupsTest::setUp ()
{
	KBError	 error	;

	m_dbInfo = rekallDBInfo ("Test.rkl", "test/test") ;
	m_form	 = rekallTestOpenForm (m_dbInfo, "test", "Popups", error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;

	m_docroot	= m_form->m_form ;
}

void	KBTestPopupsTest::tearDown ()
{
	rekallTestCloseForm ()	;
	delete	m_dbInfo	;
}

void	KBTestPopupsTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBTestPopupsTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

void	KBTestPopupsTest::testPopups0 ()
{
	KBTest	*query_yes = getTest("Query_Yes") ;
	KBValue	resval	   ;

	KBTest::setTestMode (KBScriptIF::testSuite) ;
	KBScriptError *rc  = query_yes->execute (resval, 0, 0, false) ;
	KBTest::setTestMode (KBScriptIF::testOff  ) ;
	if (rc != 0)
		CPPUNIT_FAIL(_STDSTR(rc->error().getMessage() + ": " + rc->error().getDetails())) ;
}

void	KBTestPopupsTest::testPopups1 ()
{
	KBTest	*query_no  = getTest("Query_No") ;
	KBValue	resval	   ;

	KBTest::setTestMode (KBScriptIF::testSuite) ;
	KBScriptError *rc  = query_no ->execute (resval, 0, 0, false) ;
	KBTest::setTestMode (KBScriptIF::testOff  ) ;
	if (rc != 0)
		CPPUNIT_FAIL(_STDSTR(rc->error().getMessage() + ": " + rc->error().getDetails())) ;
}

void	KBTestPopupsTest::testPopups2 ()
{
	KBTest	*query_yes = getTest("Query_Yes_Wrong") ;
	KBValue	resval	   ;

	KBTest::setTestMode (KBScriptIF::testSuite) ;
	KBScriptError *rc  = query_yes->execute (resval, 0, 0, false) ;
	KBTest::setTestMode (KBScriptIF::testOff  ) ;
	if (rc == 0)
		CPPUNIT_FAIL("Unexpected success") ;
	if (rc->error().getDetails().find("got 'NO' expected 'YES'") < 0)
		CPPUNIT_FAIL(_STDSTR(QString("Unexpected result: %1").arg(rc->error().getDetails()))) ;
}

void	KBTestPopupsTest::testPopups3 ()
{
	KBTest	*query_yes = getTest("Query_Yes_NoPopup") ;
	KBValue	resval	   ;

	KBTest::setTestMode (KBScriptIF::testSuite) ;
	KBScriptError *rc  = query_yes->execute (resval, 0, 0, false) ;
	KBTest::setTestMode (KBScriptIF::testOff  ) ;
	if (rc == 0)
		CPPUNIT_FAIL("Unexpected success") ;
	if (rc->error().getDetails().find("Popup expected") < 0)
		CPPUNIT_FAIL(_STDSTR(QString("Unexpected result: %1").arg(rc->error().getDetails()))) ;
}

void	KBTestPopupsTest::testPopups4 ()
{
	KBTest	*query_yes = getTest("Query_Yes_BadPopup") ;
	KBValue	resval	   ;

	KBTest::setTestMode (KBScriptIF::testSuite) ;
	KBScriptError *rc  = query_yes->execute (resval, 0, 0, false) ;
	KBTest::setTestMode (KBScriptIF::testOff  ) ;
	if (rc == 0)
		CPPUNIT_FAIL("Unexpected success") ;
	if (rc->error().getDetails().find("Popup expected") < 0)
		CPPUNIT_FAIL(_STDSTR(QString("Unexpected result: %1").arg(rc->error().getDetails()))) ;
}

void	KBTestPopupsTest::testPopups5 ()
{
	KBTest	*query_yes = getTest("Prompt_One") ;
	KBValue	resval	   ;

	KBTest::setTestMode (KBScriptIF::testSuite) ;
	KBScriptError *rc  = query_yes->execute (resval, 0, 0, false) ;
	KBTest::setTestMode (KBScriptIF::testOff  ) ;
	if (rc != 0)
		CPPUNIT_FAIL(_STDSTR(rc->error().getMessage() + ": " + rc->error().getDetails())) ;
}

void	KBTestPopupsTest::testPopups6 ()
{
	KBTest	*query_yes = getTest("Prompt_Two") ;
	KBValue	resval	   ;

	KBTest::setTestMode (KBScriptIF::testSuite) ;
	KBScriptError *rc  = query_yes->execute (resval, 0, 0, false) ;
	KBTest::setTestMode (KBScriptIF::testOff  ) ;
	if (rc != 0)
		CPPUNIT_FAIL(_STDSTR(rc->error().getMessage() + ": " + rc->error().getDetails())) ;
}

void	KBTestPopupsTest::testPopups7 ()
{
	KBTest	*query_yes = getTest("Prompt_Cancel") ;
	KBValue	resval	   ;

	KBTest::setTestMode (KBScriptIF::testSuite) ;
	KBScriptError *rc  = query_yes->execute (resval, 0, 0, false) ;
	KBTest::setTestMode (KBScriptIF::testOff  ) ;
	if (rc != 0)
		CPPUNIT_FAIL(_STDSTR(rc->error().getMessage() + ": " + rc->error().getDetails())) ;
}

void	KBTestPopupsTest::testPopups8 ()
{
	KBTest	*query_yes = getTest("Choice_A") ;
	KBValue	resval	   ;

	KBTest::setTestMode (KBScriptIF::testSuite) ;
	KBScriptError *rc  = query_yes->execute (resval, 0, 0, false) ;
	KBTest::setTestMode (KBScriptIF::testOff  ) ;
	if (rc != 0)
		CPPUNIT_FAIL(_STDSTR(rc->error().getMessage() + ": " + rc->error().getDetails())) ;
}

void	KBTestPopupsTest::testPopups9 ()
{
	KBTest	*query_yes = getTest("Choice_C") ;
	KBValue	resval	   ;

	KBTest::setTestMode (KBScriptIF::testSuite) ;
	KBScriptError *rc  = query_yes->execute (resval, 0, 0, false) ;
	KBTest::setTestMode (KBScriptIF::testOff  ) ;
	if (rc != 0)
		CPPUNIT_FAIL(_STDSTR(rc->error().getMessage() + ": " + rc->error().getDetails())) ;
}

void	KBTestPopupsTest::testPopups10 ()
{
	KBTest	*query_yes = getTest("Choice_Cancel") ;
	KBValue	resval	   ;

	KBTest::setTestMode (KBScriptIF::testSuite) ;
	KBScriptError *rc  = query_yes->execute (resval, 0, 0, false) ;
	KBTest::setTestMode (KBScriptIF::testOff  ) ;
	if (rc != 0)
		CPPUNIT_FAIL(_STDSTR(rc->error().getMessage() + ": " + rc->error().getDetails())) ;
}

CppUnit::Test
	*KBTestPopupsTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("popups") ;

	if (rekallTestSkip ("popups")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;

	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBTestPopupsTest> ("noDisplay", &KBTestPopupsTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBTestPopupsTest> ("withGUI",   &KBTestPopupsTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBTestPopupsTest>
			(	"testPopups0",
				&KBTestPopupsTest::testPopups0
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTestPopupsTest>
			(	"testPopups1",
				&KBTestPopupsTest::testPopups1
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTestPopupsTest>
			(	"testPopups2",
				&KBTestPopupsTest::testPopups2
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTestPopupsTest>
			(	"testPopups3",
				&KBTestPopupsTest::testPopups3
		)	)	;
//		s->addTest
//		(	new CppUnit::TestCaller<KBTestPopupsTest>
//			(	"testPopups4",
//				&KBTestPopupsTest::testPopups4
//		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTestPopupsTest>
			(	"testPopups5",
				&KBTestPopupsTest::testPopups5
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTestPopupsTest>
			(	"testPopups6",
				&KBTestPopupsTest::testPopups6
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTestPopupsTest>
			(	"testPopups7",
				&KBTestPopupsTest::testPopups7
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTestPopupsTest>
			(	"testPopups8",
				&KBTestPopupsTest::testPopups8
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTestPopupsTest>
			(	"testPopups9",
				&KBTestPopupsTest::testPopups9
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTestPopupsTest>
			(	"testPopups10",
				&KBTestPopupsTest::testPopups10
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_REPORTS *testsPopupsTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBTestPopupsTest::suite (__settings) ;
}
