/***************************************************************************
    file	         : popups_suites.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#define		private	public

#include	<qmainwindow.h>
#include	<qstring.h>
#include	<qmap.h>
#include	<qregexp.h>

#if		__KB_PLAT_WIN32
#define		PAUSE	_sleep (1)
#else
#include	<unistd.h>
#define		PAUSE	usleep(100000)
#endif

#include	"rk_application.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_button.h"
#include	"kb_dom.h"
#include	"kb_field.h"
#include	"kb_ascii.h"
#include	"kb_macro.h"
#include	"kb_test.h"
#include	"kb_testsuite.h"
#include	"kb_testsuiteresultsdlg.h"

#include	"accessor.h"
#include	"tester.h"
#include	"tester_kbase.h"

#include	"api_if.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_REPORTS	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_REPORTS
#endif



class	KBTestPopupsSuitesTest : public KBAPIIF
{
	KBDBInfo		*m_dbInfo	;
	TestOpenForm		*m_form		;
	KBForm			*m_docroot	;

	KBTest			*getTest	(const QString &) ;

public	:

	KBTestPopupsSuitesTest	() ;
	KBTestPopupsSuitesTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testPopups0	() ;
	void			testPopups1	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBTestPopupsSuitesTest::KBTestPopupsSuitesTest ()
{
}

KBTestPopupsSuitesTest::KBTestPopupsSuitesTest
	(	std::string		name
	)
	:
	KBAPIIF	(name)
{
}

KBTest	*KBTestPopupsSuitesTest::getTest
	(	const QString		&name
	)
{
	LITER
	(	KBTest,
		m_docroot->getTests(),
		t,
		if (t->getName() == name) return t ;
	)

	CPPUNIT_FAIL (_STDSTR(QString("Cannot find test '%1'").arg(name))) ;
	return	0 ;
}

void	KBTestPopupsSuitesTest::setUp ()
{
	KBError	 error	;

	m_dbInfo = rekallDBInfo ("Test.rkl", "test/test") ;
	m_form	 = rekallTestOpenForm (m_dbInfo, "test", "Popups", error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;

	m_docroot	= m_form->m_form ;
}

void	KBTestPopupsSuitesTest::tearDown ()
{
	rekallTestCloseForm ()	;
	delete	m_dbInfo	;
}

void	KBTestPopupsSuitesTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBTestPopupsSuitesTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

void	KBTestPopupsSuitesTest::testPopups0 ()
{

	KBTestSuite	*suite	= 0 ;

	TITER
	(	TestSuite,
		m_docroot->getChildren(),
		t,

		if (t->getName() == "Query_Show")
		{	suite	= t ;
			break	;
		}
	)

	CPPUNIT_ASSERT(suite != 0) ;

	KBTestSuiteResultsDlg tsrDlg ;

	KBTestSuite::executeStart   (m_docroot->getDocRoot()) ;
	suite->executeTests	    (true) ;
	KBTestSuite::executeResults (m_docroot->getDocRoot(), tsrDlg) ;
	KBTestSuite::executeFinish  (m_docroot->getDocRoot()) ;

	CPPUNIT_ASSERT_EQUAL((uint)1, tsrDlg.errorCount()) ;
}

void	KBTestPopupsSuitesTest::testPopups1 ()
{

	KBTestSuite	*suite	= 0 ;

	TITER
	(	TestSuite,
		m_docroot->getChildren(),
		t,

		if (t->getName() == "Query_None")
		{	suite	= t ;
			break	;
		}
	)

	CPPUNIT_ASSERT(suite != 0) ;

	KBTestSuiteResultsDlg tsrDlg ;

	KBTestSuite::executeStart   (m_docroot->getDocRoot()) ;
	suite->executeTests	    (true) ;
	KBTestSuite::executeResults (m_docroot->getDocRoot(), tsrDlg) ;
	KBTestSuite::executeFinish  (m_docroot->getDocRoot()) ;

	CPPUNIT_ASSERT_EQUAL((uint)1, tsrDlg.errorCount()) ;
}

CppUnit::Test
	*KBTestPopupsSuitesTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("PopupsSuites") ;

	if (rekallTestSkip ("PopupsSuites")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;

	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBTestPopupsSuitesTest> ("noDisplay", &KBTestPopupsSuitesTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBTestPopupsSuitesTest> ("withGUI",   &KBTestPopupsSuitesTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBTestPopupsSuitesTest>
			(	"testPopupsSuites0",
				&KBTestPopupsSuitesTest::testPopups0
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTestPopupsSuitesTest>
			(	"testPopupsSuites1",
				&KBTestPopupsSuitesTest::testPopups1
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_REPORTS *testsPopupsSuitesTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBTestPopupsSuitesTest::suite (__settings) ;
}
