/***************************************************************************
    file	         : verify.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qmainwindow.h>
#include	<qstring.h>
#include	<qmap.h>
#include	<qregexp.h>

#include	"rk_application.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_docroot.h"
#include	"kb_dom.h"
#include	"kb_field.h"
#include	"kb_choice.h"
#include	"kb_link.h"
#include	"kb_button.h"
#include	"kb_label.h"
#include	"kb_ascii.h"
#include	"kb_macro.h"
#include	"kb_test.h"

#include	"accessor.h"
#include	"tester.h"
#include	"tester_kbase.h"

#include	"api_if.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_REPORTS	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_REPORTS
#endif



class	KBTestVerifyTest : public KBAPIIF
{
	KBDBInfo		*m_dbInfo	;
	TestOpenForm		*m_form		;
	KBMacroExec		*m_macro	;

	QString			nullToEmpty	(const QString &) ;

public	:

	KBTestVerifyTest	() ;
	KBTestVerifyTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testVerify1	() ;
	void			testVerify2	() ;
	void			testVerify3	() ;
	void			testVerify4	() ;
	void			testVerify5	() ;
	void			testVerify6	() ;
	void			testVerify7	() ;
	void			testVerify8	() ;
	void			testVerify9	() ;
	void			testVerify10	() ;
	void			testVerify11	() ;
	void			testVerify12	() ;
	void			testVerify13	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBTestVerifyTest::KBTestVerifyTest ()
{
}

KBTestVerifyTest::KBTestVerifyTest
	(	std::string		name
	)
	:
	KBAPIIF	(name)
{
}

void	KBTestVerifyTest::setUp ()
{
	KBError	error	;

	m_dbInfo	= rekallDBInfo ("Test.rkl", "test/test") ;
	m_form		= rekallTestOpenForm (m_dbInfo, "test", "Verify", error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage() + ": " + error.getDetails()),
		m_form != 0
	)	;

	m_docroot	= m_form->m_form ;
	m_macro		= new KBMacroExec
			  (	m_docroot->getDocRoot()->getDocLocation().dbInfo(),
				m_docroot->getDocRoot()->getDocLocation().server(),
				"tests"
			  )	;
	m_macro->setName ("verify") ;

}

void	KBTestVerifyTest::tearDown ()
{
	rekallTestCloseForm ()	;
	delete	m_dbInfo	;
}

void	KBTestVerifyTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBTestVerifyTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

QString	KBTestVerifyTest::nullToEmpty
	(	const QString	&text
	)
{
	return	text.isNull() ? QString("") : text ;
}

void	KBTestVerifyTest::testVerify1 ()
{
	KBField		*srce	= getField  ("Field_0", "Field_0") ;
	srce->setValue	(0, KBValue("A.VALUE")) ;

	QStringList	args	;
	KBError		error	;

	args.append	(srce->getPath(false))	;
	args.append	(srce->getName())	;
	args.append	("0")			;
	args.append	(QString("%1:A.VALUE").arg(KB::ITString)) ;

	if (!m_macro->append ("VerifyValue", args, QString::null, error))
		CPPUNIT_FAIL (_STDSTR(error.getMessage())) ;

	KBTest::setTestMode(KBScriptIF::testSuite) ;
	KBScriptError *rc = m_macro->execute (m_docroot) ;
	KBTest::setTestMode(KBScriptIF::testOff  ) ;
	if (rc != 0)
		CPPUNIT_FAIL(rc->error().getMessage() + ": " + rc->error().getDetails()) ;
}

void	KBTestVerifyTest::testVerify2 ()
{
	KBField		*srce	= getField  ("Field_0", "Field_0") ;
	srce->setValue	(0, KBValue("A.VALUE")) ;

	QStringList	args	;
	KBError		error	;

	args.append	(srce->getPath(false))	;
	args.append	(srce->getName())	;
	args.append	("0")			;
	args.append	(QString("%1:B.VALUE").arg(KB::ITString)) ;

	if (!m_macro->append ("VerifyValue", args, QString::null, error))
		CPPUNIT_FAIL (_STDSTR(error.getMessage())) ;

	KBTest::setTestMode(KBScriptIF::testSuite) ;
	KBScriptError *rc = m_macro->execute (m_docroot) ;
	KBTest::setTestMode(KBScriptIF::testOff  ) ;
	if (rc == 0)
		CPPUNIT_FAIL("Unexpected success") ;
}

void	KBTestVerifyTest::testVerify3 ()
{
	KBField		*srce	= getField  ("Field_0", "Field_0") ;
	srce->setValue	(0, KBValue("A.VALUE")) ;

	QStringList	args	;
	KBError		error	;

	args.append	(srce->getPath(false))	;
	args.append	(srce->getName())	;
	args.append	("1")			;
	args.append	(QString("%1:A.VALUE").arg(KB::ITString)) ;

	if (!m_macro->append ("VerifyValue", args, QString::null, error))
		CPPUNIT_FAIL (_STDSTR(error.getMessage())) ;

	KBTest::setTestMode(KBScriptIF::testSuite) ;
	KBScriptError *rc = m_macro->execute (m_docroot) ;
	KBTest::setTestMode(KBScriptIF::testOff  ) ;
	if (rc == 0)
		CPPUNIT_FAIL("Unexpected success") ;
}

void	KBTestVerifyTest::testVerify4 ()
{
	KBChoice	*srce	= getChoice  ("Combo_0", "Combo_0") ;

	QStringList	args	;
	KBError		error	;

	args.append	(srce->getPath(false))	;
	args.append	(srce->getName())	;
	args.append	("0")			;
	args.append	("|A|B|C|D")		;

	if (!m_macro->append ("VerifyChoices", args, QString::null, error))
		CPPUNIT_FAIL (_STDSTR(error.getMessage())) ;

	KBTest::setTestMode(KBScriptIF::testSuite) ;
	KBScriptError *rc = m_macro->execute (m_docroot) ;
	KBTest::setTestMode(KBScriptIF::testOff  ) ;
	if (rc != 0)
		CPPUNIT_FAIL(rc->error().getMessage() + ": " + rc->error().getDetails()) ;
}

void	KBTestVerifyTest::testVerify5 ()
{
	KBChoice	*srce	= getChoice  ("Combo_0", "Combo_0") ;

	QStringList	args	;
	KBError		error	;

	args.append	(srce->getPath(false))	;
	args.append	(srce->getName())	;
	args.append	("0")			;
	args.append	("|A|B|C|X")		;

	if (!m_macro->append ("VerifyChoices", args, QString::null, error))
		CPPUNIT_FAIL (_STDSTR(error.getMessage())) ;

	KBTest::setTestMode(KBScriptIF::testSuite) ;
	KBScriptError *rc = m_macro->execute (m_docroot) ;
	KBTest::setTestMode(KBScriptIF::testOff  ) ;
	if (rc == 0)
		CPPUNIT_FAIL("Unexpected success") ;
}

void	KBTestVerifyTest::testVerify6 ()
{
	KBChoice	*srce	= getChoice  ("Combo_0", "Combo_0") ;

	QStringList	args	;
	KBError		error	;

	args.append	(srce->getPath(false))	;
	args.append	(srce->getName())	;
	args.append	("1")			;
	args.append	("|A|B|C|D")		;

	if (!m_macro->append ("VerifyChoices", args, QString::null, error))
		CPPUNIT_FAIL (_STDSTR(error.getMessage())) ;

	KBTest::setTestMode(KBScriptIF::testSuite) ;
	KBScriptError *rc = m_macro->execute (m_docroot) ;
	KBTest::setTestMode(KBScriptIF::testOff  ) ;
	if (rc == 0)
		CPPUNIT_FAIL("Unexpected success") ;
}

void	KBTestVerifyTest::testVerify7 ()
{
	KBLink		*srce	= getLink  ("Link_0", "Link_0") ;

	QStringList	args	;
	KBError		error	;

	args.append	(srce->getPath(false))	;
	args.append	(srce->getName())	;
	args.append	("0")			;
	args.append	("|1|2|3|4|5|6|7|8|9")	;

	if (!m_macro->append ("VerifyChoices", args, QString::null, error))
		CPPUNIT_FAIL (_STDSTR(error.getMessage())) ;

	KBTest::setTestMode(KBScriptIF::testSuite) ;
	KBScriptError *rc = m_macro->execute (m_docroot) ;
	KBTest::setTestMode(KBScriptIF::testOff  ) ;
	if (rc != 0)
		CPPUNIT_FAIL(rc->error().getMessage() + ": " + rc->error().getDetails()) ;
}

void	KBTestVerifyTest::testVerify8 ()
{
	KBLink		*srce	= getLink  ("Link_0", "Link_0") ;

	QStringList	args	;
	KBError		error	;

	args.append	(srce->getPath(false))	;
	args.append	(srce->getName())	;
	args.append	("0")			;
	args.append	("|1|2|3|4|5|6|7|8|X")	;

	if (!m_macro->append ("VerifyChoices", args, QString::null, error))
		CPPUNIT_FAIL (_STDSTR(error.getMessage())) ;

	KBTest::setTestMode(KBScriptIF::testSuite) ;
	KBScriptError *rc = m_macro->execute (m_docroot) ;
	KBTest::setTestMode(KBScriptIF::testOff  ) ;
	if (rc == 0)
		CPPUNIT_FAIL("Unexpected success") ;
}

void	KBTestVerifyTest::testVerify9 ()
{
	KBLink		*srce	= getLink  ("Link_0", "Link_0") ;

	QStringList	args	;
	KBError		error	;

	args.append	(srce->getPath(false))	;
	args.append	(srce->getName())	;
	args.append	("1")			;
	args.append	("|1|2|3|4|5|6|7|8|9")	;

	if (!m_macro->append ("VerifyChoices", args, QString::null, error))
		CPPUNIT_FAIL (_STDSTR(error.getMessage())) ;

	KBTest::setTestMode(KBScriptIF::testSuite) ;
	KBScriptError *rc = m_macro->execute (m_docroot) ;
	KBTest::setTestMode(KBScriptIF::testOff  ) ;
	if (rc == 0)
		CPPUNIT_FAIL("Unexpected success") ;
}

void	KBTestVerifyTest::testVerify10 ()
{
	KBButton	*srce	= getButton  ("Button", "Button") ;

	QStringList	args	;
	KBError		error	;

	args.append	(srce->getPath(false))	;
	args.append	(srce->getName())	;
	args.append	("0")			;
	args.append	("ButtonTextNot")	;

	if (!m_macro->append ("VerifyText", args, QString::null, error))
		CPPUNIT_FAIL (_STDSTR(error.getMessage())) ;

	KBTest::setTestMode(KBScriptIF::testSuite) ;
	KBScriptError *rc = m_macro->execute (m_docroot) ;
	KBTest::setTestMode(KBScriptIF::testOff  ) ;
	if (rc == 0)
		CPPUNIT_FAIL("Unexpected success") ;
}

void	KBTestVerifyTest::testVerify11 ()
{
	KBButton	*srce	= getButton  ("Button", "Button") ;

	QStringList	args	;
	KBError		error	;

	args.append	(srce->getPath(false))	;
	args.append	(srce->getName())	;
	args.append	("0")			;
	args.append	("ButtonText")	;

	if (!m_macro->append ("VerifyText", args, QString::null, error))
		CPPUNIT_FAIL (_STDSTR(error.getMessage())) ;

	KBTest::setTestMode(KBScriptIF::testSuite) ;
	KBScriptError *rc = m_macro->execute (m_docroot) ;
	KBTest::setTestMode(KBScriptIF::testOff  ) ;
	if (rc != 0)
		CPPUNIT_FAIL(rc->error().getMessage() + ": " + rc->error().getDetails()) ;
}

void	KBTestVerifyTest::testVerify12 ()
{
	KBLabel	*srce	= getLabel  ("Label", "Label") ;

	QStringList	args	;
	KBError		error	;

	args.append	(srce->getPath(false))	;
	args.append	(srce->getName())	;
	args.append	("0")			;
	args.append	("LabelTextNot")	;

	if (!m_macro->append ("VerifyText", args, QString::null, error))
		CPPUNIT_FAIL (_STDSTR(error.getMessage())) ;

	KBTest::setTestMode(KBScriptIF::testSuite) ;
	KBScriptError *rc = m_macro->execute (m_docroot) ;
	KBTest::setTestMode(KBScriptIF::testOff  ) ;
	if (rc == 0)
		CPPUNIT_FAIL("Unexpected success") ;
}

void	KBTestVerifyTest::testVerify13 ()
{
	KBLabel	*srce	= getLabel  ("Label", "Label") ;

	QStringList	args	;
	KBError		error	;

	args.append	(srce->getPath(false))	;
	args.append	(srce->getName())	;
	args.append	("0")			;
	args.append	("LabelText")	;

	if (!m_macro->append ("VerifyText", args, QString::null, error))
		CPPUNIT_FAIL (_STDSTR(error.getMessage())) ;

	KBTest::setTestMode(KBScriptIF::testSuite) ;
	KBScriptError *rc = m_macro->execute (m_docroot) ;
	KBTest::setTestMode(KBScriptIF::testOff  ) ;
	if (rc != 0)
		CPPUNIT_FAIL(rc->error().getMessage() + ": " + rc->error().getDetails()) ;
}

CppUnit::Test
	*KBTestVerifyTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("TestVerify") ;

	if (rekallTestSkip ("TestVerify")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;

	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBTestVerifyTest> ("noDisplay", &KBTestVerifyTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBTestVerifyTest> ("withGUI",   &KBTestVerifyTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBTestVerifyTest>
			(	"testVerify1",
				&KBTestVerifyTest::testVerify1
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTestVerifyTest>
			(	"testVerify2",
				&KBTestVerifyTest::testVerify2
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTestVerifyTest>
			(	"testVerify3",
				&KBTestVerifyTest::testVerify3
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTestVerifyTest>
			(	"testVerify4",
				&KBTestVerifyTest::testVerify4
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTestVerifyTest>
			(	"testVerify5",
				&KBTestVerifyTest::testVerify5
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTestVerifyTest>
			(	"testVerify6",
				&KBTestVerifyTest::testVerify6
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTestVerifyTest>
			(	"testVerify7",
				&KBTestVerifyTest::testVerify7
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTestVerifyTest>
			(	"testVerify8",
				&KBTestVerifyTest::testVerify8
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTestVerifyTest>
			(	"testVerify9",
				&KBTestVerifyTest::testVerify9
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTestVerifyTest>
			(	"testVerify10",
				&KBTestVerifyTest::testVerify10
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTestVerifyTest>
			(	"testVerify11",
				&KBTestVerifyTest::testVerify11
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTestVerifyTest>
			(	"testVerify12",
				&KBTestVerifyTest::testVerify12
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTestVerifyTest>
			(	"testVerify13",
				&KBTestVerifyTest::testVerify13
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_REPORTS *testsVerifyTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBTestVerifyTest::suite (__settings) ;
}
